function [A,B,C,D,states,x0,x0sw,rlsw,u,x,y,freq,Asw,Bsw,Csw,Dsw,Hlin]=...
		 circ2ss(rlc,switches,source,line_dist,yout,y_type,unit,...
		 blocs,srcstr,BrancheREF,power2sys_flag,fid_outfile,...
		 freq_sys,option,vary_name,vary_val)
%
% CIRC2SS produces the continuous state-space representation of a linear 
%         electrical circuit. The state variables are inductor currents and
%         capacitor voltages.
% 
%         CIRC2SS can be called in two different ways:
%
%     1)  CIRC2SS is called automatically by the POWER2SYS function producing
%         the topology and state-space representation of an electrical circuit
%         previously drawn in  a SIMULINK window
%         from the blocks of the ELECTRICAL LIBRARY.
%     2)  Directly from the Matlab window
%     
%  Input arguments
%  ===============
%       The number of arguments may be 7, 12, 13, 14 or 16.
%       The first 7 arguments must be specified. Arguments 8 to 16 are optional.
%
%       rlc : Branch matrix (nbr,6) or (nbr,7) specifying the network topology
%             and R L C values; see format below
%  switches : switch matrix (nsw,7) ; see format  below
%    source : source matrix (nsrc,5) or (nsrc,6) or nsrc(7) ; see format  below
%  line_dist: distributed parameter line matrix (nline,7) ; see format  below
%      yout : string matrix (nb_output,ncar) of output expressions
%             see format below
%    y_type : vector indicating output types
%             (0=voltage output ; 1=current output)
%      unit : units used to specify R L and C values in the rlc matrix 'rlc
%             unit= 'OHM' R L C values specied in ohms at fundamental frequeny
%                    specified by freq_sys.
%             unit= 'OMU' R L C values specied in ohms millihenries (mH) and 
%                    microfarads(uF)
%
%       The following  9 arguments are optional:
%
%            Specify empty variables [] for the next 3 arguments 
%            if CIRC2SS is not called by POWER2SYS :
%     blocs : string matrix containing  names of blocks  
%    srcstr : string matrix containing names of source blocks
%             starting with  I_ ou V_ prefix
% BrancheREF: Integer matrix= cross reference list between POWER2SYS topology 
%             and rlc matrix
%
%power2sys_flag: Integer =0: CIRC2SS is called by ETAT or from the Matlab window.
%                         Informations on circuit (topology, modes, outputs ...)
%                         are displayed during execution.
%                      1: if CIRC2SS is called by POWER2SYS
%                         Only version number, number of states, inputs
%                         and outputs are displayed.
%                      2: Silent mode. 
%
%fid_outfile: File identifier of output file containing detailed messages 
%
%  freq_sys : Fundamental frequency (Hz) considered for specification of
%             L and C reactances if unit = 'OHM';  default value = 60 Hz
%
%    option : Integer vector specifying options (5);default values= [0 0 0 0 0]
%                option(1)= reference node number used for pi transmission lines
%                          -1 Node is not specified (The user wil be prompted to
%                           specify a node number).
%                option(2)= used to specify normalization of A B C D matrices
%                           in order to use the per-unit system.
%                           This option is not available from POWER2SYS
%                           0 : no normalization ;
%                           1 : Use per-unit system ;
%                               (Base voltage and base power are specified
%                               by option(3) and opotion(4))
%                          -1 : The user will be prompted to specify 
%                               a base voltage and a base power.
%                option(3)= base voltage (kVrms Phase-Phase)
%                option(4)= base power (MVA 3 phase)
%                option(5)= 0 used only for debugging; (normal value=0)
%                           1 allows to display detailed calculation of A B C D
%                            used only for debugging purposes. 
%
% vary_name : String matrix (nvary,n) containing the variable names 
%             used in output expressions          
%  vary_val : Vector containing the values of the variable names specified
%             in vary_name.      
%
%  Output arguments
%  ================
%   A,B,C,D :  State-space matrices of the linear circuit with all switches open
%              A(nstates,nstates) B(nstates,ninput)
%              C(noutput,nstates) D(noutput,ninput)
%    states :  String matrix (n,nstates) containing the names of the
%              state variables. Each string has the following format:
%                   Inductor  currents: Il_bxx_nzz1_zz2 
%                   Capacitor voltages: Uc_bxx_nzz2_zz2 
%                   where : xx = branch number 
%                           zz1= 1st node number of the branch
%                           zz2= 2nd node number of the branch
%        x0 :  Column vector of initial values of state variables
%              considering the status (open or closed) of switches
%      x0sw :  Vector of initial values of switch currents
%      rlsw :  Matrix (nsw,2) containing the R and L values of series switch
%              impedances in Ohm and Henry,
%     u,x,y :  Matrices  u(ninput,nfreq) x(nstates,nfreq) y(noutput,nfreq)
%              containing the steady-state complex values of inputs, states
%              and outputs . Each column correspond to a different source
%              frequency as specified by the next argument (freq).
%      freq :  Column vector containing the (nfreq) source frequencies 
%              ordered by increasing frequencies.
%    Asw,Bsw : State-space matrices of the circuit including the closed switches
%    Csw,Dsw   (one extra state per closed switch)
%       Hlin : Complex transfer impedance matrix (noutput,ninput) of the linear
%              system corresponding to the first non-zero frequency
%              of the FREQ vector.
%                   
%
% Format of the RLC input matrix
% ==============================
%         Two formats are allowed:
%         6 columns :  No branch numbering; 
%                      Branch numbers correspond to the RLC line numbers(1-nbr)
%         7 columns :  Branch numbering is imposed by the user
%
%         Each line of the RLC matrix must be specified according to the
%         following format:
%
%                 (1)   (2)  (3)  (4)  (5)  (6)   [7]
%               node1 node2  TYPE  R    Xl  Xc   [Nobr] for a RLC or line branch
%          or   node1 node2  TYPE  R    L   C    [Nobr]
%               node1 node2  TYPE  R    Xl  U    [Nobr] for a transformer branch
%          or   node1 node2  TYPE  R    L   U    [Nobr]
%
%               node1 : Node number 1 of the branch
%               node2 : Node number 2 of the branch
%                       Note:
%                       Node numbers must be positive or zero.
%                       Decimal node numbers are allowed.                       
%                TYPE : Code indicating the type of connexion of R L C elements
%                       (0-3) or the transmission line length (negative value).
%                       0 : series R L C
%                       1 : parallel R L C
%                       2 : transformer winding
%                       3 : coupled (mutual) winding
%                           For a mutual or a tranformer having N windings,
%                           N+1 consecutive lines must be specified:
%                          -N lines TYPE=2 or TYPE=3; (one line per winding)
%                           Each line specifies R/L/U or R/Xl/Xc
%                           where : R/L R/Xl = winding resistance and leakage
%                                reactance or winding resist. and self reactance
%                                   U = Nominal voltage of transformer winding 
%                                       (specify 0 for TYPE 3)
%                          - one line TYPE=1 for the magnetizing branch of a
%                            transformer (parallel R/L or R/XLl)
%                            or oneline TYPE=0 for a mutual impedance
%                            (series R/L or R/XL)
% 
%                       Negative value:  transmission line length (km)
%                       ---------------
%                        For a transmission line, the R,L,C or R/Xl/Xc values
%                        must be specified in (ohm/km) or (ohm,mH,uF /km)
%
%                R    : Branch resistance (ohms or pu)
%                Xl   : Branch inductive reactance  (ohms or pu at FREQ_SYS)
%                       or transformer winding leakage reactance (ohms)
%                 L   : Branch inductance (mH)
%                Xc   : Branch capacitive reactance (ohms or pu at FREQ_SYS)
%                       (The negative sign of Xc is optional)
%                 C   : Capacitance (uF)
%                 U   : Nominal voltage of transformer winding
%                       Same units Volts or kV must be used for each winding.
%                       For a mutual (TYPE=3), this value must be set to zero.
%
%                       Zero values
%                       ----------- 
%                       For R, L or Xl, C or Xc, in a series or parallel branch
%                       indicates that the corresponding element does not exist.
%
%                       Restrictions for transformers and mutuals
%                       -----------------------------------------
%                     - Nul values are not allowed for winding resistances
%                       of transformers and mutuals. Specify a very low value
%                      (e.g. 1e-6 pu based on rated voltage and power)
%                       to simulate a quasi ideal transformer.
%                       The leakage reactances can be set to zero.
%                     - The resistive part of the magnetizing branch of a   
%                       transformer must have a finite value. Specify a very
%                       high value (low losses, e.g 1e4 pu or 0.01% based on 
%                       rating) to simulate a quasi ideal transformer.
%                       The inductive part of the magnetizing branch can be 
%                       set to infinite (No reactive losses; specify Xm=0)
%                     - A nul value is not allowed for the mutual inductance 
%                       of coupled windings. The resistive part of 
%                       series mutual branch can set to zero. 
%
%    Format of the SOURCE input matrix
%    =================================
%         Three formats are allowed:
%         5 columns :  All sources are generating the same frequency specified
%                      by FREQ_SYS
%         6 columns : The frequency of each source is specified in col. 6
%         7 columns : This column is used to specify the type
%                     of non linear element modelled by the current source
%
%         Each line of the SOURCE matrix must be specified according to the
%         following format:
%
%                 (1)   (2)  (3)    (4)   (5)    [6]    [7]
%               node1 node2  TYPE  AMP   PHASE  [FREQ]  [MODEL]
%
%         node1,node2 :Node numbers corresponding to the source terminals
%                      Polarity conventions:
%                        Voltage source : Node 1 is the positive terminal.
%                        Current source : Current is flowing out of node 1. 
%                TYPE : Code indicating type of source
%                       0= voltage source
%                       1= current source
%                 AMP : amplitude of the AC or DC voltage or current (V,A or pu)
%               PHASE : phase of the AC voltage or current (degres)
%                FREQ : Frequency (Hz) of the generated voltage or current.
%                       For a DC voltage or current source specify
%                       PHASE=0 and FREQ=0. AMP can be set to a negative value. 
%                       Generated signal:
%                                          AMP*sin(2*pi*FREQ*t+PHASE) for AC
%                                          AMP for DC
%               MODEL : integer specifying the type of non linear element
%                       modelled by the current source (saturable inductance,
%                       thyristor, switch ...) ;  Used by POWER2SYS only.
%
% 
%    Format of the SWITCHES input matrix
%    =================================
%    Switches are non linear elements simulating mechanical or electronic
%    devices such as circuit breakers diodes or thyristors. As other non linear
%    elements, they are simulated by current sources driven by the voltage
%    appearing across their terminals. Therefore, they cannot have a perfectly
%    zero impedance. They are simulated as ideal switches in series with a
%    series R-L circuit.
%    The switch parameters must be specified  in a line of the SWITCHES matrix 
%    in 7 different columns according to the following format:
%
%                 (1)   (2)  (3)    (4)   (5)     (6)     (7)
%              node1  node2 STATUS   R    L/Xl    No_I    No_U
%
%        node1, node2 : Node numbers cooresponding to the switch terminals
%              STATUS : Code indicating the initial status of the switch at t=0
%                       0= open
%                       1= closed
%                  R  : Resistance of the switch when closed (ohms or pu)
%                L/Xl : Inductance of the switch when closed (mH)
%                       or inductive reactance (ohms or pu at FREQ_SYS)
%                       Note: For these last two fields, the same units as
%                             specified for the RLC matrix (UNIT) must be used.
%                             Nul inductance values arec not allowed. However,
%                             the resistance value can be set to zero.
%
%                       The next two fields specify the current input number
%                       and the voltage output number to be used for 
%                       interconnecting the switch model to state-space block
%
%                no_I : Current input number coming from the output of
%                       the switch model.
%                no_U : Voltage output number driving the input of the 
%                       switch model.
%
%    Format of the LINE_DIST matrix
%    ==============================
%           Each row of the LINE_DIST matrix is used to specify
%           a distributed parameter transmission line.
%           The number of columns of LINE_DIST depends on the number of phases
%           of the transmission line.
%           For a NPHASE line, the first (4+3*NPHASE+NPHASE^2) columns are used.
%           For example, for a three-phase line, 22 columns are used.
%
%                 (1)   (2)   (3)   (4)  (5...                 ...)
%              NPHASE  No_I  No_U  LONG  L/Xl   Zc  Rm   SPEED Ti
%
%           NPHASE = Number of phases of the transmission line
%            No_I  = Input number in the SOURCE matrix corresponding to the
%                    first currrent source Is_1 of the line model.
%                    Each line model uses 2*NPHASE current sources specified
%                    in the SOURCE matrix as follows:
%                    Is_1 Is_2 ...Is_nphase for the sending end followed by
%                    Ir_1 Ir_2 ...Ir_nphase for the receiving end.
%            No_U  = Output number of the state-space corresponding to the
%                    first voltage output Vs_1 feeding the line model.
%                    Each line model uses 2*NPHASE voltage outputs
%                    in the SOURCE matrix as follows:
%                    Vs_1 Vs_2 ...Vs_NPHASE for the sending end followed by
%                    Vr_1 Vr_2 ...Vr_NPHASE for the sending end followed by
%           LONG   = Length of the line (km)
%             Zc   = Vector of the NPHASE modal caracteristic impedances (ohms)
%             Rm   = Vector of the NPHASE modal series linear resistances
%                    (ohms/km)
%           SPEED  = Vector of the NPHASE modal propagation speeds (km/s)
%             Ti   = Transformation matrix from  mode to phase currents
%                    such that Iphase=Ti.Imode
%                    The  NPHASE*NPHASE matrix is given in vector format
%                    [col_1,col_2, ... col_NPHASE]
%
% Format of the YOUT matrix
% =========================
%   The desired outputs are specified by a string matrix (YOUT). Each line of
%   the YOUT natrix must be an algebraic expressioncontaining any linear
%   combination of states and state derivatives specifdied according tho the
%   following format:
%                 
%                    Uc_bn  : Capacitor voltage of branch #n
%                    Il_bn  : Inductor current of branch #n
%          dUc_bn,  dIl_bn  : Derivative of Uc_bn or Il_bn
%                   Un, In  : Source voltage or current specified by line #n
%                             of the SOURCE matrix
%                 U_nx1_x2  : Voltage between node x1 and node x2
%                     I_bn  : Current in branch #n
%                             For a parallel RLC branch, I_bn corresponds
%                              to the total current IR+IL+IC
%                  I_bn_nx  : Current flowing into node x of a transmission 
%                             line specified by line #n of the RLC matrix.
%                             (This current includes the series inductive branch
%                              current and the capacitive shunt current).
%
%   Each output expression is built from the above voltage and current variable
%   names, constants, variable names, parenthesis and  operators (+-*/^) in
%   order to form a valid MATLAB expression.  
%
%                  Example :  YOUT=[ '10*I_b1 + Uc_b3 -L2*dIl_b2',
%                                    'U_n10_20                  '];
%
%   If variable names are used (ex. L2 in the above example), their names and
%   values must be specified by the two input arguments VARY_NAME and VARY_VAL.
%
%        Sign conventions for voltages and currents
%        ------------------------------------------
%   I_bn Il_bn, In = Branch cuurent, inductor current of branch
%                    or current of source #n 
%                    is oriented from node1 to node2.
%        I_bn_nx   = Current at one end (node x) of a transmiision line  
%                    If x= node1, the current is entering the line
%                    If x= node2, the current is going out of the line
%        Uc_bn, Un = Voltage across capacitor or voltage source
%                    = Unode1 - Unode2
%        U_nx1_x2  = Voltage between nodes x1 and x2 = Ux1 - Ux2
%
%
%   See also  POWER2SYS

%   G. Sybille (IREQ) April 1995, Jan-31-96 
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.4 $

nargs = nargin;
if nargs == 7
    blocs=[];
    srcstr=[];
    BrancheREF=[];
    power2sys_flag=0;
    freq_sys=60;
    option=[0 0 0 0 0];
    vary_name=[]; vary_val=[]; nvary=0;
	fid_outfile=0;
elseif nargs ==12,
    freq_sys=60;
    option=[0 0 0 0 0];
    vary_name=[]; vary_val=[]; nvary=0;
elseif nargs ==13,
    option=[0 0 0 0 0];
    vary_name=[]; vary_val=[]; nvary=0;
elseif nargs==14
    vary_name=[]; vary_val=[]; nvary=0;
 
elseif nargs ==16,
    [nvary,n]=size(vary_name);
    if length(vary_val)~=nvary,
        error( sprintf(...
        ['The length of vector ''vary_val'' (%g) does not correspond',...
         ' to the number of variables declared in ''vary_name'' (%g)'],...
         size(vary_name),nvary));
    end
else
    error( sprintf...
	('Number of input arguments (%g) must be 7, 12, 13, 14 or 16',nargs));
end
%
% Indicate version #
% ------------------
if power2sys_flag<=1,
	version='4.5';
	fprintf('\nETAT Version %s\n',version);
	if fid_outfile~=1,
		fprintf(fid_outfile,'\n\nCIRC2SS Version %s\n',version);
	end
end
%
% Read options
% ------------
omega=freq_sys*2*pi;
neu_ref=option(1);
ubase=option(3);
pbase=option(4);
if option(5)==1, debug_etat=1; else, debug_etat=0;   end
%
[nb_sortie,n]=size(yout);
%
[nbr_src,n]=size(source);
%
% Validation and processing of input matrices 'rlc' 'switches' and 'source'
% ======================================================================
%
[rlc,liste_neu,ind_neu,ind_neusrc,ind_neusw, mat_tr,unit,rlccar]=etapar(...
	rlc,switches,source,nb_sortie,neu_ref,unit,power2sys_flag,fid_outfile);
%
fprintf(fid_outfile,'Number of outputs %g\n',nb_sortie);
%
%
%Compute Matrices [A] and [B]; 
%=============================
      [A,B,Cvn,Dvn,states,var_branche,nbvar1,rlcm,rlc1,...
       L_combi,C_combi,Il_relat,Uc_relat,nb_ldep,nb_cdep,...
	   liste_neu,liste_neured,Ti,mat_tr,c_ligne] = etaab(...
       rlc,source,liste_neu,ind_neu,ind_neusrc,mat_tr,unit,freq_sys,...
	   nb_sortie,debug_etat,blocs,srcstr,BrancheREF,power2sys_flag,fid_outfile);
%
%
% Display oscillatory modes (Frequency; damping factor)
% =====================================================
if power2sys_flag==0,
	valp=eig(A);
	[n,i]=sort(imag(valp)); % Sort eigenvalues by increasing frequency
	valp=valp(i);
	fprintf('\nOscillatory modes and damping factors:\n');
	for i=1:nbvar1,
		if imag(valp(i))>0,
		fmode=imag(valp(i))/2/pi;           % Frequency 
		zeta=-real(valp(i))/abs(valp(i));   % Damping factor
		fprintf(sprintf('F=%gHz zeta=%g\n',fmode,zeta));
		end
	end
end
%
[nbvar,n]=size(states);
%
C=zeros(nb_sortie,nbvar1);
D=zeros(nb_sortie,nbr_src);
vec_c0=zeros(1,nbvar1);
vec_d0=zeros(1,nbr_src);
%
% Compute Matrices [C] and [D]
% ============================
% Evaluate output expressions
% ---------------------------
for ivary=1:nvary
	eval([vary_name(ivary,:),'=vary_val(ivary);'])
end
%
fprintf(fid_outfile,'\n');
for i_sortie=1:nb_sortie,
	chaine=yout(i_sortie,:);
		% Voltage output
		% --------------
	if y_type(i_sortie)==0, 
		sortie=['y_u',int2str(i_sortie)];
		% Current output
		% --------------
	elseif y_type(i_sortie)==1,
		sortie=['y_i',int2str(i_sortie)];
	end
%
% Preprocessing of output expression for line i_sortie of [C] and [D]
% -------------------------------------------------------------------
%
fprintf(fid_outfile,[sortie,'=',chaine,'\n']); 
%
 [expr_c,expr_d,vec_c,vec_d,vec_u] = etacd(chaine,sortie,rlcm,rlc1,...
      source,states,var_branche,nbvar1,liste_neu,Ti,L_combi,C_combi,...
      Il_relat,Uc_relat,nb_ldep,nb_cdep,c_ligne,debug_etat);
%
% Compute line i_sortie of [C] and [D]
% ------------------------------------
	if length(expr_c)>0 & nbvar1>0, 
		str=['C(',int2str(i_sortie),',:)=',expr_c,';']; 
		eval(str);
	end
	if length(expr_d)>0, 
		str=['D(',int2str(i_sortie),',:)=',expr_d,';']; 
		eval(str);
	end
end
%
%
% Append '*' at the end of dependant state-variable names
% -------------------------------------------------------
if nbvar1<nbvar,
	for i=nbvar1+1:nbvar,
		n=find(states(i,:)==' ');
		states(i,n(1))='*';
	end
end
%
%  Normalise matrices A B C  for per-unit  system
%  ===============================================
%
[ntr,n]=size(mat_tr);
%
if option(2)==1 & ubase>0 & pbase>0,
	fprintf('\nBase voltage : %g kVrms ph-ph\n',ubase);
	fprintf('Base power : %g MVA 3ph\n',pbase);
elseif option(2)<0 | (option(2)==1 & (ubase<=0 | pbase<=0)),
	norm_pu=input(...
	'Do you want to use the per-unit system and normalize A B C D ? (y/n):','s');
	if norm_pu(1)=='y' | norm_pu(1)=='o',
		ubase=input('Base voltage (kVrms ph-ph): ');
		pbase=input('Base power (MVA 3ph): ');
		option(2)=1;
	end
end
%
if option(2)==1,
	if ntr>0,
		error(...
		'Impossible to normalize [A] [B] [C] [D] with transformers');
	end
	zbase=ubase^2/pbase;
	for i=1:nbvar1,
		% Matrix [A]
		for k=1:nbvar1,
			if states(i,1)=='U' & states(k,1)=='I',...
 			A(i,k)=A(i,k)/zbase;end
			if states(i,1)=='I' & states(k,1)=='U',...
			A(i,k)=A(i,k)*zbase;end
		end
		% Matrix [B]
		for k=1:nbr_src,
			if states(i,1)=='U' & source(k,3)==1,...
			B(i,k)=B(i,k)/zbase;end
			if states(i,1)=='I' & source(k,3)==0,...
			B(i,k)=B(i,k)*zbase;end
		end
	end
	for i=1:nb_sortie,
		% Matrix [C]
		for k=1:nbvar1,
			if y_type(i)==0 & states(k,1)=='I',...
			C(i,k)=C(i,k)/zbase;end
			if y_type(i)==1 & states(k,1)=='U',...
			C(i,k)=C(i,k)*zbase;end
		end
		% Matrix [D]
        for k=1:nbr_src, 
			if y_type(i)==0 & source(k,3)==1,...
			D(i,k)=D(i,k)/zbase; end 
			if y_type(i)==1 & source(k,3)==0,...
			D(i,k)=D(i,k)*zbase;end 
        end 
	end
end
%
%
%  Processing of switches
%  ======================
[nsw,n]=size(switches);
%
%
% Unit Conversions
% ----------------
rlsw=zeros(nsw,2); % Matrix containing R L values in Ohms , Henrys
%
for ib=1:nsw,
	% Convert to Ohms Henrys 
	if all(unit=='OMU'),
		R=switches(ib,4);
		L=switches(ib,5)/1000;
	elseif all(unit=='OHM'),
		R=switches(ib,4);
		L=switches(ib,5)/omega;
	elseif all(unit=='P.U')
		error ('Impossible to use the per-unit system');
	end
	%
	rlsw(ib,1)=R; rlsw(ib,2)=L;
end
%
% Compute steady-state
% ====================
[u,x,y,x0,x0sw,Asw,Bsw,Csw,Dsw,freq,Hlin]=...
 etass(A,B,C,D,switches,source,rlsw,unit,freq_sys,line_dist);
%
% Display outputs in polar form
% =============================
if power2sys_flag==0,
	for ifreq=1:length(freq),
		fprintf('\nSteady state outputs @ F=%g Hz :\n',...
		freq(ifreq));
		for i=1:nb_sortie
			if y_type(i)==1,
				str='y_i';
				if option(2)==1,
				unit=['pu/',num2str(pbase),'MVA'];
				else, unit='Amperes'; end
			else,
				str='y_u';
				if option(2)==1,
				unit='pu'; else, unit='Volts';end
			end
			if freq(ifreq)>0,
				fprintf([str,sprintf(...
				['%g= %.4g',unit,' < %.4g deg.\n'],...
				i,abs(y(i,ifreq)),angle(y(i,ifreq))*180/pi)]);
			else
				fprintf([str,sprintf(...
				['%g= %.4g',unit,'\n'],...
				i,real(y(i,ifreq)))]);
			end
		end
	end
end
