function [A,B,C,D,x0,state_var,inputs,outputs,uss,xss,yss,freqyss,Hlin, ...
	rlc,source,switchns,nb_true_sensor,nb_models,models,y_type,yout,true_source,inoutinfo] ...
	= power2sys(simuwin,options);
%POWER2SYS Simulink graphical interface analyser for Electrical Toolbox
%	[A,B,C,D,X0,STATE_VAR,SOURCES,OUTPUTS,USS,XSS,YSS,RLC,SOURCE,SWITCH, ...
%	NB_TRUE_SENSOR,NB_MODELS,MODELS,Y_TYPE,YOUT]=POWER2SYS(SIMUWIN,OPTION);
%
% 	PURPOSE
%
% 	Analyse electrical network built with powerlib library 
%
% 	SYNOPSIS
%
% 	POWER2SYS;
% 	[A,B,C,D,X0,STATE_VAR,SOURCES,OUTPUTS,USS,XSS,YSS,FREQYSS,HLIN]= ...
%							POWER2SYS(SIMWIN)
% 	[A,B,C,D,X0,STATE_VAR,SOURCES,OUTPUTS,USS,XSS,YSS,FREQYSS,HLIN]]= ...
%	                                                POWER2SYS(SIMWIN,OPTIONS)
%
% 	DESCRIPTION
%
% 	POWER2SYS is used to compute the equivalent State-Space model of an
%	electrical network built with `Electrical blockset' blocks library. 
%	Netsim extracts the linear part of the simuwin system and build the 
%	corresponding state-space model.Non-linear elements are simulated by 
%	current sources (state-space model input) driven by the voltage across 
%	the non-linear element. After POWER2SYS has been called, the internal 
%	structure of the system becomes functionnal, and the simulation of the
%	electrical network can be made normally by using the 'Simulation/Start'
%	menu command.
%
% 	[ A,B,C,D,X0,STATE_VAR,SOURCES,OUTPUTS,USS,XSS,YSS,FREQYSS,Hlin ] =
%	 POWER2SYS(SIMWIN) 
% 	computes the equivalent state-space model of the SIMULINK window named
%	SIMUWIN. A,B,C,D are the standard matrices of the state-space system
% 	described by the equations x'=Ax+Bu and y=Cx+Du. X0 is a vector
% 	containing the initial conditions of the state-space variables listed
% 	in STATE_VAR.
% 
% 	STATE_VAR is the list of the names of the state-space
% 	variables (vector x). Each line of STATE_VAR begins with a prefix,
% 	`Uc_' for capacitor voltages or`Il_' for inductance currents, followed
% 	by the name of the block in which the element (C or L) is found.
%	Inductance currents senses and capacitor voltages are defined as taken 
%	from the input of the block to the output of the block. 
%	A suffix is added to the line for blocks made of more than 2 inductances
%	or capacitors: for exemple, the `Linear transformer' block will produce 
%	STATE_VAR lines for its leakage inductances with the suffixe `coil:x' 
%	where x is the winding number of the transformer. 
%
% 	INPUTS is the names list of the names of the inputs of the system 
%	(vector u). Each line of INPUTS begins with a prefix, `U_' for
% 	voltage sources or `I_' for current sources, followed by the name of the
% 	source block. Currents sources are defined as taken from the input of 
%	the block to the output of the block while voltages are defined as taken
%	from the output of the block to the input of the block. A suffix may end
%	a line of INPUTS for multi-source block like, for exemple, the 
%	`Simplified alternator' block that produce two inputs.
%
% 	OUTPUTS is the list of the outputs of the state-space system (vector y).
%	Each line of OUTPUTS begins with a prefix, `U_' for voltage ouputs or 
%	`I_' for current outputs , followed by the name of the block which 
%	produce the output.
%
%	USS contains the steady-state complex values of the sources of 
%	the system. 
%
%	XSS contains the steady-state complex values of the state-space
%	variables of the system.
%
%	YSS contains the steady-state complex values of the outputs of 
%	the system. 
%
%	FREQYSS is the column vector containing the sources (inputs) frequencies
%	ordered by increasing values.
%
%	HLIN is the complex transfer impedance matrix of the linear system 
%	corresponding to the first non-zero frequency of the FREQYSS vector.
%
%	power2sys(simwin,options) will run power2sys with special functional flags. 
%	The string variable options may contains one or many of the following 
%	flag caracters in any order:
%	`w':	power2sys pop-up warning conditions if any.
%	`o':	power2sys produce a .net file containing processing information.
%	`s':	enable auto-save of Simulink window simwin after analysis.
% 	The `s' flag is used to automatically save the state-space system
% 	constructed by power2sys in the Simulink window.

%   C. Dufour, Patrice Brunelle (TEQSIM) 28-10-96, 15-09-97
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.4 $

GUIflag  = [];
TESTflag = [];
AUTOEXECflag=[];
nsflag   = [0 0 0 0];

if nargin==0
  	str=['POWER2SYS error: Missing input argument! Usage: >>power2sys(sys)'];
	error(str);
end
if exist(simuwin)==0
	str=['POWER2SYS error: Cannot find simulink model: ' simuwin '.mdl' ];
	error(str);
end
if isempty(find_system('name',simuwin))
	open_system(simuwin);
end
if nargin==2
	alloc=[];
	bad=[];
	loc=findstr(options,'w'); % warning flag
	nsflag(1)=(~isempty(loc));
	alloc=[alloc loc];
	loc=findstr(options,'o'); % output file flag
 	nsflag(2)=(~isempty(loc));
	alloc=[alloc loc];
	loc=findstr(options,'s'); % auto-save flag
	nsflag(4)=(~isempty(loc));
	alloc=[alloc loc];
	GUIflag=findstr(options,'g'); % silent mode for powergui
	TESTflag=findstr(options,'t');% test mode, MAT file skipped.
	AUTOEXECflag=findstr(options,'&'); % if autoexecuted by the checksum mechanism.
	for i=1:length(options)
		if isempty(findstr('wosgt& ',options(i)))
			bad=[bad options(i)];
		end
	end			
	if length(bad)>0
		str=['POWER2SYS error: Valid options are: ''w'' ''o'' or ''s'' '];
		error(str);
	end
end
if nsflag(2)==1 % OPEN .NET FILE FOR INFO :
	nsfid=fopen([simuwin '.net'],'wt');
	if nsfid==-1
		str=['POWER2SYS error: cannot open output file ' simuwin '.net'];
		error(str);
	end 
else
	nsfid=0;
end
if isempty(GUIflag)
	disp('    ')
	disp(['Processing ' simuwin ' ...']);
end


%  GRAPHICAL ANALYSIS :

[rlc,source,switchns,line_dist,true_source,nb_true_sensor,nb_models,models, ...
            models_param yout,y_type,inputs,outputs,blocs,CustomBlocNumber, ...
     ParamNumber,BrancheREF,Noeuds,lignes,capteurlist,capteurindice,others] ...
     = nsgrafx(simuwin,nsfid,nsflag);


if nsflag(3)==1
	if nsflag(1)==1
		fclose(nsfid);
	end
	return;
end


inoutinfo(1).inputs=inputs; % keep this matrix for GUI
inputs  = nsrtrn(inputs);	
outputs = nsrtrn(outputs);
% noms des blocs sans le 'root':
blocswo=[blocs(:,length(simuwin)+2:size(blocs,2)) zeros(size(blocs,1),10)]; 
blocswo=nsrtrn(blocswo);
		
% SOLVE ISOLATED NETWORKS:

[rlc,allneu,nores]=nscinet(rlc,source);

if nsflag(2)==1 & max(nores)>1
fprintf(nsfid,'\nSpecial message: %i isolated networks have been found during analysis.\n',max(nores));
	fprintf(nsfid,'                 Resistive branches have been added from one point of the main \n');
	fprintf(nsfid,'                 network to one point in each of the other networks.\n');
	fprintf(nsfid,'                 This will in no way affect the results of the whole network \n');
	fprintf(nsfid,'                 since not current loop have been added in the process. \n');
	fprintf(nsfid,'                 However, voltage measurement between two of the formerly \n');
	fprintf(nsfid,'                 isolated networks is not allowed since those measurement \n');
	fprintf(nsfid,'                 would be dependant upon the location of the added branches. \n');
end

%  COHERANCE :
   
[rlc,BrancheREF,y_type,yout,capteurindice,outputs]= ...
	nscoeran(blocs,blocswo,lignes,Noeuds,rlc,source,others, ...
  BrancheREF,CustomBlocNumber,capteurindice,capteurlist,y_type, ...
        yout,inputs,outputs,allneu,nores,nsflag);
				
%  ETAT :

unit='OMU';
[A,B,C,D,var_nom,x0,x0sw,rlsw,uss,xss,yss,freqyss,Asw,Bsw,Csw,Dsw,Hlin ]= ...  
    	  circ2ss(rlc,switchns,source,line_dist,yout,y_type,unit,blocswo, ...
	          inputs,BrancheREF,1+~isempty(GUIflag),nsfid);
  
% verifier si les conditions initiales sont reelles:

x0_check=max(abs(x0));
if isnan(x0_check)==1 | x0_check==Inf
	errmess=['WARNING! Computation of initial conditions has given the following result: x0= '  num2str(x0_check)   '!  |--> Initial values of state-space variables set to 0' ];
 	nserror(errmess,[],-1);
	x0=zeros(size(x0));
	x0sw=zeros(size(x0sw));		
end

% WRITE STATE-VARIABLES NAMES ACCORDING TO THE BLOCK NAMES:

br_ref=[];
state_var=[];
for i=1:size(A,1)
	loc=findstr(var_nom(i,:),'_');
	br_ref=sscanf(var_nom(i,loc(1)+2:loc(2)-1),'%i');
	state_var(i,:)=[var_nom(i,1:loc(1)) blocswo(BrancheREF(br_ref,1),:)];

	% traitement de la matrice state_var pour variables d'etat multiples  
	% se rapportant au meme bloc
	% case: Line
	if CustomBlocNumber(BrancheREF(br_ref,1))==3 & BrancheREF(br_ref,2)<999
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->sect:' ...
		num2str(BrancheREF(br_ref,2)) zeros(1,length(state_var(i,:)) ...
		- length(nsdezero(state_var(i,:))) ...
		-length(num2str(BrancheREF(br_ref,2)))-7)];
 	end

	% CASE: MUTUAL INDUCTANCE :
	if CustomBlocNumber(BrancheREF(br_ref,1))==4
	   if BrancheREF(br_ref,2)==1
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:1' ...
			zeros(1,length(state_var(i,:)) ...
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==2
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:2' ...
			zeros(1,length(state_var(i,:)) ...
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==3
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:3' ...
			zeros(1,length(state_var(i,:)) ...
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==4
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->Core'   ...
			zeros(1,length(state_var(i,:)) ...
			-length(nsdezero(state_var(i,:)))-6)];
	   end
	end

	% CASE: LINEAR TRANSFORMER :
	if CustomBlocNumber(BrancheREF(br_ref,1))==5
	   if BrancheREF(br_ref,2)==1
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:1' ...
			zeros(1,length(state_var(i,:)) ... 
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==2
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:2' ...
			zeros(1,length(state_var(i,:)) ... 
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==3
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:3' ...
			zeros(1,length(state_var(i,:)) ... 
			-length(nsdezero(state_var(i,:)))-8)];
 	   end
	   if BrancheREF(br_ref,2)==4
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->Core'   ...
			zeros(1,length(state_var(i,:)) ...   
			-length(nsdezero(state_var(i,:)))-6)];
	   end

	end

	% CASE: SATURABLE TRANSFORMER :
	if CustomBlocNumber(BrancheREF(br_ref,1))==6
	   if BrancheREF(br_ref,2)==1
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:1' ...
			zeros(1,length(state_var(i,:)) ...
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==2
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:2' ...
			zeros(1,length(state_var(i,:)) ...
 			-length(nsdezero(state_var(i,:)))-8)];
	   end
	   if BrancheREF(br_ref,2)==3
 		state_var(i,:)=[ nsdezero(state_var(i,:)) '->coil:3' ...
			zeros(1,length(state_var(i,:)) ... 
			-length(nsdezero(state_var(i,:)))-8)];
	   end
	end
end
 
% REMOVE /n CHARACTERS:
state_var=nsrtrn(state_var);

x0perm=x0;     

% return if power2sys is called from the command line.
if isempty(AUTOEXECflag)
	return
end


%  BUILT SIMULATION STRUCTURE INTO SENSORS :

simsensor=nssyswrt(blocs,nb_models,models,models_param,nb_true_sensor, ...
		true_source,switchns,x0sw,capteurlist,capteurindice,simuwin, ...
      size(A),size(B),size(C),size(D));
   
% Initialization of MACHINES structure for machine load flow.
machines=powerflow(simuwin,source,inputs,outputs);

if isempty(TESTflag)

   % Data structure.
   psbdatas.A=A;
   psbdatas.B=B;
   psbdatas.C=C;
   psbdatas.D=D;
   psbdatas.x0=x0;
   psbdatas.uss=uss;
   psbdatas.xss=xss;
   psbdatas.yss=yss;
   psbdatas.state_var=state_var;
   psbdatas.inputs=inputs;
   psbdatas.outputs=outputs;
   psbdatas.freqyss=freqyss;
   psbdatas.phisat=[];
   psbdatas.isat=[];
   psbdatas.nb_points=[];
   psbdatas.fluxinit=[];
   psbdatas.Hlin=Hlin;
   psbdatas.x0perm=x0perm;
   psbdatas.true_source=true_source;
   psbdatas.nb_true_sensor=nb_true_sensor; 
   psbdatas.source=source;
   psbdatas.switchns=switchns; 
   psbdatas.rlsw=rlsw; 
   psbdatas.unit=unit; 
   psbdatas.line_dist=line_dist;
   psbdatas.simsensor=simsensor; 
   psbdatas.machines=machines; 
   psbdatas.inoutinfo=inoutinfo;
   psbdatas.ref_maskvalue=[];
   psbdatas.ref_lines=[];
   psbdatas.lockstates='off';

   %verify the lockstates field.
   zyz=loadsavedata(simuwin,'load');
   if ~isempty(zyz)
	lockstates=zyz.lockstates;
	if strcmp(lockstates,'on')
		psbdatas.x0=zyz.x0;
	end
	psbdatas.lockstates=zyz.lockstates;
   end

   loadsavedata(simuwin,'save',psbdatas);

end 

% make the connection beetween the data saved and the state-space block of equivalent circuit.
mskinit=['pss=powericon(''loadsavedata'',bdroot,''load'');A=pss.A;B=pss.B;C=pss.C;D=pss.D;x0=pss.x0;'];
set_param([simsensor,'/source/State-Space'],'MaskInitialization',mskinit);

% Initialization of distributed parameter lines models.
[noutput,ninput]=size(D);
nsdplini(line_dist,freqyss,yss,ninput,simsensor,nb_models,models);

% Initialization of saturable transformers.
%  initial flux calculations: phi=f(i) vectors fot the saturable transformer.

phisat=[];
isat=[];
nb_points=[];
fluxinit=[];

% Si on retrouve des NaN ou Inf dans Yss, alors 
% mettre flux initial a zero fluxflag=isnan(sum(sum(0*yss)));
% Si a la frequence 0 , on a un tension en regime permanent aux bornes de la 
% branche saturable, alors on met le flux a zero.

if nb_models(4)>0	
	for i=1:nb_models(4)
		ptr_models=sum(nb_models(1:3));
		mskentries=get_param(deblank(models(ptr_models+i,:)), ...
			'Mask Entries');

	        retvar=nsparm(mskentries,deblank(models(ptr_models+i,:)));
		mask1=retvar{1};
		mask2=retvar{2};
		IsatPhisat=retvar{5};
		Corelossres_Initflux=retvar{6};
		powerbase = mask1(1);
		wbase  = 2*pi*mask1(2);
		voltbase  = mask2(1);
		phibase   = voltbase/wbase;
		ibase     = powerbase/voltbase;
		inver = [];
		for q = size(IsatPhisat,1):-1:2
			inver = [inver; 0.0-IsatPhisat(q,:)];
		end
	    	IsatPhisat = [inver ; IsatPhisat(2:size(IsatPhisat,1),:)];
		nb_points(i)=size(IsatPhisat,1);		
		if size(isat,1)>0
			diff=size(IsatPhisat,1)-size(isat,2);
			if diff<0
				IsatPhisat=[IsatPhisat;zeros(0-diff,2)];
			elseif diff>0
				isat=[isat zeros(size(isat,1),diff)];
				phisat=[phisat zeros(size(isat,1),diff)];
			else
				;
			end
		end

		isat(i,:)= [IsatPhisat(:,1)*ibase*sqrt(2)]';
		phisat(i,:) = [IsatPhisat(:,2)*phibase*sqrt(2)]';

		% Initialisation du flux des transfo saturable
		% --------------------------------------------
		% Si le parametre Phi0 est declare, le flux initial de
		% l'integrateur est calcule comme suit:
		% 	Phi_init=Phibase*Phi0 dans l'integrateur
		% Sinon, on calcule le flux initial d'apres la valeur de la 
		% tension de regime permanent aux bornes de la branche 
		% saturable, calculee par etat.Cette valeur est retournee en 
		% complexe dans le vecteur de sortie YSS de power2sys.
		% Soit n le numero de la sortie correspondante, 
		% pour chaque frequence de source fs_i, 
		% la composante du flux initial est donnee par:
		%
		% Phi_init(i) = -real(Yss(n))/(2*pi*fs_i)
		%
		% Si on a plusieurs frequences, les sorties sont dans differents 
		% vecteurs colonne de la matrice YSS (ordonnee par frequences 
		% croissantes) Les flux initial est alors la somme de chacune 
		% des composantes calculees comme ci-dessus.

		% correction:
		ptr_models=ptr_models-nb_models(1); % nb_models(1) ne produit pas de sorties
		if length(Corelossres_Initflux)==2
			fluxinit(i)=Corelossres_Initflux(2)*phibase*sqrt(2);
		%Si on retrouve des NaN ou Inf dans Yss mettre flux initial a 0:
		elseif isnan(sum(0*yss(ptr_models+i,:)))==1
			fluxinit(i)=0;
		fprintf(nsfid,'\nWarning! Infinite amplitude frequency componant at a saturable transformer :\n');
		fprintf(nsfid,'          -> initial flux set to zero:\n');
		% Si a la frequence 0 , on a un tension en regime permanent 
		% aux bornes de la branche saturable
		% -> on met le flux a zero , car sinon ca donne infini
		elseif freqyss(1)==0 & 	yss(ptr_models+i,1)>1000*eps
			fluxinit(i)=0;  % cas ou yss~=0 pour f=0
		fprintf(nsfid,'\nWarning! Presence of a DC componant at the saturable transformer core:\n');
		fprintf(nsfid,'          -> initial flux set to zero:\n');
		elseif length(Corelossres_Initflux)==1
			% calcul a partir de YSS
			fluxinit(i)=0;
			for j=1:size(yss,2)
			    if abs(yss(ptr_models+i,j))>1000*eps 
				if freqyss(j)==0 % modif oct.96
				fluxinit(i)=0;
				else
				fluxinit(i)=fluxinit(i)-real(yss(ptr_models+i,j))/(2*pi*freqyss(j));
				end % fin modif
			    end
			end
		else
			error('power2sys err:2');				
		end
		
		% set the masks of the models in equivalent circuit.
		pp1=mat2str(isat(i,:));
		pp2=mat2str(phisat(i,:));
		pp3=mat2str(fluxinit(i));
		maskentries=[pp1,'|',pp2,'|',pp3];
		set_param([simsensor,'/source/transfosat/d',num2str(i)],'MaskValueString',maskentries);

	end
end


% MESSAGES :

if ~isempty(source) & nsflag(2)==1 
  fprintf(nsfid,'\nSteady-state values of outputs read as follow:\n');
  fprintf(nsfid,'Output name:\n\n');
  for i=1:size(outputs,1)
	if outputs(i,1)=='I'
		str='Amps ';
	elseif outputs(i,1)=='U'
		str='Volts';
	else
		error('power2sys err:3');
	end
	fprintf(nsfid,'%s:\n',outputs(i,:));
	for j=1:length(freqyss)
	  if freqyss(j)>0,
	    fprintf(nsfid,['     %5.3e(Hz) :  %5.3e ' str '< %5.3e deg.\n'], ...
			freqyss(j),abs(yss(i,j)),angle(yss(i,j))*180/pi );
	  else
	    fprintf(nsfid,['     %5.3e(Hz) :  %5.3e ' str '\n'], ...
			freqyss(j),real(yss(i,j)));
	  end
	end
	if isempty(freqyss)
	  fprintf(nsfid,'No inputs found in network!');
	end 
	fprintf(nsfid,'\n');
  end
end

 
if isempty(GUIflag)
  	disp(' ');
end
if nsflag(2)==1&isempty(GUIflag)
     disp(['Processing information and steady-state output stored in file: ' ...
     	simuwin '.net']);
     disp(' ');
end
if isempty(simsensor)&isempty(GUIflag)
	disp('Netsim error: No measurement block found in network');
	disp('              Simulink system not stored !');
        error
elseif isempty(GUIflag)
	disp(['Simulink equivalent state-space system stored in block: ', ...
		nsrtrn(simsensor)]);
end
if isempty(GUIflag)
  	disp(' ');
end
if nsflag(4)==1
	if isempty(GUIflag)
	  disp(['Flag ''s'' detected. Netsim autosave of Simulink window: ' ...
	  simuwin ]);
	  disp(' ');
	end
	close_system(simuwin,1);
	open_system(simuwin);
elseif isempty(GUIflag)
	disp(['Netsim has modified the Simulink window: ' simuwin ]);
	disp(['Consequently, the "Save" button has been enabled']);
	disp('  ');
end
if isempty(GUIflag)
  	disp('Ready for simulation');
end
if nsflag(2)==1
	fclose(nsfid);
end
disp(' ')


