function [machines]=powerflow(system,source,inputs,outputs,machines,init,freq)

% FLOWMAC is a utility function used to solve a load flow in a three-phase
% multimachine power system built with the Power System Blockset.
% FLOWMAC also initializes the SIMULINK model so that the simulation starts
% in steady state.
%
%    Syntax:  [machines]=powerflow(system,source,inputs,outputs)
%             [machines]=powerflow(system,[],[],[],machines)
%             [machines]=powerflow(system,[],[],[],machines,init)
%             [machines]=powerflow(system,[],[],[],machines,init,freq)
%
% Input arguments
% ===============
%       system : String = Name of the SIMULINK system containing the circuit
%       source : Source matrix (nb_input,7) containing:
%                col. 1, 2 Node numbers col. 1,2
%                Col. 3    Source type (0=voltage 1=current) 
%                Col. 4 5  Magnitude (V or A)  and phase (Degrees)
%                Col. 6    Frequency (Hz) 
%                Col. 7    Model type represented by the source type
%                          Two current sources are used for each
%                          machine three- phase winding
%                          stator and rotor
%                          31=Simplified synchronous machine stator
%                          32=Synchronous machine stator
%                          33=Asynchronous machine stator
%                          34=Asynchronous machine rotor
%                          35=Permanent MAgnet stator
%       inputs : String matrix (nb_input,n) of voltage and cuurent  source names
%      outputs : String matrix (nb_output,n) of output names
%     machines : (1,nb_mac) structure array of machine load flow parameters     
%                           See description below.
%         init : flag (0 or 1)  default value=0
%                init=0 : FLOWMAC will compute its own initial values
%                         for machine currents and motor slip guesses
%                         This option should be used for the first load flow
%                init=1 : FLOWMAC will use initial conditions 
%                         corresponding to the last last load flow
%                         This option may help load flow convergence.
%         freq : System frequency (Hz) used to solve the load flow
%                Default value freq=60 Hz;
%
% The MACHINES structure array is used to specify the reference power P
% and voltage VT for synchronous machines and reference mechanical power PMEC
% for asynchronous motors. The same  structure is used as an output argument
% to return updated values of power (P),  terminal voltage (VT), % mechanical
% power (PMEC), synchronous machine field Voltage (EF) and motor SLIP.
%
%     machines : (1,nb_mac) structure array with the following fields:
%        .name : String matrix containing the names of the nb_mac machines
%        .type : Integer specifying type of machine.
%                31 Simplified alternator;
%                32 Synchronous machines
%                33 Asynchronous motor
%                35 Permanent magnet motor
%     .bustype : Integer specifying machine bus type 
%                BUSTYPE=1 : PV bus ; Voltage(MACHINES.VT )
%                            and active power (MACHINES.P) are specified
%                BUSTYPE=2 : Swing bus ; Voltage (MACHINES.VT) and angle
%                            (MACHINES.PHASE) are specified.
%                            The power will be adjusted to balance the 
%                            power generated or absorbed by other machines
%                            and sources plus loads and losses.
%                If the circuit contains at least one voltage source
%                No swing is needed. All machine buses are declared type 1.
%                If the circuit contains no voltage source, one machine
%                must be specified as a swing bus (BUSTYPE=2).
%                        
%       .input : Integer =index to the line of the INPUTS string matrix
%                corresponding to the machines I_A current source 
%      .output : Integer =index to the line of the OUTPUTS string matrix
%                corresponding to the machines U_AB output
%           .P : Three-phase active powers (watts)
%                For the swing bus, the specified power is the initial guess.
%           .Q : Three-phase reactive powers (watts)
%                Sign conventions: For synchronous machines: P,Q>0 indicate
%                power going out of  the machine (generator mode).
%                For asynchronous machines P, Q >0 indicate power entering
%                the machine (motor mode).
%          .Vt : Machine terminal A-B voltage (volts rms);
%        .Phase: Phase angle of the A-Neutral terminal voltage (degrees)
%                for the synchronous machine declared as swing bus (BUSTYPE=2)
%          .Ef : Field voltage of synchronous machines (pu)
%         .Pmec: Mechanical power (pu) .P>0 for synchronous machines in generator
%                mode and for asynchronous machines in motor mode.
%         .slip: slip of asynchronous machines (pu)
%
% Output argument
% ===============
%     machines : (1,nb_mac) machine structure arrray
%
%               -If number of input argument = 4  the MACHINES structure is
%                returned with  initialized fields  NAME TYPE INPUT and OUTPUT.
%                P, Q U Vt Phase Ef Pmec are set to zero. BUSTYPE is set to 1;
%
%               -If number of input argument = 5 or 6 the MACHINES structure is
%                used as an input argument to specify the desired power flow
%                and as an output argument to return the load flow solution.
%                FLOWMAC returns an updated MACHINES structure with
%                new values of P Q VT PHASE EF PMEC and SLIP according to the
%                load flow solution;
%
%               -For synchronous machines declared with MACHINES.BUSTYPE=1,
%                specify the desired active power and voltage in MACHINES.P and
%                MACHINES.VT.
%                The obtained active and reactive powers are returned in
%                MACHINES.P and MACHINES.Q. The required mechanical power is
%                returned in MACHINES.PMEC.
%
%               -For the machine declared with MACHINES.BUSTYPE=2, specify
%                voltage and phase angle in MACHINES.VT and MACHINES.PHASE. 
%                The obtained active and reactive powers are returned in
%                MACHINES.P and MACHINES.Q. The required mechanical power is
%                returned in MACHINES.PMEC.
%
%               -For asynchronous machines (MACHINES.BUSTYPE=3), specify
%                the desired mechanical power in MACHINE.PMEC. 
%                The obtained active and reactive powers are returned in
%                MACHINES.P and MACHINES.Q and the obtained mechanical power
%                is returned in MACHINES.PMEC. Slip is returned in MACHINES.SLIP.
%              

%   G. Sybille (IREQ)  June 1997,September 1997
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.4 $

j=sqrt(-1);

FlowSolver='psbflowsolv'; % Simulink system used to solve load flow
nargs=nargin;

if nargs==4,
% Initialize MACHINES structure
% =============================
	[nb_input,n]=size(inputs);
	[nb_output,ncar]=size(outputs);
	nb_mac=0;
	machines=[];

	for i=1:nb_input,
    % Check type of current source simulating the machines stator
    % 31 Simplified alternator;
    % 32 Synchronous machines
    % 33 Asynchronous motor

		is_mac=(source(i,7)==31|source(i,7)==32|source(i,7)==33);
		str=deblank(inputs(i,:));
		nA=findstr(str,'_A'); if isempty(nA), nA=0; end;
		strlength=length(str);
		nA=nA(length(nA));
	 
		if  is_mac & nA~=0 & (nA==strlength-1 | nA==strlength-7)
			nb_mac=nb_mac+1;
			machines(nb_mac).name=inputs(i,3:nA-1);
			machines(nb_mac).type=source(i,7);
			if source(i,7)==33,
				machines(nb_mac).bustype=3; % asynch. motor
			else
				machines(nb_mac).bustype=1; % simpl. or complete synchr. machine
			end
			machines(nb_mac).input=i;
			for iout=1:nb_output;
				nAB=findstr(outputs(iout,:),'_AB');
				nABr=findstr(outputs(iout,nAB:ncar),'_ABrotor');
				if isempty(nABr), nABr=0; end
				if nAB & ~nABr & strcmp(outputs(iout,1:2),'U_') & ...
				   strcmp(outputs(iout,3:nAB-1),machines(nb_mac).name)
					machines(nb_mac).output=iout;
					break;
				end
			end
			machines(nb_mac).P=0;
			machines(nb_mac).Q=0;
			machines(nb_mac).Vt=0;
			machines(nb_mac).Phase=0;
			machines(nb_mac).Ef=0;
			machines(nb_mac).Pmec=0;
			machines(nb_mac).slip=0;
		end
	end

else
if nargs==5,
	 init=0;
	 freq=60;
elseif nargs==6,
	 freq=60;
elseif nargs~=5 & nargs~=6 & nargs~=7 
	error('The number of input arguments must be 5 6 or 7');
 end 

% SOLVE LOAD FLOW
% ===============
	machines1=machines;
	%eval(['load ',system]);
        psbdatas=loadsavedata(system,'load');

        A=psbdatas.A;
        B=psbdatas.B;
        C=psbdatas.C;
        D=psbdatas.D;
        x0=psbdatas.x0;
        uss=psbdatas.uss;
        xss=psbdatas.xss;
        yss=psbdatas.yss;
        inputs=psbdatas.inputs;
        outputs=psbdatas.outputs;
        freqyss=psbdatas.freqyss;
        Hlin=psbdatas.Hlin;
        x0perm=psbdatas.x0perm;
        source=psbdatas.source;
        switchns=psbdatas.switchns; 
        rlsw=psbdatas.rlsw; 
        unit=psbdatas.unit; 
        line_dist=psbdatas.line_dist;
        simsensor=psbdatas.simsensor; 
        machines=psbdatas.machines; 

	machines_saved=psbdatas.machines;

	%Use MACHINES input argument instead of structure saved in loadsavedata
	machines=machines1; 

	[nb_input,n]=size(inputs);
	[nb_output,ncar]=size(outputs);
	ifreq=find(freqyss==freq);
	if isempty(ifreq)      
		error(sprintf...
		('Specified frequency %g Hz not found in freqyss vector',freq));
	end
	nb_mac=length(machines);
	ind_A=[]; ind_AB=[]; Pref=[]; Uref=[]; Pmec=[]; bus_type=[];
	Phaseref=0;
	No_Swing_Bus=0;
	for imac=1:nb_mac
		ind_AB=[ind_AB,machines(imac).output];
		ind_A=[ind_A,machines(imac).input];
		bus_type(imac)=machines(imac).bustype;
		switch bus_type(imac)
			case 1
			Pref(imac)=machines(imac).P;
			Uref(imac)=machines(imac).Vt;
			Pmec(imac)=0;                     

			case 2
			Phaseref=machines(imac).Phase;
			No_Swing_Bus=imac;
			Pref(imac)=machines(imac).P;
			Uref(imac)=machines(imac).Vt;
			Pmec(imac)=0;                     

			case 3
			Pref(imac)=0;                 
			Uref(imac)=0;                   
			Pmec(imac)=machines(imac).Pmec;
		end
	end

	ind_BC=ind_AB+1;
	ind_Yothers=1:nb_output;
	ind_Yothers([ind_AB,ind_BC])=[];
	ind_B=ind_A+1;
	ind_Uothers=1:nb_input;
	ind_Uothers([ind_A,ind_B])=[];
	% Reorder H(nb_output,nb_input) system transfer function 
	% and U(nb_input) input vector of source values
	% =======================================================
	% Reorder  lines of H so that the first 2*nb_mac voltage outputs are:
	%  U_ACmac1 UAC_mac2 ... U_BC_mac1 U_BC_mac2
	% and reorder  columns so that the first 2*nb_mac current inputs are:
	%  IA_mac1 IA_mac2 ... IB_mac1 IB_mac2

	%
	H=Hlin([ind_AB,ind_BC,ind_Yothers],[ind_A,ind_B,ind_Uothers],ifreq);

	%Replace the first nb_mac U_AB outputs with U_AC=U_AB+U_BC 
	for i=1:nb_mac,
		H(i,:)=H(i,:)+H(i+nb_mac,:);
	end

	% Reorder the input complex vector U so that the first 2*nb_mac
	%  current inputs are:
	%  IA_mac1 IA_mac2 ... IB_mac1 IB_mac2

	U=uss(:,ifreq);
	% Set system current inputs from switches to zero
	% because the Hlin transfer function includes the closed switches
	[nb_switches,n]=size(switchns);
	if ~isempty(switchns) U(switchns(:,6))=zeros(nb_switches,1); end

	U=U([ind_A,ind_B,ind_Uothers]);

	%Save index vectors into original INPUTS and OUTPUTS string matrices
	%-----------------------------------------------------------------
	index_input=1:length(inputs);
	index_input=index_input([ind_A,ind_B,ind_Uothers]);
	index_output=1:length(outputs);
	index_output=index_output([ind_AB,ind_BC,ind_Yothers]);

	% Add a dummmy source if the only sources are the current sources
	% associated with machines
	if nb_input==2*nb_mac,
	% New column in H; and new line in U
		nb_input=nb_input+1;
		H(:,nb_input)=zeros(nb_output,1);
		U(nb_input)=0;
	end


    %Find parameters of asynchronous machines (r1 x1 r2 x2 xm)
	% and machine model of the simulink Load flow solver (Ke Kir Kii)
    %----------------------------------------------------------------
	Vnom=zeros(1,nb_mac);
	R1=zeros(1,nb_mac); X1=zeros(1,nb_mac);
	r2=zeros(1,nb_mac); x2=zeros(1,nb_mac);
	Ke=diag(ones(1,nb_mac)); Kir=ones(1,nb_mac); Kii=ones(1,nb_mac);
	smaxT=zeros(1,nb_mac); P_maxT=zeros(1,nb_mac);

	for i=1:nb_mac
		% Find order no_order of the machine among all machines having
		% the same type in the SOURCE matrix
		no_order=length(find(source(ind_A,7)'==machines(i).type & ind_A<=ind_A(i)));  

	   if machines(i).type==33
			 % Get parameters of Asyncronous machines
             % ---------------------------------------
             block_title=[simsensor,'/source/async_mac/d',int2str(no_order)];

             MaskValStr=get_param(block_title,'MaskValueString');
             n=findstr(MaskValStr,'|') ;
             n1=n(1); n2=n(2); n3=n(3); n4=n(4); n5=n(5); n6=n(6);

            % 1st string = vector of [Pnom Vnom fnom]
            val=str2num(MaskValStr(1:n1-1));
            Pnom=val(1); Vnom(i)=val(2); fnom=val(3);
            Vbase=Vnom(i)/sqrt(3)*sqrt(2);
            Zbase=Vnom(i)^2/Pnom;
            Ibase=Vbase/Zbase;

            % 2nd string = vector of Rs Lls (pu); 
            val=str2num(MaskValStr(n1+1:n2-1));
            r1=val(1)*Zbase; x1=val(2)*Zbase;

            % 3rd string = vector of Rr' Llr' (pu); 
            val=str2num(MaskValStr(n2+1:n3-1));
            r2(i)=val(1)*Zbase; x2(i)=val(2)*Zbase;

            % 4th string = Lm (pu)
            val=str2num(MaskValStr(n3+1:n4-1));
            xm=val(1)*Zbase;

            % 5th string = H(s) F(p.u) p=nb. of pairs of poles
            val=str2num(MaskValStr(n4+1:n5-1));
            speed_rpm=60*fnom/val(3);

			 % Compute parameters of Asynchronous machine models
			 % used in the Simulink Load flow solver
             % ------------------------------------------------
			Z1=j*xm*(r1+j*x1)/(r1+j*(x1+xm));
			R1(i)=real(Z1);
			X1(i)=imag(Z1);

			Ke(i,i)=j*xm/(r1+j*(x1+xm));
			Kir(i)=1+x2(i)/xm;
			Kii(i)=-r2(i)/xm;

			% Find slip and power at max. torque (smaxT and P_maxT)
			% for later calculation of slip guess
			%r2/s at maximum torque  = sqrt(R1+sqrt(R1^2+(X1+x2)^2)); 
			smaxT(i)=r2(i)/sqrt(R1(i)+sqrt(R1(i)^2+(X1(i)+x2(i))^2));
			V1a=abs(Vnom(i)/sqrt(3)*j*xm/(r1+j*(x1+xm))); %Thevenin voltage
			omegas=speed_rpm/60*2*pi;
			Tmax=1/omegas*3/2*V1a^2/(R1(i)+sqrt(R1(i)^2+(X1(i)+x2(i))^2));
			P_maxT(i)=Tmax*omegas*(1-smaxT(i)); % power at max Torque

		else
			R1(i)=0; X1(i)=0;
			%Set r2 x2 to 1 to avoid division per zero in Eth/Z(s) block
			r2(i)=1; x2(i)=1; 
			Ke(i,i)=0;
			Kir(i)=0;
			Kii(i)=0;
		end
	end

	if init
		t_start_current_mot=0; %Inject motor current at t=0


		% USE INITIAL CONDITIONS FROM PREVIOUS LOAD FLOW
		real_current_guess=[]; imag_current_guess=[];
		real_Van_init=[]; imag_Van_init=[];
		slip_guess=ones(1,nb_mac);
		for imac=1:nb_mac
			no_input=machines_saved(imac).input;
			no_output=machines_saved(imac).output;
			real_current_guess(imac)=real(uss(no_input,ifreq));
			imag_current_guess(imac)=imag(uss(no_input,ifreq));
			Van=(2*yss(no_output,ifreq)+yss(no_output+1,ifreq))/3; % Van=R2*Vab
			real_Van_init(imac)=real(Van);
			imag_Van_init(imac)=imag(Van);
			if bus_type(imac)==3,
				slip_guess(imac)=machines_saved(imac).slip;
			end
		end               
	else
		% COMPUTE INITIAL CONDITIONS
		% Set initial current guesses, initial motor voltage and slip guesses
		t_start_current_mot=0.5; %Inject motor current at t=0.5s
		real_current_guess=zeros(1,nb_mac);
		imag_current_guess=zeros(1,nb_mac);
		slip_guess=ones(1,nb_mac);
		real_Van_init=zeros(1,nb_mac);
		imag_Van_init=zeros(1,nb_mac);
		for imac=1:nb_mac
			if bus_type(imac)==3,
				% Assume s  proportionnal to Pmec between 0 and P_maxT 
				slip_guess(imac)=smaxT(imac)*Pmec(imac)/P_maxT(imac);
				real_Van_init(imac)=Vnom(i)/sqrt(3)*sqrt(2);
			else
				real_current_guess(imac)=Pref(imac)/Uref(imac)/sqrt(3)*sqrt(2);
			end
		end
	end


	% Encode parameters in the SIMULINK Load Flow solver
	% ==================================================
	psbflowsolv([],[],[],'load');


	% Mux and Constants blocks  at input of Y=H*U block
	% -------------------------------------------------
	set_param([FlowSolver,'/LoadFlow/Real_Imac'],...
	'Inputs',mat2str([nb_mac,nb_mac]));
	set_param([FlowSolver,'/LoadFlow/Imag_Imac'],...
	'Inputs',mat2str([nb_mac,nb_mac]));

	str=sprintf('real(U(%d:%d))',2*nb_mac+1,nb_input);
	set_param([FlowSolver,'/LoadFlow/Real_Uothers'],'Value',str);
	str=sprintf('imag(U(%d:%d))',2*nb_mac+1,nb_input);
	set_param([FlowSolver,'/LoadFlow/Imag_Uothers'],'Value',str);

	no_sig=[2*nb_mac,nb_input-2*nb_mac];
	set_param([FlowSolver,'/LoadFlow/Real_U'],'Inputs',mat2str(no_sig));
	set_param([FlowSolver,'/LoadFlow/Imag_U'],'Inputs',mat2str(no_sig));

	% Selector blocks at output  of Y=H*U block
	% -------------------------------------------
	str=sprintf('%d:%d',1,2*nb_mac);
	set_param([FlowSolver,'/LoadFlow/Real_Vmac_ac_bc'],'Elements',str,...
		'InputPortWidth',int2str(nb_output));
	set_param([FlowSolver,'/LoadFlow/Imag_Vmac_ac_bc'],'Elements',str,...
		'InputPortWidth',int2str(nb_output));

	str=sprintf('%d:%d',1,nb_mac);
	set_param([FlowSolver,'/LoadFlow/Real_Vmac_ac'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));
	set_param([FlowSolver,'/LoadFlow/Imag_Vmac_ac'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));

	% Selector blocks at output  of S=V*I' block
	% -------------------------------------------
	str=sprintf('%d:%d',1,nb_mac);
	set_param([FlowSolver,'/LoadFlow/PQ_values/Real_Sac'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));
	set_param([FlowSolver,'/LoadFlow/PQ_values/Imag_Sac'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));

	str=sprintf('%d:%d',nb_mac+1,2*nb_mac);
	set_param([FlowSolver,'/LoadFlow/PQ_values/Real_Sbc'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));
	set_param([FlowSolver,'/LoadFlow/PQ_values/Imag_Sbc'],'Elements',str,...
		'InputPortWidth',int2str(2*nb_mac));

	% Matrix Gain Blocks for selection of controlled power and/or voltage 
	%--------------------------------------------------------------------
	set_param([FlowSolver,'/LoadFlow/K_Pgen'],'MaskValueString',...
	mat2str(diag(bus_type==1)));
	set_param([FlowSolver,'/LoadFlow/K_Ugen'],'MaskValueString',...
	mat2str(diag(bus_type==1)));
	set_param([FlowSolver,'/LoadFlow/K_Uswing'],'MaskValueString',...
	mat2str(diag(bus_type==2)));

	% Matrix Gain blocks selecting synchronous machine currents
	% and asynchronous motor currents
    %----------------------------------------------------------
	set_param([FlowSolver,'/LoadFlow/K_mac1'],'MaskValueString',...
	mat2str(diag(bus_type~=3)));
	set_param([FlowSolver,'/LoadFlow/K_mac2'],'MaskValueString',...
	mat2str(diag(bus_type~=3)));
	set_param([FlowSolver,'/LoadFlow/K_mot1'],'MaskValueString',...
	mat2str(diag(bus_type==3)));
	set_param([FlowSolver,'/LoadFlow/K_mot2'],'MaskValueString',...
	mat2str(diag(bus_type==3)));

	% Selector blocks of voltage calculation of asynchronous motors
	% -------------------------------------------------------------
	str=sprintf('%d:%d',1,nb_mac);
	set_param([FlowSolver,'/LoadFlow/Async_Machines/VPP>VAN/Vac_r'],...
	'Elements',str,'InputPortWidth',int2str(2*nb_mac));
	set_param([FlowSolver,'/LoadFlow/Async_Machines/VPP>VAN/Vac_i'],...
	'Elements',str,'InputPortWidth',int2str(2*nb_mac));
	str=sprintf('%d:%d',nb_mac+1,2*nb_mac);
	set_param([FlowSolver,'/LoadFlow/Async_Machines/VPP>VAN/Vbc_r'],...
	'Elements',str,'InputPortWidth',int2str(2*nb_mac));
	set_param([FlowSolver,'/LoadFlow/Async_Machines/VPP>VAN/Vbc_i'],...
	'Elements',str,'InputPortWidth',int2str(2*nb_mac));


	%Solve the Load Flow
	%===================
	tmax=1.0;  
% use FixedStep discrete time step=0.5
	options=simset('Solver','FixedStepDiscrete','FixedStep',0.50,...
	'SrcWorkspace','current');
	[tout,xout,yout]=sim(FlowSolver,[0 tmax],options);


	% Display solution
	% ================
	% yout=output vector contains:
	% output 1: Real(Ia1 Ia2 Ia3 Ib1 Ib2 Ib3 ... )
	% output 2: Imag(Ia1 Ia2 Ia3 Ib1 Ib2 Ib3 ... )
	% output 3: Real(Vac1 Vac2 Vac3 Vbc1 Vbc2 Vbc3 ... )
	% output 4: Imag(Vac1 Vac2 Vac3 Vbc1 Vbc2 Vbc3 ... )
	% output 5: P1 P2 P3 ...
	% output 6: Q1 Q2 Q3 ...
	% output 7: Pmec(mac1 mac2 mac2 ...) % Pmec of sync mach set to 0
	% output 8: slip(mac1 mac2 mac2 ...) % slip of sync mach set to 1


	nt=length(tout);
	%Magnitudes and phases of machine currents
	%---------------------------------------
	Ia_real=yout(nt,1:nb_mac);
	Ib_real=yout(nt,  nb_mac+1:2*nb_mac);
	Ia_imag=yout(nt,2*nb_mac+1:3*nb_mac);
	Ib_imag=yout(nt,3*nb_mac+1:4*nb_mac);

	Vac_real=yout(nt,4*nb_mac+1:5*nb_mac);
	Vbc_real=yout(nt,5*nb_mac+1:6*nb_mac);
	Vac_imag=yout(nt,6*nb_mac+1:7*nb_mac);
	Vbc_imag=yout(nt,7*nb_mac+1:8*nb_mac);

	Pmac=yout(nt,8*nb_mac+1:9*nb_mac);
	Qmac=yout(nt,9*nb_mac+1:10*nb_mac);

	Pmec_mac=yout(nt,10*nb_mac+1:11*nb_mac);
	slip_mac=yout(nt,11*nb_mac+1:12*nb_mac);

	Pmec_=[tout,yout(:,10*nb_mac+1:11*nb_mac)]
	slip_=[tout,yout(:,11*nb_mac+1:12*nb_mac)]

	Imac_a=Ia_real+j*Ia_imag;
	Imac_b=Ib_real+j*Ib_imag;
	Vmac_ac=Vac_real+j*Vac_imag;
	Vmac_bc=Vbc_real+j*Vbc_imag;

	% Phase angle correction to match the reference Swing bus phase angle
	if No_Swing_Bus
		Van=(2*Vmac_ac(No_Swing_Bus)-Vmac_bc(No_Swing_Bus))/3; % AN voltage
		Dphi=-angle(Van)+Phaseref*pi/180;
		Imac_a=Imac_a*exp(j*Dphi); 
		Imac_b=Imac_b*exp(j*Dphi); 
		Vmac_ac=Vmac_ac*exp(j*Dphi); 
		Vmac_bc=Vmac_bc*exp(j*Dphi); 
	end

	% Display solution
	% ================
	%12345678901 1234567890 1234577890 1234567890 1234567890 1234567901 12345678
	fprintf(...
	'Pmac(W)     Uac(V)     Ubc(V)     Ia(A)      Ib(A)      errP       errUac\n');
	fprintf(...
	'Qmac(var)   (deg.)     (deg.)     (deg.)     (deg.)     (pc)        (pc)\n');
	for i=1:nb_mac
	   fprintf('%s\n',machines(i).name)
		%Modules et erreurs
		switch bus_type(i)
			case 1
			errP=(Pmac(i)-Pref(i))/Pref(i)*100;
			errU=(abs(Vmac_ac(i))/sqrt(2)-Uref(i))/Uref(i)*100;
			slip_mac(i)=0;

			case 2
			errP=0;
			errU=(abs(Vmac_ac(i))/sqrt(2)-Uref(i))/Uref(i)*100;
			slip_mac(i)=0;

			case 3
			errP=0;
			errU=0;

		end
		fprintf('%-11.4g %-10.4g %-10.4g %-10.4g %-10.4g %-11.4g %-11.4g\n',...
		Pmac(i),abs(Vmac_ac(i)),abs(Vmac_bc(i)),...
		abs(Imac_a(i)),abs(Imac_b(i)),errP,errU);

		%Q et Angles
		fprintf(['%-11.4g %-10.2f %-10.2f %-10.2f %-10.2f\n'],Qmac(i),...
		angle(Vmac_ac(i))*180/pi,angle(Vmac_bc(i))*180/pi,...
		angle(Imac_a(i))*180/pi,angle(Imac_b(i))*180/pi);
	end

	% Encode initial conditions in SIMULINK machine models
	% ====================================================
	 %         12345678901 12345678901 12345778901 12345678901 12345678901 
	fprintf('\nPmac(pu)    Van(pu)     Ean(pu)     Ia(pu)      Pmec(pu)    slip\n');
	fprintf('Qmac(pu)     (deg.)      (deg.)     (deg.)\n');
	%

	for i=1:nb_mac,
	   fprintf('%s\n',machines(i).name)
	   % Terminal voltage Van is calculated as : Van= 1/3(2Vac-Vbc)
	   % where n= internal neutral.
	   % This equation is true under the assumption that the
	   % three currents and three internal voltages Ean Ebn Ecn are balanced.
	   % (Terminals voltages Van Vbn Vcn to internal neutral are also balanced)
	   Van=(2*Vmac_ac(i)-Vmac_bc(i))/3; % Terminal voltage
	   
		% Find order no_order of the machine among all machines having
		% the same type in the SOURCE matrix
	   no_order=length(find(source(ind_A,7)'==machines(i).type & ind_A<=ind_A(i)));  

	   switch machines(i).type
		  
		  case 31 % SIMPLIFIED ALTERNATOR ; Units in pu 
			 
			 block_title=[simsensor,'/source/simp_alt/d',int2str(no_order)];

			 % Get string of parameter values = Four strings separated by |
			 % --------------------------------------------------------------
			 MaskValStr=get_param(block_title,'MaskValueString');
			 n=findstr(MaskValStr,'|') ; n1=n(1); n2=n(2); n3=n(3); n4=n(4);

			 % Find rated values and Compute base values
			 %-------------------------------------------
			 % 1st string = vector of [Pnom Vnom fnom]
			 val=str2num(MaskValStr(1:n1-1));
			 Pnom=val(1); Vnom=val(2); 
			 Vbase=Vnom/sqrt(3)*sqrt(2);
			 Zbase=Vnom^2/Pnom;
			 Ibase=Vbase/Zbase;   

			 % Find internal voltage,initial electrical angle Theta,
			 % Mechanical power
			 %------------------------------------------------------
			 % 3rd string = vector of internal impedance [R X] (pu)
			 val=str2num(MaskValStr(n2+1:n3-1));
			 Z_int=val(1)*Zbase + j*val(2)*Zbase; % Machine int. impedance (ohms)
			 Ean=Van+Z_int*Imac_a(i); % Internal voltage (Phase A)
			 %Theta= Initial angle of rotor  in electrical degrees
			 Theta=angle(Ean)*180/pi; 
			 Pmec=3/2*real(Ean*Imac_a(i)');
	   
			 % Update initial conditions Theta0; 3 current magnitudes and phases
			 % -----------------------------------------------------------------
			 % 4th string=vector of init. conditions
			 % [dw0 theta0 Ia Ib Ic PhiA PhiB PhiC]
			 val=str2num(MaskValStr(n3+1:n4-1)); dw0=val(1);
			 str_init=sprintf('[%g %g %g %g %g %g %g %g]',dw0,Theta,...
			 abs(Imac_a(i))/Ibase,abs(Imac_b(i))/Ibase,...
			 abs(Imac_a(i)+Imac_b(i))/Ibase,... 
			 angle(Imac_a(i))*180/pi,angle(Imac_b(i))*180/pi,...
			   angle(-Imac_a(i)-Imac_b(i))*180/pi);
			 MaskValStr=[MaskValStr(1:n3),str_init];
			 set_param(block_title,'MaskValueString',MaskValStr);
	   
		  case 32 % SYNCHRONOUS MACHINE; Standard parameters in pu.
			 % Get string of parameter values = Seven strings separated by |
			 % --------------------------------------------------------------
			 block_title=[simsensor,'/source/sync_mac/d',int2str(no_order)];

			 MaskValStr=get_param(block_title,'MaskValueString');
			 n=findstr(MaskValStr,'|') ;
			 n1=n(1); n2=n(2); n3=n(3); n4=n(4); n5=n(5); n6=n(6);

			 % Find rated values and Compute base values
			 %-------------------------------------------
			 % 1st string = vector of [Pnom Vnom fnom]
			 val=str2num(MaskValStr(1:n1-1));
			 Pnom=val(1); Vnom=val(2); 
			 Vbase=Vnom/sqrt(3)*sqrt(2);
			 Zbase=Vnom^2/Pnom;
			 Ibase=Vbase/Zbase;

			 % Find internal voltage,initial electrical angle Theta,
			 % Mechanical power
			 %-------------------------------------------------------
			 % 2nd string = vector of Rs Ll Lmd Lmq (pu); (Ld=Ll+Lmd; Lq=Ll+Lmq)
			 val=str2num(MaskValStr(n1+1:n2-1));
			 Rs=val(1)*Zbase; % Stator resistance (ohms)
			 Xd=(val(2)+val(3))*Zbase; Xq=(val(2)+val(4))*Zbase; % Machine Xd  Xq (ohms)
			 % Ef1 vector in phase with Ef=Ean ; (Fitzgerald P317)
			 % Ef1=Vterminal +Ia*Rs + j*Ia*Xq :
			 Ef1= Van + Imac_a(i)*Rs + j*Imac_a(i)*Xq; 
			 % Id = vector lagging Ef1 by 90 degrees :
			 Id=abs(abs(Imac_a(i)))*sin(angle(Ef1)-angle(Imac_a(i))) * exp(j*(angle(Ef1)-pi/2));
			 Iq=abs(abs(Imac_a(i)))*cos(angle(Ef1)-angle(Imac_a(i))) * exp(j*(angle(Ef1)));
			 % Internal voltage Ef=Ean = Vt + Rs*Ia + jId*Xd +j*Iq*Xq;
			 Ean= Van + Imac_a(i)*Rs + j*Id*Xd +j*Iq*Xq;
			 fprintf('Id=%6.4f pu <%6.2f deg.; Iq=%6.4f pu <%6.2f deg. Ef= %6.4f pu <%6.2f deg.\n',...
				abs(Id)/Ibase,angle(Id)*180/pi,abs(Iq)/Ibase,angle(Iq)*180/pi,...
				abs(Ean)/Vbase,angle(Ean)*180/pi);
			 %Theta= Initial angle of rotor  in electrical degrees
			 Theta=angle(Ean)*180/pi-90; 
			 %Mechanical power= Electrical power + Losses in stator
			 Pmec=3/2*(real(Van*Imac_a(i)') + Rs*abs(Imac_a(i))^2); 
	   
			 % Update initial conditions Theta0; 3 current magnitudes and phases
			 % -----------------------------------------------------------------
			 % 6th string=vector of init. conditions
			 % [dw0 theta0 Ia Ib Ic PhiA PhiB PhiC Vf=Ean]
			 val=str2num(MaskValStr(n5+1:n6-1)); dw0=val(1); Vf=val(9);
			 str_init=sprintf('[%g %g %g %g %g %g %g %g %g]',dw0,Theta,...
			 abs(Imac_a(i))/Ibase,abs(Imac_b(i))/Ibase,...
			 abs(Imac_a(i)+Imac_b(i))/Ibase,... 
			 angle(Imac_a(i))*180/pi,angle(Imac_b(i))*180/pi,...
			 angle(-Imac_a(i)-Imac_b(i))*180/pi,abs(Ean)/Vbase);
			 MaskValStr=[MaskValStr(1:n5),str_init,MaskValStr(n6:length(MaskValStr))];
			 set_param(block_title,'MaskValueString',MaskValStr);

		  case 33 % ASYNCHRONOUS MACHINE; Standard parameters in pu.

			Ean=0;
			Pmec=Pmec_mac(i);

			 % Get string of parameter values = Seven strings separated by |
			 % --------------------------------------------------------------
			 block_title=[simsensor,'/source/async_mac/d',int2str(no_order)];

			 MaskValStr=get_param(block_title,'MaskValueString');
			 n=findstr(MaskValStr,'|') ;
			 n1=n(1); n2=n(2); n3=n(3); n4=n(4); n5=n(5); n6=n(6);

			 % 1st string = vector of [Pnom Vnom fnom]
			 val=str2num(MaskValStr(1:n1-1));
			 Pnom=val(1); Vnom=val(2); 
			 Vbase=Vnom/sqrt(3)*sqrt(2);
			 Zbase=Vnom^2/Pnom;
			 Ibase=Vbase/Zbase;

			 % Update initial conditions  slip, Theta0, 3 current magnitudes, 3 current phases
			 % -------------------------------------------------------------------------------
			 % 6th string=vector of init. conditions  [slip theta0 Ia Ib Ic PhiA PhiB PhiC ]
			 val=str2num(MaskValStr(n5+1:n6-1)); Theta=val(2);
			 str_init=sprintf('[%g %g %g %g %g %g %g %g]',slip_mac(i),Theta,...
			 abs(Imac_a(i))/Ibase , abs(Imac_b(i))/Ibase , abs(Imac_a(i)+Imac_b(i))/Ibase,... 
			 angle(Imac_a(i))*180/pi,angle(Imac_b(i))*180/pi , angle(-Imac_a(i)-Imac_b(i))*180/pi);
			 MaskValStr=[MaskValStr(1:n5),str_init,MaskValStr(n6:length(MaskValStr))];
			 set_param(block_title,'MaskValueString',MaskValStr);

	   end



		% PRINT Load flow solution and initial conditions; update MACHINES
		%-----------------------------------------------------------------
		%Modules
		fprintf('%-11.4g %-11.4g %-11.4g %-11.4g %-11.4g %-11.4g\n',...
		Pmac(i)/Pnom,abs(Van)/Vbase,abs(Ean)/Vbase,abs(Imac_a(i))/Ibase,...
		Pmec/Pnom,slip_mac(i));
		%Q + Angles 
		fprintf(['%-11.4g %-11.2f %-11.2f %-11.2f\n'],...
		Qmac(i)/Pnom,angle(Van)*180/pi,angle(Ean)*180/pi,angle(Imac_a(i))*180/pi)

		% Uptade MACHINES.P, Q, and U
		machines(i).P=Pmac(i); % Three phase active power (watts)
		machines(i).Q=Qmac(i); % Three phase reactive power (wars)
		machines(i).Vt=abs(Vmac_ac(i)-Vmac_bc(i))/sqrt(2); % ABrms voltage(Volts)
		machines(i).Ef=abs(Ean)/Vbase;
		machines(i).Pmec=Pmec;     
		machines(i).slip=slip_mac(i);     
	end

	%Compute and save new inputs, outputs, initial conditions
	%========================================================
	% New source matrix containing updated Mag. and Phase of machine currents
	% -----------------------------------------------------------------------
	for i=1:nb_mac
		source(index_input(i),4)=abs(Imac_a(i));
		source(index_input(i),5)=angle(Imac_a(i))*180/pi;
		source(index_input(i+nb_mac),4)=abs(Imac_b(i));
		source(index_input(i+nb_mac),5)=angle(Imac_b(i))*180/pi;
		source(index_input(i),6)=freq; % update IA source frequency
        source(index_input(i+nb_mac),6)=freq; % update IB source frequency
	end

	[uss,xss,yss,x0,x0sw,Asw,Bsw,Csw,Dsw,freq,Hlin ]=...
			 etass(A,B,C,D,switchns,source,rlsw,unit,freqyss,line_dist);
	x0perm=x0;

	% Reinitialize distributed parameter line models
	% -----------------------------------------------
	nsdplini(line_dist,freqyss,yss,nb_input,simsensor);

	% Save updated variables in .mat file
	% -----------------------------------

        % structure des donnees:
        psbdatas.A=A;
        psbdatas.B=B;
        psbdatas.C=C;
        psbdatas.D=D;
        psbdatas.x0=x0;
        psbdatas.uss=uss;
        psbdatas.xss=xss;
        psbdatas.yss=yss;
        psbdatas.inputs=inputs;
        psbdatas.outputs=outputs;
        psbdatas.freqyss=freqyss;
        psbdatas.Hlin=Hlin;
        psbdatas.x0perm=x0perm;
        psbdatas.source=source;
        psbdatas.switchns=switchns; 
        psbdatas.rlsw=rlsw; 
        psbdatas.unit=unit; 
        psbdatas.line_dist=line_dist;
        psbdatas.simsensor=simsensor; 
        psbdatas.machines=machines; 

   loadsavedata(simuwin,'save',psbdatas);
	close_system(FlowSolver,0);

end
	
