function [params] = bl25conv(nom,in2,in3,in4,mecin,iniin,simsat,satin,...
                             dispvfd,choice)

%BL25CONV
%
% Converts Synchronous Machine block parameters before simulation. Converts 
% from either SI fundamental or pu standard parameters.
% Revised 1997-Jun-30
%

%   Synchronous Machine model by:
%   Louis-A. Dessaint, Ph.D., and Roger Champagne, ing.
%
%   Standard to fundamental conversion part of this program by:
%   I. Kamwa, IREQ (Hydro-Quebec), Varennes, Quebec
%
%   Polynomial approximation of exponential part of this program by:
%   M. Gavrilovic, IREQ (Hydro-Quebec), Varennes, Quebec
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters which are always the same (all three dialog boxes)
Pn=nom(1); Vn=nom(2); fn=nom(3);	% Nominal parameters
p=mecin(3);				% pairs of poles
wen=2*pi*fn;				% nominal electrical ang. freq.
wmn=wen/p;				% nominal angular speed
Vb=sqrt(2/3)*Vn;			% base voltage
Ib=sqrt(2/3)*Pn/Vn;			% base current

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fundamental SI parameters entered
if (choice=='1')
   sta=in2;		% stator parameters
   rot=in3;		% field parameters
   dam=in4;		% damper parameters 
   iounits='on';	% I/O in SI units

   Zb=Vn^2/Pn;		% base impedance
   Lb=Vn^2/Pn/wen;	% base inductance

   iopu=iniin(3:5)/Ib;			% init. currents converted to p.u.
   Rs=sta(1)/Zb; Ll=sta(2)/Lb;		% Stator parameters
   Lmd=sta(3)/Lb; Lmq=sta(4)/Lb;
   Rf=rot(1)/Zb; Llfd=rot(2)/Lb;	% Rotor parameters
   Rkd=dam(1)/Zb; Llkd=dam(2)/Lb;	% Damper parameters
   Rkq=dam(3)/Zb; Llkq=dam(4)/Lb;
   mec=[mecin(1)*wmn^2/2/Pn,...		% Inertia constant
        mecin(2)*wmn^2/Pn,...		% friction coefficient
        p];				% pole pairs

   if (length(nom)<4 | nom(4)==0)
      Ifn=0;
      Vfns=Rf*Vb/Lmd;		% nominal field voltage viewed from rotor;
      Vfopu=iniin(9)/Vfns;	% initial field voltage converted to p.u.
   else
      Ifn=nom(4);		% Ifd producing 1 pu no-load term. voltage
      Vfnr=Rf*Pn/(Ifn*Lmd^2);	% nominal field voltage viewed from stator;
      Vfopu=iniin(9)/Vfnr;	% initial field voltage converted to p.u.
      if ~isempty(satin)
         sat=[ satin(1,:)/Ifn; satin(2,:)/Vn ];
      end
   end

   ini=[iniin(1:2),iopu,iniin(6:8),Vfopu];

   if (~isempty(satin) & Ifn==0)
      DlgName=gcb;
      ErrorString=('You can''t model saturation in S.I. units if Ifn is not known.');
      errordlg(ErrorString,DlgName)
      return
   end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Standard p.u. parameters entered         
else
   X=in2;	% reactances
   tau=in3;	% time constants
   ra=in4;	% stator resistance
   mec=mecin;
   ini=iniin;
   sat=satin;
   iounits='off';  % I/O in p.u.

   xd=X(1); xpd=X(2); xsd=X(3);	% reactances
   xq=X(4); xsq=X(5); xa=X(6);
   Tpd=tau(1); Tsd=tau(2); Tsqo=tau(3); % time constants
   Ato=xd*Tpd/xpd+(xd/xsd-xd/xpd+1)*Tsd;Bto=xd*Tpd*Tsd/xsd; % (cf. Canay, 1993)
   TDO=sort(abs(roots([1 Ato Bto])));
   Tpdo=TDO(2);Tsdo=TDO(1);
   xmd=xd-xa;			% mutual reactance
   Tsq=xsq*Tsqo/xq;		% (cf. Canay, 1993)

   % ESTIMATION OF DAMPER TIME CONSTANT (cf. Adkins et Takeda, 1974)

   a=(xpd-xa)/(xd-xpd);
   b=a*xmd*(xsd-xa)/(a*xd+xa*xmd-(a+xmd)*xsd);
   c=(b+a*xmd*xa/(xa*xmd+a*xd))/Tsd;
   Tkd=b/c;

   %%%%%%%%%%%%%%%%%%% cf. Salvatore & Savino, 1983 %%%%%%%%%%%%%%%%%%%

   Pd=Tpd*Tsd; Sd=Tpd+Tsd;
   Pdo=Tpdo*Tsdo; Sdo=Tpdo+Tsdo;
   k1=(Sdo-Sd)*(Pdo*Sd-Pd*Sdo)-(Pdo-Pd)^2;
   k2=Pdo*Sd-Pd*Sdo+Tkd*(Tkd*(Sdo-Sd)-2*(Pdo-Pd));
   k3=Pdo-Tkd*(Sdo-Tkd);
   k4=Pdo-Pd-Tkd*(Sdo-Sd);
   k5=(Pdo*Sd-Pd*Sdo)-Tkd*(Pdo-Pd);

   % THREE EQUAL MUTUAL INDUCTANCES: FICTITIOUS LEAKAGE INDUCTANCE (xc)

   xmd=xd*k4/k3; Rf=(xmd^2/xd)*k2/k1;
   xc=xd-xmd;
   xkf=0;    
   Rkd=Rf*k1/k4^2;
   xf=Rf*k5/k4;
   xkd=Tkd*Rkd;
   Rf=Rf/wen;Rkd=Rkd/wen;
   xmq=xq-xc;			% leakage inductance identical for both axis
   xmqx=xmq*xc/xq;
   xkq=(xmq*Tsq-xmqx*Tsqo)/(Tsqo-Tsq); % exact value: cf. Umans et al., 1978.
   Rkq=(xkq+xmq)/(wen*Tsqo);

   % PARAMETERS FOR SYNCHRONOUS MACHINE MODEL (all values brought to
   % stator equivalents)

   Rs=ra; Ll=xc;	% stator
   Llfd=xf;		% rotor
   Llkd=xkd; Llkq=xkq;	% dampers
   Lmd=xmd; Lmq=xmq;	% mutual inductances
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data prepared for proper return format
sta=[Rs Ll Lmd Lmq];	% stator values
fld=[Rf Llfd];		% field values
dam=[Rkd Llkd Rkq Llkq];% damper values

if isempty(satin)
    params=['[',...
         num2str(nom),']|[',...
         num2str(sta),']|[',...
         num2str(fld),']|[',...
         num2str(dam),']|[',...
         num2str(mec),']|[',...
         num2str(ini),']|',...
         simsat,'|[ ]|',...
         dispvfd,'|',...
         iounits,...
       ];
else
    params=['[',...
         num2str(nom),']|[',...
         num2str(sta),']|[',...
         num2str(fld),']|[',...
         num2str(dam),']|[',...
         num2str(mec),']|[',...
         num2str(ini),']|',...
         simsat,'|[',...
         num2str(sat(1,:)),' ; ',num2str(sat(2,:)),']|',...
         dispvfd,'|',...
         iounits,...
       ];
end              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%				END OF BL25CONV
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
