function [Zc,rmode,v,Ti]=blmodlin(nphase,f,r,l,c)
%
% Computation of modal transformation for N phase line
%
% R L C parameters can be specified by NxN mtrices or by sequence parameters.
%
% Specification of sequence parameters is permitted for the following
% types of symetrical lines:
%
% - 2 phases (ex: DC bipole)  :  [R1 R0] [L1 L0] [C1 C0] 
% - 3 phases (AC line)        :  [R1 R0] [L1 L0] [C1 C0] 
% - 6 phases (2 identical coupled 3 phase AC lines) :
%                             :  [R1 R0 R0m] [L1 L0 L0m] [C1 C0 C0m]
%
% input arguments
% ---------------
% f   frequency for specification of r x b parameters (Hz)
% r   linear resistance   scalar, [1,nseq] or [N,N]  (ohm/km)
% l   linear reactance    scalar, [1,nseq] or [N,N]  (H/km)
% c   linear susceptance  scalar, [1,nseq] or [N,N]  (F/km)
%
% output arguments
% ----------------
%    Zc : vector (nmode) of caracteristic impedances for each mode (ohms)
% rmode : vector (nmode) of series resistance for each mode  (ohms/km)
%     v : vector (nmode) of propagation speeds for each mode (km/s)
%    Ti : Current tansformation matrix (nmode*nmode)
%         such that: Iphase=Ti*Imode
%

%   G. Sybille   96-03-8
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

j=sqrt(-1);
w=2*pi*f;
%
[nline,ncol]=size(r);
%
% CONVERSION OF SEQUENCE COMPONENTS TO MATRICES(nphase x nphase)
%---------------------------------------------------------------
if nline==1 & ncol==1,
% Single phase line
% -----------------
	y=j*c*w;
	z=r+j*l*w;
elseif nline==1 & (ncol==2 | ncol==3),
% Symetrical Line specified by sequence components
% Build symetrical matices z and y
% ------------------------------------------------
%   NPHASE=2 Specification of Z0 Z1
%   -------------------------------
    if nphase==2, 
		zself=(r(2)+r(1))/2 + j*(l(2)+l(1))/2*w;  
		zmut= (r(2)-r(1))/2 + j*(l(2)-l(1))/2*w;  
		yself=j*(c(2)+c(1))/2*w;  
		ymut= j*(c(2)-c(1))/2*w; 
		%
		z=zmut*ones(2,2);  z(1,1)=zself; z(2,2)=zself;
		y=ymut*ones(2,2);  y(1,1)=yself; y(2,2)=yself;
%
%   NPHASE=3 Specification of Z0 Z1
%   -------------------------------
    elseif nphase==3,
%       Specification of Z0 Z1 Z0m
		zself=(r(2)+2*r(1))/3 + j*(l(2)+2*l(1))/3*w;  
		zmut= (r(2)-r(1))/3 + j*(l(2)-l(1))/3*w;  
		yself=j*(c(2)+2*c(1))/3*w;  
		ymut=j*(c(2)-c(1))/3*w; 
		%
		z=zmut*ones(3,3);  z(1,1)=zself; z(2,2)=zself; z(3,3)=zself;
		y=ymut*ones(3,3);  y(1,1)=yself; y(2,2)=yself; y(3,3)=yself;
%
%   NPHASE=6 Specification of Z0 Z1 Z0m
%   -----------------------------------
    elseif nphase==6,
		zself=(r(2)+2*r(1))/3 + j*(l(2)+2*l(1))/3*w;  
		zmut= (r(2)-r(1))/3 + j*(l(2)-l(1))/3*w;  
		zmut12= r(3)/3 + j*l(3)/3*w;  
		yself=j*(c(2)+2*c(1))/3*w;  
		ymut=j*(c(2)-c(1))/3*w; 
		ymut12=j*c(3)/3*w; 
		%
		z=zmut*ones(6,6); 
		z(1,1)=zself; z(2,2)=zself; z(3,3)=zself;
		z(4,4)=zself; z(5,5)=zself; z(6,6)=zself;
		z(1:3,4:6)=zmut12*ones(3,3);
		z(4:6,1:3)=zmut12*ones(3,3);
		y=ymut*ones(6,6); 
		y(1,1)=yself; y(2,2)=yself; y(3,3)=yself;
		y(4,4)=yself; y(5,5)=yself; y(6,6)=yself;
		y(1:3,4:6)=ymut12*ones(3,3);
		y(4:6,1:3)=ymut12*ones(3,3);
    end
else,
	y=j*c*w;
	z=r+j*l*w;
end
%
% Find if R L C matrices are continuously transposed
% --------------------------------------------------
% A line is continuously transposed if 1) all diagonal elements are equal
% and 2) and all non-digonal elements are equal.
transpose=1; 
for i=1:nphase
	for k=1:nphase,
		if i==k,
			% check if diagonal terms are equal
			if ~(z(i,i)==z(1,1) &  y(i,i)==y(1,1)),
				transpose=0;
			end
		else
			% check if off diagonal terms are equal
			if ~(z(i,k)==z(1,2) &  y(i,k)==y(1,2)),
				transpose=0;
			end
		end
	end
end
%
% For 6-phase line, find if the mutual between circuits 1 and 2 are equal
% -----------------------------------------------------------------------
twocir_sym=0;
if nphase==6 & ~transpose,
	if all(all(z(1:3,1:3)==z(4:6,4:6))) & ... 
	all(all(y(1:3,1:3)==y(4:6,4:6))) & ...
	all(all(z(1:3,4:6)==z(4:6,1:3))) & ... 
	all(all(y(1:3,4:6)==y(4:6,1:3)))
		twocir_sym=1;
	end
end
%
%  TRANSFORMATION MATRICES
%  ------------------------
if transpose,
% continuously transposed lines
% -----------------------------
	% Previous Karrenbauer transformation  for 3 phases
	%Ti=[1  1  1  
	%	1 -2  1
	%	1  1  -2];
	% General transformation used by EMTP for Nphase:
	Ti=zeros(nphase,nphase);
	Ti(1:nphase,1)=1/sqrt(nphase)*ones(nphase,1);
	for i=2:nphase
		Ti(i,i)=-(i-1)/sqrt(i*(i-1));
		for k=2:nphase,
			Ti(1:k-1,k)=1/sqrt(k*(k-1))*ones(k-1,1);
		end
	end
	Tv=Ti;
%
elseif twocir_sym,
% 2 coupled three phase identical circuits (Ti used by EMTP)
% ----------------------------------------
	Ti=1/sqrt(6)*[1   1   sqrt(3)   1   0        0  
	              1   1  -sqrt(3)   1   0        0  
	              1   1   0        -2   0        0  
	              1  -1   0         0   sqrt(3)  1
	              1  -1   0         0  -sqrt(3)  1
	              1  -1   0         0   0       -2];
	Tv=Ti;
%
else,
% asymetrical lines
% -----------------
	yz=y*z;
	zy=z*y;
	[Ti,vp]=eig(yz);
	[Tv,vp]=eig(zy);
end
%
%  MODAL PARAMETERS
%  ----------------
zmode=inv(Tv)*z*Ti;
ymode=inv(Ti)*y*Tv;
%
rmode=real(diag(zmode))';
Zc=sqrt(imag(diag(zmode))./imag(diag(ymode)))';
v=w./sqrt(imag(diag(zmode)).*imag(diag(ymode)))';
Ti=real(Ti);
%
% Check propagation speeds
% ------------------------
n=find(v>=300000);
if ~isempty(n),
	error(sprintf('A propagation speed of %g km/s has been found for mode %d; Propagation speeds must be < 300000 km/s!',v(n(1)),n(1)));
end
%
