function [A,B,Cvn,Dvn,states,var_branche,nbvar1,rlcm,rlc1,...
         L_combi,C_combi,Il_relat,Uc_relat,nb_ldep,nb_cdep,...
         liste_neu,liste_neured,Ti,mat_tr,c_ligne]=...
  etaab(rlc,source,liste_neu,ind_neu,ind_neusrc,mat_tr,unit,f0,...
         nb_sortie,debug_etat,blocs,srcstr,BrancheREF,netsim_flag,fid)
%
%   ETAAB is a utility function used by ETATFUN.
%         It performs the following functions:
%         -  Preprocessing:  Modif. of rlc and mat_tr matrices; unit conversion;
%         -  Identification of state variables
%         -  Verification of circuit coherence
%            inductors and current source in serie; capacitors and voltage
%            sources in parallel; isolated networks; ....
%         -  Finds the relations between dependant state variables (Uc and Il) 
%         -  Computes A and B matrices such that x'=[A]x+[B].u
%         -  Computes Cn and Dn matrices relating nodes voltages
%            to states (x) and inputs (u) such that [Vn]=[Cvn]*x + [Dvn]*u  
%
%     
%  Input arguments
%  ===============
%       rlc : matrix (nbr,6) or (nbr,7) 
%             specifying the network topology and R L C values;
%    source : source matrix (nsrc,5) or (nsrc,6) 
%  liste_neu: vector of node numbers
%    ind_neu: matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
% ind_neusrc: matrix (nsrc,2) of indices of source node numbers in 'liste_neu
%     mat_tr: matrix (ntr,n) of transformers and mutually coupled windings
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
%             ....
%      unit : units used to specify R L and C values in the rlc matrix
%        f0 : frequency used to specify units in OHMS amd PU
%  nb_sortie: Number of outputs
% debug_etat: flag used for debugging; normal value=0
%     blocs : string matrix containing  names of blocks (used by NETSIM)  
%    srcstr : string matrix containing names of source blocks (used by NETSIM)
%             starting with  I_ ou V_ prefix
% BrancheREF: Integer matrix= cross reference list between NETSIM topology 
%             and rlc matrix
%netsim_flag: Integer =0 if ETATFUN is called by ETAT or from the Matlab window.
%                      1 if ETATFUN is called by NETSIM
%       fid : File identifier of output file containing detailed messages 
%
%  Output arguments
%  ================
%       A,B :  State-space matrices of the linear circuit
%              with all switches  open
%     Cn,Dn :  Observation matrices of node voltages Vn to the states
%              relating Vn to states x and inputs u:
%                        [Vn]=[Cvn]*x + [Dvn]*u
%    states :  String matrix containing the names of the state variables
%var_branche:  Matrix (nstate,2) of branch numbers of the different state
%              variables (Uc or Il) cooresponding to line number in rlc matrix.
%              var_branche(n,1)= ib =branch number of state variable n
%              var_branche(n,2)= 0  if Uc or Il is independant
%                              = iline if Uc or Il is dependant
%                                (iline = line number of Uc_relat or Il_relat)
%    nbvar1 :  number of independant state variables 
%      rlcm :  modified rlc matrix with tranformer and mutual winding branches
%              replaced by series R-L branch
%      rlc1 :  Matrix (nbr,3) of values of branch parameters
%              converted in Ohms, Henries and Farads
%   L_combi :  Vector of dependant and independant inductor
%              current state variables; The nb_ldep last values are numbers
%              of dependant currents.
%   C_combi :  Vector of dependant and independant capacitor
%              voltage state variables; The nb_cdep last values are numbers
%              of dependant voltages.
%  Il_relat :  Matrix (nb_ldep,nb_lcombi) of coefficients ki of relations
%              k1*I1+k2*I2, ...,+kn*Inb_lcombi=0 between
%              dependant and independant currents; 1 line/dependant current
%  Uc_relat  : Matrix (nb_cdep,nb_ccombi) of coefficients ki of relations
%              k1*U1+k2*U2, ...,+kn*Unb_ccombi=0 between
%              dependant and independant voltages; 1 line/dependant voltage
%    nb_ldep : Number of dependant inductor currents in vector L_combi
%    nb_cdep : Number of dependant capacitor voltages in vector C_combi
%  liste_neu : Vector of modified node numbers (including intermediate
%              node numbers of transformers and mutuals)
%liste_neured: Node numbers of reduced network with voltage sources
%              and dependant inductors short-circuited.
%              These node numbers correspond to the node voltages Vn
%              calculated by  [Vn]=[Cvn]*x + [Dvn]*u
%        Ti  : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
%     mat_tr : Modified matrix (ntr,n) of transformers
%              and mutually coupled windings
%              Mutuals: Self and mutual resist.replaced by winding resist. Rs-Rm
%              Modified resistance distribution if secondary winding leakage 
%              reactance are zero.
%    c_ligne : Shunt capacitance (Farads) at both end of each
%              eqivalent pi section of transmission lines 

%              G. Sybille April 1995 ; 
%                 Revised Feb.25 1997
%===============================================================================
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  types of sources     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The 'strtype' string matrix is used yo display NETSIM error messages
% related to improper use of non linear elements

% 97-02-25 Modification of numbering of non linear elements

srctype=[
'voltage source                                              ';  %1
'current source                                              ';  %2
];
srctype(8:14,:)=[
'breaker (modelled as a current source)                      ';  %8
'surge arrester (modelled as a current source)               ';  %9
'DC voltage source                                           ';  %10
'controlled voltage source                                   ';  %11
'controlled current source                                   ';  %12
'                                                            ';  %13
'transformer saturable reactor (modelled as a current source)';  %14
];
srctype(21:25,:)=[
'diode (modelled as a current source)                        ';  %21
'thyristor (modelled as a current source)                    ';  %22
'GTO thyristor (modelled as a current source)                ';  %23
'MOSFET (modelled as a current source)                       ';  %24
'Ideal switch (modelled as a current source)                 ';  %25
];
srctype(31:35,:)=[
'simplified alternator stator (modelled as 2 current sources)'; %31
'synchronous machine stator (modelled as a current sources)  '; %32
'asynchronous machine stator (modelled as 2 current sources) '; %33
'asynchronous machine rotor (modelled as 2 current sources)  '; %34
'permnent. magnet motor (modelled as 2 current sources)      '; %35
];

%   Preprocessing of input data 
%   ============================

omega=2*pi*f0;
rmin=1e-7;
%
nbneu=length(liste_neu);
[nbr,n]=size(rlc);
[nbr_src,n]=size(source);
[ntr,n]=size(mat_tr);
%
% Form original  Ti1 matrix  corresponding to rlc
% before modifications of transformer branches
nbneu1=nbneu; ind_neu1=ind_neu;
Ti1=[];
for ib=1:nbr
	i1=ind_neu1(ib,1); i2=ind_neu1(ib,2);
	if i1<nbneu1, Ti1(i1,ib)=1;  end
	if i2<nbneu1, Ti1(i2,ib)=-1; end
end
%
% Processing of transformers and mutuals
% Modify rlc matrix-->rlcm; and matrix mat_tr
%============================================
rlcm=rlc;
%mat_tr: matrix (ntr,n) of transformers
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
%             .....                                                            
%
for itr=1:ntr
	n=mat_tr(itr,2); % number of windings
	ib=mat_tr(itr,1);
	ibmag=ib+n;  % Magnetisation or mutual branch number
	% modiy rlcm, liste_neu and ind_neu
	for i=1:n,
		ib1=ib+i-1;
		%
		%  Mutuelles
		%  ---------
		if rlcm(ib1,3)==3,
%			Rself-Rmag; Xleakage=Xself-Xmag;
			rlcm(ib1,4)=rlcm(ib1,4)-rlcm(ibmag,4); 
			rlcm(ib1,5)=rlcm(ib1,5)-rlcm(ibmag,5); 
			mat_tr(itr,2+3*i)=rlcm(ib1,4);
		end
		%
		if i>1,
			neu_tr=rlc(ib+n,1)+0.1*i;
			if all(liste_neu~=neu_tr),
			%Create additional node for leakage reactance
				liste_neu=[liste_neu,neu_tr];
				nbneu=nbneu+1;
			else
				error(sprintf(...
				['The intermediate node %g of winding no %g',...
				' of transformer or mutual no %g is already existing!'],...
				neu_tr,n,ntr));
			end
			% 
			if rlcm(ib1,5)==0,
				% If the leakage reactance of secondary winding is zero,
				% share its resistance  50% with ideal transfo 
				% 50% with external leakage reactance 
				rlcm(ib1,4)=rlcm(ib1,4)/2;
				mat_tr(itr,8+(i-2)*3)=rlcm(ib1,4);
			else,
				rlcm(ib1,4)=0;
			end
			ind_neu(ib1,2)=nbneu;
		else,
			neu_tr=rlc(ib+n,1);
			ind_neu(ib1,2)=ind_neu1(ib+n,1);
		end
		rlcm(ib1,2)=neu_tr; rlcm(ib1,3)=0; rlcm(ib1,6)=0;
	end
end
%
%  Unit conversion; and transmission line processing
%  =================================================
%
rlc1=zeros(nbr,3); % Matrix of branch parameters R L C (Ohms , Henrys Farads)
%
c_ligne=[];
for ib=1:nbr,
	i1=ind_neu(ib,1); i2=ind_neu(ib,2);
	%
	% Conversion to ohms Henrys Farads ---> rlc1
	if all(unit=='OMU'),
		R=rlcm(ib,4);
		L=rlcm(ib,5)/1000;
		C=rlcm(ib,6)*1e-6;
	elseif all(unit=='OHM'),
		R=rlcm(ib,4);
		L=rlcm(ib,5)/omega;
		if rlcm(ib,6)~=0, C=1/abs(rlcm(ib,6))/omega; else, C=0; end
	elseif all(unit=='P.U')
		error ('Units cannot be specified in  PU with ETAT');
	end
	%
	C=C+rlc1(ib,3);  % add line capacitances 
	rlc1(ib,1)=R; rlc1(ib,2)=L; rlc1(ib,3)=C;
	if R<1e-6 & L==0 & C==0,
		error(sprintf(...
		['Branch %g ; Resistance=%g ohms; For a purely resistive ',...
		'branch, the minimun resistance allowed is 1e-6 ohms'],ib,R));
	end
%
	if rlcm(ib,3)<0,
%	Form capacitive branches of transmission lines
%   ----------------------------------------------
		long=abs(rlcm(ib,3));
		xl=L*omega; susc=C*omega;
		[z_ser,y_sh] =  etazline(long,R,xl,susc);
		rlc1(ib,1)=real(z_ser);         %  resistance of pi equivalent
		rlc1(ib,2)=imag(z_ser)/omega;   %  inductance of pi equivalent
		rlc1(ib,3)=0;
		c_ligne(ib)=imag(y_sh)/omega;   % C connected at each end of pi   
		for icol=9:10,
			ib1=rlcm(ib,icol);
			rlc1(ib1,3)=rlc1(ib1,3)+c_ligne(ib);
		end
	end
end
%
%
%
% Identify state variables
% ========================
%    states = string matrix of the nbvar state variables d'etat Il_... or Uc_...
%var_branche:  Matrix (nstate,2) of branch numbers of the different state
%              varaibles (Uc or Il) cooresponding to line number in rlc matrix.
%              var_branche(n,1)= ib =branch number of state variable n
%              var_branche(n,2)= 0  if Uc or Il is independant
%                              = iline if Uc or Il is dependant
%                                (iline = line number of Uc_relat or Il_relat)  
%
clear states var_branche ;
blanc='                    ';
nbvar=0;
for ib=1:nbr,
	if rlc1(ib,2) ~=0,
		nbvar=nbvar+1;
		str=sprintf('Il_b%g_n%g_%g',rlcm(ib,7),rlcm(ib,1),rlcm(ib,2));
		states(nbvar,:)=[str,blanc(1:20-length(str))];
		var_branche(nbvar,1)=rlcm(ib,7);
		var_branche(nbvar,2)=0;
	end
	if rlc1(ib,3) ~=0,
		nbvar=nbvar+1;
		str=sprintf('Uc_b%g_n%g_%g',rlcm(ib,7),rlcm(ib,1),rlcm(ib,2));
		states(nbvar,:)=[str,blanc(1:20-length(str))];
		var_branche(nbvar,1)=rlcm(ib,7);
		var_branche(nbvar,2)=0;
	end
end
fprintf(fid,'Total number of inductances and capacitors: %g\n',nbvar);
%
%  Form admittance matrices Y and Y0 of resistive branches
%  =======================================================
Y=zeros(nbneu);
Y0=Y; 
Zb=zeros(1,nbr);
%
%  Y= Matrice d'admittances des branches resistives
%    -Les branches serie contenant une inductance et les branches
%     parallele contenant seulement une inductance sont ouvertes (y=0)
%    -Les branches capacitives pures ou RLC parallele et les sources de tension
%     sont remplacees par une admittance y=1/rmin indiquant un court-circuit.
%    - Y est utilisee pour trouver les branches L et C independantes
%      et pour detecter les reseaux isoles
%
%  Y0=matrice d'admittance semblable a Y mais sans les branches court-cicuitees
%     (branches purement capacitives et sources de tension)
%     Y0 est utilisee pour calculer les matrices Yred et Z du reseau reduit
%     (avec branches court-cicuitees eliminees)
%
%
for ib=1:nbr
	i1=ind_neu(ib,1); i2=ind_neu(ib,2);
	R=rlc1(ib,1); L=rlc1(ib,2); C=rlc1(ib,3);
	yy=0;
%
	if rlcm(ib,3)<=0,
%   Series branch or series branch of line pi circuit
%   -------------------------------------------------
		if L==0,
			if R==0; yy=1/rmin; else, yy=1/R; end
		end
%
	elseif rlcm(ib,3)==1,
%   Parallel branch   
%   --------------
		if C~=0; yy=1/rmin; end
		if R~=0; yy=yy+1/R; end
	end
%
	if abs(Y(i1,i2)) >= 1/rmin & yy>=1/rmin ,
		if C==0,
			error(sprintf(['Short-circuit between nodes %g and %g',...
			'in parallel with a capacitor or another short-circuit'],...
			rlcm(ib,1),rlcm(ib,2)));
		end;
	end;
	% Capacitor in parallel with a short-circuit
	if yy==1/rmin & C~=0 & i1==i2 ,
			error(sprintf('The capacitor of branch %g is short-circuited ',ib));
	end;
	%
	Y(i1,i1)=Y(i1,i1)+yy; Y(i2,i2)=Y(i2,i2)+yy;
	Y(i1,i2)=Y(i1,i2)-yy; Y(i2,i1)=Y(i1,i2);
	if yy<1/rmin & i1~=i2, 
		Y0(i1,i1)=Y0(i1,i1)+yy; Y0(i2,i2)=Y0(i2,i2)+yy;
		Y0(i1,i2)=Y0(i1,i2)-yy; Y0(i2,i1)=Y0(i1,i2);
	end
	%
	% Form transformation matrices of currents and voltages Ti and Tu 
	% and impedance matrix Zb of resistive branches Zb such that:  
	%------------------------------------------------------------------
	%                     [Vb]=[Tu].[Vn]  
	%                     [In]=[Ti].[Ib]
	%                     [Vb]=[Zb].[Ib]
	%             where Vb Ib = branch voltages and currents    
	%                   Vn In = node voltages and currents     
	% Do not consider short-circuited blocks
	if i1~=i2,
	if i1<nbneu, Ti(i1,ib)=1;  Tu(ib,i1)=1 ; end
	if i2<nbneu, Ti(i2,ib)=-1; Tu(ib,i2)=-1; end
	if yy~=0, Zb(ib)=1/yy; end
	end            
end
%
% Form  Y_cc matrix  and list of purely capacitive branches nobr_cc
% -----------------------------------------------------------------
% used to find  independant capacitor voltages
% Admittance of a capacitive branch = 1.  Y_cc=integer matrix;             
% Y_cc(i,j)=n indicates cap. branch in parallel  between nodes i and j
% Y_cc(i,i)=-k indicates cap. branch connected to node i 
Y_cc=fix(rmin*(Y.*(abs(Y)>=1/rmin)));
if nbr>0,
	nobr_cc=(find((rlcm(:,3)==0 & rlc1(:,1)==0 &...
			 rlc1(:,2)==0 & rlc1(:,3)~=0) |  (rlcm(:,3)==1 & rlc1(:,3)~=0)))';
else
	nobr_cc=[];
end
%
% Replace voltage source branches with short-circuit          
% and detect voltage sources in parallel with another cap. or short-circuit
%  -------------------------------------------------------------------------
yy=1/rmin;
nbr_Ti=nbr;
nobr_srcc=[];
for ib=1:nbr_src,
	if source(ib,3)==0,
		i1=ind_neusrc(ib,1); i2=ind_neusrc(ib,2);
		Y(i1,i1)=Y(i1,i1)+yy; Y(i2,i2)=Y(i2,i2)+yy;
		if abs(Y(i1,i2)) >= 1/rmin & yy>=1/rmin,
		    if netsim_flag==1
			mess=sprintf('FATAL!  The %s named: "%s"  ',...
			deblank(srctype(source(ib,7),:)),deblank(srcstr(ib,3:size(srcstr,2))) ); 
			mess=str2mat(mess,...
			'                is in parallel with a capacitor or a short-circuit'); 
			eval('nserror(mess,[],2);');
		    else
			error(sprintf(['voltage source %g-%g in parallel with ',...
			'a capacitor or a short-circuit'],source(ib,1),source(ib,2)));
		    end
		end;
		Y(i1,i2)=Y(i1,i2)-yy; Y(i2,i1)=Y(i1,i2);
		nbr_Ti=nbr_Ti+1;
		Zb(nbr_Ti)=rmin;
		if i1<nbneu, Ti(i1,nbr_Ti)=1; Tu(nbr_Ti,i1)=1;end
		if i2<nbneu, Ti(i2,nbr_Ti)=-1; Tu(nbr_Ti,i2)=-1;end
		%
		if i1<nbneu1, Ti1(i1,nbr_Ti)=1; end
		if i2<nbneu1, Ti1(i2,nbr_Ti)=-1; end
		nobr_srcc=[nobr_srcc,ib]; % no of voltage sources to short-circuit
	end
end
%
Y0=sparse(Y0);
%
%  Contribution of transformers to Y0 and Ti matrices
%  Modify Y and Ti--> Ti2 (includes contribution to transformer nodes)
%  ------------------------------------------------------------------
 	[Y0,Y,Ti2]=etayttr(Y0,Y,Ti,mat_tr,liste_neu,ind_neu);
%
Tis=sparse(Ti);
%
% Complete  matrix Tu: Consider secondary windings of transformers
% ================================================================
%  Tu matrix is used  by Rpar etavn to calculate  node voltages         
% (Add as many lines as secondary windings)
%
[nbr_Tu,n]=size(Tu);
for itr=1:ntr
	ib1=mat_tr(itr,1); % branch no of 1st winding     
	n=mat_tr(itr,2);   % number of windings   
	for iwind=2:n,
		nbr_Tu=nbr_Tu+1;
		i1=ind_neu(ib1+iwind-1,2); i2=ind_neu1(ib1+iwind-1,2);
		if i1<nbneu, Tu(nbr_Tu,i1)=1; end
		if i2<nbneu, Tu(nbr_Tu,i2)=-1; end
	end
end
%
Tu=sparse(Tu);
%
% Detect isolated networks;
% -------------------------
[nores,nbres]=etaisole(Y);
%
% Detect current sources in series with inductors or isolated networks
%---------------------------------------------------------------------
for ib=1:nbr_src,
    if source(ib,3)==1,  
        i1=ind_neusrc(ib,1); i2=ind_neusrc(ib,2);
        if nores(i1)~=nores(i2)
		if netsim_flag==1
			mess=sprintf('FATAL! The %s named: "%s"  ',...
			deblank(srctype(source(ib,7),:)),deblank( srcstr(ib,3:size(srcstr,2)) ) );
            		mess=str2mat(mess,...
					'               is in series with an inductance, an open circuit or another current source');
			eval('nserror(mess,[],3);');
		else
			error(sprintf(['Current source %g-%g is in series with',...
            'an inductor, an open circuit or another current source'],...
			source(ib,1),source(ib,2)));
		end
        end
    end
end
%
% Relations between dependant and independant capacitor voltages
% --------------------------------------------------------------
% Uc_relat contains  nb_cdep relations f(U1,U2, ...,Unb_ccombi)=0
[Y,C_combi,nb_cdep,Zb,nobr_z,Uc_relat]=...
	etaucr(Y,Y_cc,Zb,rlcm,rlc1,Tis,nobr_cc,ind_neu,rmin,netsim_flag,fid); 
nb_ccombi=length(C_combi);
nb_cind=nb_ccombi-nb_cdep;
%
% Relations between dependant and independant inductor currents
% -------------------------------------------------------------
% Il_relat contains nb_ldep relations f(I1,I2, ...,Inb_lcombi)=0
[Y,L_combi,nb_ldep,Zb,nobr_z,Il_relat]=...
	etailr( Y,Zb,rlcm,rlc1,Ti1,ind_neu1,rmin,mat_tr,netsim_flag,fid);
nb_lcombi=length(L_combi);
nb_lind=nb_lcombi-nb_ldep;
%
% Form  Ti3 matrix = Ti1 original modified
%      et  ind_neu3   = ind_neu1 original modified
% (mag. branch and primary leakage connected to intermediate node)
% ---------------------------------------------------------------
%  (matrices used by etaelc to determine voltages induced
%  across dependant and independant L)
Ti3=Ti1;
ind_neu3=ind_neu1;
for itr=1:ntr
	ib=mat_tr(itr,1); % primary branch
	i2=ind_neu1(ib,2);
	if i2<nbneu1, Ti3(i2,ib)=0; end
	% connect output of primary leakage to intermediate node
	i2=ind_neu(ib,2);
	if i2<nbneu1, Ti3(i2,ib)=-1;end
	ind_neu3(ib,2)=i2;
end
%
% Update the list of  branches to be short-circuited
% and modify  Y0 matrix     
% ---------------------------------------------------
% open  dependant C branches
% 
for i=nb_cind+1:nb_ccombi,
	ib=C_combi(i);
	nobr_cc=nobr_cc(find(nobr_cc~=ib));
	% introduce parallel resistance
	if rlcm(ib,3)==1 & rlcm(ib,4)~=0,
		yy=1/rlc1(ib,1);
		i1=ind_neu(ib,1); i2=ind_neu(ib,2);
		Y0(i1,i1)=Y0(i1,i1)+yy; Y0(i2,i2)=Y0(i2,i2)+yy;
		Y0(i1,i2)=Y0(i1,i2)-yy; Y0(i2,i1)=Y0(i1,i2);
	end
end
% Short-circuit purely inductive L dependant branches
% ---------------------------------------------------
for i=nb_lind+1:nb_lcombi,
	ib=L_combi(i);
	if  rlcm(ib,4)==0 & rlcm(ib,6)==0, % purely inductive branch
		nobr_cc=[nobr_cc,ib];
	elseif rlcm(ib,3)==0 & rlcm(ib,4)~=0, 
	% introduce  series resistance
		yy=1/rlc1(ib,1);
		i1=ind_neu(ib,1); i2=ind_neu(ib,2);
		Y0(i1,i1)=Y0(i1,i1)+yy; Y0(i2,i2)=Y0(i2,i2)+yy;
		Y0(i1,i2)=Y0(i1,i2)-yy; Y0(i2,i1)=Y0(i1,i2);
	end
end
%
if nbr>0,
	Zb(1:nbr)=rlc1(:,1);
else
	Zb=[];
end
%
% Check for isolated networks after elimination of dependant branches
% -------------------------------------------------------------------
%[nores,nbres]=etaisole(Y(1:nbneu1,1:nbneu1));
[nores,nbres]=etaisole(Y);
%
 
if nbres>1
   	if netsim_flag==1

		mess=['FATAL!: ',num2str(nbres),...
		' isolated networks have been detected: '];
    		for ires=1:nbres
			mess=str2mat(mess,'',[' subnetwork #' num2str(ires) ':']); 
			noeuds=liste_neu(find(nores==ires));
			if length(find(noeuds==0))>0
				mess=str2mat(mess ,'Ground'); 
			end
			noeuds=noeuds(find(noeuds>0));
			rlclist=[];
			sourcelist=[];
 
			for i=1:length(noeuds)
				for j=1:size(rlc,1)
					if rlc(j,1)==noeuds(i) |  rlc(j,2)==noeuds(i)
						if find(rlclist==j)==[]
							mess=str2mat(mess ,...
							deblank(blocs(BrancheREF(j,1),:)) ); 
							blocs(BrancheREF(j,1),:);
						end
						rlclist=[rlclist j];
					end
 				end
				for j=1:size(source,1)
					if source(j,1)==noeuds(i) | source(j,2)==noeuds(i)
						if find(sourcelist==j)==[]
							mess=str2mat(mess ,...
							deblank(srcstr(j,3:size(srcstr,2))) );
							srcstr(j,:);
						end
						sourcelist=[sourcelist j];
					end
				end
			end
    		end

   		mess=str2mat(mess ,'  ');

   		eval('nserror(mess,[],5);');
	else
		fprintf('%g isolated networks have been detected:\n',nbres);
		for ires=1:nbres,
			fprintf('reseau %g:\n',ires);
			noeuds=liste_neu(find(nores==ires))
		end
		error(sprintf('THe %g above networks are isolated',nbres));
	end
end
%
% Detect voltage sources in parallel with capacitive circuit
% ==========================================================
%
[Yred,ind_neured,ind_neusrcred,liste_neured,nbneured,...
nobr_zred,nobr_zcc,noneured,Tired,Tured]= etayred(0,'I',Y0,...
nobr_cc,ind_neu,[],ind_neusrc,rlcm,source,liste_neu,nobr_z,Tis);
%
for ib=1:nbr_src,
    i1=ind_neusrcred(ib,1); i2=ind_neusrcred(ib,2);
        if source(ib,3)==0 & i1==i2,
		if netsim_flag==1
			mess=sprintf('FATAL! The %s named: "%s"   ',...
			deblank(srctype(source(ib,7),:)),deblank( srcstr(ib,3:size(srcstr,2))));
			mess=str2mat(mess,'               is in parallel with a group of capacitors and/or voltage sources') ; 
			eval('nserror(mess,[],4);');
		else
            		error(sprintf(...
					['Voltage source %g-%g in parallel with a ',...
            		'capacitive circuit and/or voltage sources'],...
            		source(ib,1),source(ib,2)));
		end
        end;
end;
%
% Sort states and var_branche according column 2 of var_branche in order to
% the nb_ldep L and nb_cdep C in the last rows
% --------------------------------------------------------------------------
for i=1:nb_cdep,
	ib=C_combi(nb_ccombi-nb_cdep+i);
	ivar=find(var_branche(:,1)==ib & states(:,1)=='U');
	var_branche(ivar,2)=i;
end
for i=1:nb_ldep,
	ib=L_combi(nb_lcombi-nb_ldep+i);
	ivar=find(var_branche(:,1)==ib & states(:,1)=='I');
	var_branche(ivar,2)=i;
end
	%
if nbvar>0,
	[n,indice]=sort(abs(var_branche(:,2)));
	var_branche=var_branche(indice,:);
	states=states(indice,:);
else
	states=[];
	var_branche=[];
end
%
nbvar1=nbvar-nb_ldep-nb_cdep;
if netsim_flag <=1,
fprintf(...
	'Computing state-space representation of linear electrical circuit...\n');
	fprintf('(%g states ; %g inputs ; %g outputs) \n',nbvar1,nbr_src,nb_sortie);
end
%
%
%
%  Build evolution matrix  [A] (dimension nbvar1 x nbvar1)
%  ========================================================
%
%   To compute term Alk of line l,  column k of matrix [A]:
%
% 1) Modify matrix  Y  Rfor the following two cases:
%    --------------------------------------------------
%  - Variable = Uc in a series branch containing no inductance ,
%  - Variable = Uc in a parallel branch
%
% 2) Build matrix  [Z] of the modified resistive network
%    --------------------------------------------------
%    for each term Alk, Form the Z matrix of the resistive network
%    by inverting the admittance matrix  --> Z=inv(Y)
%
% 3) compute diagonal elements Akk = A(k,k):
%    ------------------------------------------
%    Replace state variable  k by a 1volt voltage source if variable=Uc..
%                              or by a current source of 1A if variable=Il..
%
%    Calculate currents Is entering node  j et and leaving node i
%    of source branch ij (Node i= input ; Node j = output)
%               Is=1A          if source = inductance
%               Is=1V/(Zij +R) if source = capacitor voltage   
%                           where Zij = impedance between nodes i and j
%                                 R   = resistance of source branch    
%
%         Akk= -Ic/C ou  Ic= Is=1/(Zi_j+R)
%     ou  Akk= -Ul/L ou  Ul= Uj_i + R*1A = -Ui_j +R    (Ui_j= Ui - Uj)
%
%         Zi_j = Zii + Zjj -2Zij  (Impedance between   i and j)
%  Uj_i= Uj-Ui = Zii + Zjj -2Zij  (voltage between nodes  i and j for 
%                                  a current is injected into node j
%                                  and leaving node i)
%
% 4)   Compute off-diagonal elements Alk= A(l,k):
%      ----------------------------------------------
%    Calculate voltages Um_n=Un-Um appearing  across branch mn
%    containing state variable l (Node m= input; Node n =output)
%
%         Alk= -Ic/C ou  Ic= Um_n/(Zm_n+R)  ou      Um_n = Un - Um
%     ou  Alk= -Ul/L ou  Ul= Um_n =Un - Um
%
%          Zm_n = Zmm + Znn -2Zmn          (Impedance between   m and n)
%   Um_n= Un-Um =(Zmi - Zmj + Znj -Zni)is  (Voltage beteewn m et n for 
%                                          a  current is injected into node j
%                                          and leaving node i)
A=[]; Cvn=[];
el=[];
%
for icol=1:nbvar1,
	% Diagonal elements
	% =================
	ib=var_branche(icol,1);
	R= rlc1(ib,1); L= rlc1(ib,2); C= rlc1(ib,3);
	if states(icol,1)=='U', type='C'; else, type='L'; end
	%
	%
	[Yred,ind_neured,ind_neusrcred,liste_neured,nbneured,...
	nobr_zred,nobr_zcc,noneured,Tired,Tured]= etayred(ib,type,Y0,...
	nobr_cc,ind_neu,nobr_srcc,ind_neusrc,rlcm,source,liste_neu,nobr_z,Tis);
%
	i1=ind_neured(ib,1); i2=ind_neured(ib,2);
	neu1=ind_neu(ib,1); neu2=ind_neu(ib,2);
	Yisol=Y;
	%
	% Modify admittance matrix Y and obtain matrix Z of resistive network 
	% -------------------------------------------------------------------
	if states(icol,1)=='U',
		% eliminate short-circuit due to  purely C branch in matrix  Yisol
        if(rlcm(ib,3)==0 & R==0 & L==0 & C~=0) | (rlcm(ib,3)==1 & C~=0),
			yy=1/rmin;
			Yisol(neu1,neu1)=Yisol(neu1,neu1)-yy;
			Yisol(neu2,neu2)=Yisol(neu2,neu2)-yy;
			Yisol(neu1,neu2)=Yisol(neu1,neu2)+yy;
			Yisol(neu2,neu1)=Yisol(neu1,neu2);
		end
		if R~=0 & rlcm(ib,3)<=0 & L==0,
			yy=1/R; % eliminate admittance of series branch 
		elseif R~=0 & rlcm(ib,3)==1 & C~=0,
			yy=-1/R; % add admittance of aparallel branch 
		else,
			yy=0;
		end
		Yred(i1,i1)=(Yred(i1,i1)-yy); Yred(i2,i2)=(Yred(i2,i2)-yy);
		Yred(i1,i2)=(Yred(i1,i2)+yy); Yred(i2,i1)=Yred(i1,i2);
		%
		Yisol(neu1,neu1)=Yisol(neu1,neu1)-yy;
		Yisol(neu2,neu2)=Yisol(neu2,neu2)-yy;
		Yisol(neu1,neu2)=Yisol(neu1,neu2)+yy;
		Yisol(neu2,neu1)=Yisol(neu1,neu2);
	end
	if rlcm(ib,3)==1, R=0; end
%
	% Check network topology (isolated networks)
	% ------------------------------------------
	%[nores,nbres]=etaisole(Yred);
	[nores,nbres]=etaisole(Yisol); nores=nores(noneured);
	Z=zeros(nbneured);
	if nbres==1 & nbneured>1,
		% eliminate last node in order  to get non singular matrix
		Z(1:nbneured-1,1:nbneured-1)=inv(Yred(1:nbneured-1,1:nbneured-1));
	elseif nbres==2,
		% 2 sub-networks
		% ---------------
		% 2 terminals of the source coonected to the same sub-network
		if nores(i1)==nores(i2), 
            		fprintf(sprintf(...
			['The following nework was found to be isolated when ',...
			'computing column %g of matrix A ',states(icol,:)],icol));
			noeuds=liste_neured(find(nores~=nores(i1)));
			if netsim_flag==1
				mess=str2mat('FATAL! The following network was found to be isolated when computing the A matrix:');
				noeuds=noeuds(find(noeuds>0));
				rlclist=[];
				sourcelist=[];
 
				for i=1:length(noeuds)
					for j=1:size(rlc,1)
						if rlc(j,1)==noeuds(i) |  rlc(j,2)==noeuds(i)
							if find(rlclist==j)==[]
								mess=str2mat(mess ,...
								deblank(blocs(BrancheREF(j,1),:)) );
								blocs(BrancheREF(j,1),:);
							end
						rlclist=[rlclist j];
						end
 					end
					for j=1:size(source,1)
						if source(j,1)==noeuds(i) | source(j,2)==noeuds(i)
							if find(sourcelist==j)==[]
								mess=str2mat(mess,...
								deblank(srcstr(j,3:size(srcstr,2))) );
								srcstr(j,:);
							end
						sourcelist=[sourcelist j];
						end
					end
				end
				eval('nserror(mess,[],6);');
			else
				error([]);
			end
		end
	elseif nbneured>1
		if netsim_flag==1
			mess=['FATAL! Networks were found to be isolated when computing '];
   			mess=str2mat(mess ,[' the column %g og A matrix ',num2str(icol)]);
			eval('nserror(mess,[],7);');
		else

			error(sprintf (['%g Isolated networks when computing ',...
			'column %g of  matrix A\n'],nbres,icol));
		end
 	end
	%
	% Off-diagonal elements A(icol,icol)
	% ==================================
	if states(icol,1)=='U' , % Source=CAPACITOR   
	%==========================================
		if nbres==2 &  nores(i1)~=nores(i2),
		% source Uc connected between 2 isolated sub-networks
			is=0;
		else,
			if rlcm(ib,3)<=0 & L~=0,
				is=0;
			else,
				zz=Z(i1,i1)+ Z(i2,i2)- 2*Z(i1,i2)+ R;
				if zz~=0,
					is=1/zz;
				else,
					if netsim_flag==1
						mess=['FATAL: Capacitive network in parallel with voltage source(s)'];
						eval('nserror(mess,[],8);');
					else

						% Error if capacitive circuit in parallel with 
						% a voltage source 
						% ----------------------------------------------
				   		[Ib]=etaibra('C',1,ib,ib,ind_neu,ind_neured,...
				                     Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc);
						n=find(abs(Ib)>0.99); n=n(find(n<=nbr));
						str=['The capacitive circuit containing branches : ',int2str(ib)];
						for i=1:length(n), str=[str,' ',int2str(n(i))]; end
						str=[str,' is in parallel with voltage source(s)'];
						error(str);
					end
 				end
			end
		end
		%  Calculate branch currents  [Ib]
		%  -------------------------------
		[Ib]=etaibra('C',is,ib,ib,ind_neu,ind_neured,...
		                  Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc);
		%
		% Calculate voltages induced in transformers                 
		% ------------------------------------------
		[Vtr]=etavtr(ib,rlc,mat_tr,Ib);
		%
		if debug_etat,
			fprintf('Branche source: %g\n',ib);
			for i=1:length(Ib), fprintf('Ib%g=%g\n',i,Ib(i)); end
			for itr=1:ntr,
				fprintf('Tr%g :',itr);
				for i=1:mat_tr(itr,2), fprintf('V%g=%g ',i,Vtr(itr,i)); end
				fprintf('\n');
			end
		end
		%
		% Calculate voltages induced across inductances of L_combi and
		% currents flowing in capacitors of  C_combi
		% ------------------------------------------------------------
		if nb_lcombi,
			[el,Vtr]=etaelc(ib,i1,i2,'C',is,nores,L_combi,Il_relat,nb_ldep,...
				  ind_neu3,rlcm,rlc1, Ti3,nobr_z,nobr_zred,Tured,Z,Zb,...
				  mat_tr,Vtr,debug_etat);
		end
		%
		if nb_ccombi,
			ic_comb=etaicc(ib,neu1,neu2,'C',is,nores,C_combi,Uc_relat,...
			        nb_cdep,ind_neu,rlcm,rlc1,Ti2,nobr_z,Ib,mat_tr,debug_etat);
			end
		%
		n=[];
		if length(C_combi)>0, n=find(C_combi==ib); end
		if length(n)>0,
			A(icol,icol)=-ic_comb(n)/C;
		else
			A(icol,icol)=-is/C;
		end
		Usource=1;  %capacitor voltage replaced by a 1Volt source
		Isource=is; %source current 
	%
	elseif states(icol,1)=='I', %Source=inductance
	%==============================================
		if nbres==2 &  nores(i1)~=nores(i2) 
			
			if netsim_flag==1
				mess=['FATAL: The inductance ' deblank(blocs(BrancheREF(ib,1),:)) ];
				mess=str2mat(mess ,'              is in series with another inductance or a current source');
				eval('nserror(mess,[],10);');
			else

				error(sprintf(['Inductance %g-%g in series with another '...
				' inductance or a current source'],rlcm(ib,1),rlcm(ib,2)));
			end
 		else,
			is=1;
			%  Calculate branch currents [Ib]
			%  ------------------------------
			[Ib]=etaibra('L',is,ib,ib,ind_neu,ind_neured,...
			                   Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc);
			%
			% Calculate voltages induced in transformers
			% -------------------------------------------
			[Vtr]=etavtr(ib,rlc,mat_tr,Ib);
			if debug_etat,
				fprintf('Branche source: %g\n',ib);
				for i=1:length(Ib), fprintf('Ib%g=%g\n',i,Ib(i)); end
				for itr=1:ntr,
					fprintf('Tr%g :',itr);
					for i=1:mat_tr(itr,2), fprintf('V%g=%g ',i,Vtr(itr,i)); end
					fprintf('\n');
				end
			end
			%
			% Calculate voltages induced across inductances of L_combi and
			% currents flowing in capacitors of  C_combi
			% ---------------------------------------------------------
			n=[]; el=[];
			if nb_lcombi,
			[el,Vtr]=etaelc(ib,i1,i2,'L',is,nores,L_combi,Il_relat,nb_ldep,...
					ind_neu3,rlcm,rlc1,Ti3,nobr_z,nobr_zred,Tured,Z,Zb,...
					mat_tr,Vtr,debug_etat);
				n=find(L_combi==ib);
			end
			if nb_ccombi,
			ic_comb=etaicc(ib,neu1,neu2,'L',is,nores,C_combi,Uc_relat,...
			        nb_cdep,ind_neu,rlcm,rlc1,Ti2,nobr_z,Ib,mat_tr,debug_etat);
			end
			%
			if length(n)>0,
				Ul=el(n);
			else,
				Ul=Z(i1,i1)+ Z(i2,i2)- 2*Z(i1,i2)+ R;
			end
		end
		A(icol,icol)=-Ul/L;
		Usource=Ul; %voltage across inductor replaced with a 1A current source
		Isource=1 ; % source current
		if abs(Usource)<=abs(1.01*rmin*Isource), A(icol,icol)=0; end 
	end
	%
	if debug_etat ,
		fprintf(sprintf('A(%g,%g)=%g\n',icol,icol,A(icol,icol)));
		pause
	end
	%
	% Compute column icol of Cvn such that
	% ====================================
	%
	%      [Vn]=[Cvn]*x + [Dvn]*u
	%
	%   ou [Vn]= vector of node voltages
	%        x = vecteur of states  
	%        u = vecteur of inputs
	%
	if type=='L',
		is1=is; el1=-el; Vtr1=Vtr;
	else,
		is1=-is; el1=el; Vtr1=-Vtr;
	end
	[Vn]=  etavn(ib,type,is1,i1,i2,Z,rlc1,source,liste_neu,ind_neu,...
		ind_neusrc,liste_neured,ind_neured,ind_neusrcred,noneured,...
		nbres,nores,el1,Tu,L_combi,nb_ldep,mat_tr,Vtr1);
	%
	if debug_etat,
		for i=1:nbneu,
			 fprintf('V%-4g %g\n',i,Vn(i));
		end
	end
	Cvn(1:nbneu-1,icol)=Vn(1:nbneu-1);
	%
	% Off-diagonal elements of A
	% ==========================
	for ivar=1:nbvar1,
		if ivar~=icol,
			ib=var_branche(ivar,1);
			j1=ind_neured(ib,1); j2=ind_neured(ib,2);
			R= rlc1(ib,1); L= rlc1(ib,2); C= rlc1(ib,3);
			if rlcm(ib,3)==1 | R==0, R=rmin; end
			%
			% Compute off-diagoanl elements of A(ivar,icol)
			% ---------------------------------------------
			%
			% Voltage Uj2_1 = Uj2 - Uj1 voltage
			% between nodes j1 and j2 for current is entering
			% into node i2  and leaving node i1 
			% (current i1-->i2 in the source)
			%
			Uj2_1=0;
			if nbres==1& j1~=j2,
				Uj2_1=(Z(j1,i1) - Z(j1,i2) +Z(j2,i2) - Z(j2,i1))*is; end
			%
			if states(ivar,1)=='U',
			n=[];
			if length(C_combi)>0, n=find(C_combi==ib); end
			if length(n)>0,
				ic=ic_comb(n);
			else,
				if nbres==2 & is==0,
					ic=0;
				else,
					if rlcm(ib,3)<=0,
						if var_branche(icol,1)==var_branche(ivar,1),
							ic=1;
						else,
							if L~=0 & ~any(L_combi(nb_lind+1:nb_lcombi)==ib),
								ic=0; 
							else,
								ic=Ib(ib);
							end
						end
					else,
						ic=Ib(ib);
					end
				end
			end
			% Invert  sign of ic if source type 'L' in order to respect
			% sign convention for  varaibles Uc Il 
			if type=='L', ic=-ic; end
			% 
			A(ivar,icol)=-ic/C;
			%
			elseif states(ivar,1)=='I',
				n=[];
				if length(L_combi)>0, n=find(L_combi==ib); end
				if length(n)>0,
					Ul=el(n);
				else,
					if nbres==2 & is==0,
						%  inductor voltages= 0 ou 1V
						if nores(j1)==nores(j2),
							Ul=0;
						else,
							if nores(i1)==nores(j1),
								Ul=1;
							else,
								Ul=-1;
							end
						end
					else,
						if rlcm(ib,3)<=0 & var_branche(icol,1)==var_branche(ivar,1),
							if is==0,
								Ul=-1; % Ul=voltage across capacitor
							end
						else,
							Ul=Uj2_1;
						end
					end
				end
				% Invert  sign of Ul if source type 'C' in order to respect
				% sign convention for  varaibles Uc Il 
				if type=='C', Ul=-Ul; end
				%
				A(ivar,icol)=-Ul/L;
			end
			%
			if debug_etat, 
				fprintf(sprintf('A(%g,%g)=%g\n',ivar,icol,A(ivar,icol)));
				pause
			end
		end
	end
end
%
%  Build matrix [B] (dim nbvar1*nbr_src)
%  =====================================
%
B=[]; Dvn=[];
ib1=nbr;
%
for icol=1:nbr_src,
	if source(icol,3)==0, 
		ib1=ib1+1; % branch no. of voltage source
		type='U';
	else,
		type='I';
	end
	%
	[Yred,ind_neured,ind_neusrcred,liste_neured,nbneured,...
	nobr_zred,nobr_zcc,noneured,Tired,Tured]= etayred(icol,type,Y0,...
	nobr_cc,ind_neu,nobr_srcc,ind_neusrc,rlcm,source,liste_neu,nobr_z,Tis);
	%
	i1=ind_neusrcred(icol,1); i2=ind_neusrcred(icol,2);
	neu1=ind_neusrc(icol,1); neu2=ind_neusrc(icol,2);
	Yisol=Y;
	%
	% eliminate short-circuit due to voltage source in matrix Yisol
	if type=='U',
		yy=1/rmin;
		Yisol(neu1,neu1)=Yisol(neu1,neu1)-yy;
		Yisol(neu2,neu2)=Yisol(neu2,neu2)-yy;
		Yisol(neu1,neu2)=Yisol(neu1,neu2)+yy;
		Yisol(neu2,neu1)=Yisol(neu1,neu2);
	end
	%
	% Build matrix Z of resistive network with voltage sources eliminated
	% -------------------------------------------------------------------
%
    %[nores,nbres]=etaisole(Yred);
	[nores,nbres]=etaisole(Yisol); nores=nores(noneured);
%
	Z=zeros(nbneured);
    if nbres==1 & nbneured>1,
        Z(1:nbneured-1,1:nbneured-1)=inv(Yred(1:nbneured-1,1:nbneured-1));
    elseif (nbres==2 & nores(i1)==nores(i2)) | nbres>2, 
        fprintf (['%g isolated networks detected during computation of',...
		' column %g of matrix B\n'],nbres,icol);
		for ires=1:nbres,
			fprintf('reseau %g:\n',ires);
			noeuds=liste_neured(find(nores==ires))
		end
	if netsim_flag==1
		mess=['FATAL! Isolated network found during computation of the B matrix'];
		eval('nserror(mess,[],9);');
	else
       	 	error([]);
	end
    end
    %  
	% Source de tension de 1V
	% -----------------------
    if source(icol,3)==0,
        % Current going out voltage source (entering node1 and leaving node2)
        if nbres==2 &  nores(i1)~=nores(i2),
		% source Uc connected between 2 sub-networks
            is=0;
        else,
            is=1/(Z(i1,i1)+ Z(i2,i2)- 2*Z(i1,i2));
        end
		%  Compute branch currents  [Ib]
		%  -----------------------------
		%
		[Ib]=etaibra('U',is,icol,ib1,ind_neusrc,ind_neusrcred,...
		                   Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc);
		%
		% Compute voltages induced in transformers          
		% ----------------------------------------
		[Vtr]=etavtr(0,rlc,mat_tr,Ib);
		%
		if debug_etat,
			fprintf('Branche source: %g\n',icol);
			for i=1:length(Ib), fprintf('Ib%g=%g\n',i,Ib(i)); end
			for itr=1:ntr,
				fprintf('Tr%g :',itr);
				for i=1:mat_tr(itr,2), fprintf('V%g=%g ',i,Vtr(itr,i)); end
				fprintf('\n');
			end
		end
		%
		% Compute induced voltages in inductances of L_combi
		if nb_lcombi,
		[el,Vtr]=etaelc(ib1,i1,i2,'U',is,nores,L_combi,Il_relat,nb_ldep,...
			  ind_neu3,rlcm,rlc1, Ti3,nobr_z,nobr_zred,Tured,Z,Zb,...
			  mat_tr,Vtr,debug_etat);
		end
		% Compute currents in capacitors of  C_combi
		if nb_ccombi,
		ic_comb=etaicc(ib1,neu1,neu2,'U',is,nores,C_combi,Uc_relat,...
				nb_cdep,ind_neu,rlcm,rlc1,Ti2,nobr_z,Ib,mat_tr,debug_etat);
		end
    %
    else,
    % 1A current source
    % ------------------
        if nbres==2 &  nores(i1)~=nores(i2),
		if netsim_flag==1
			nsmsgerror='FATAL: Inductance in series with another inductance or current source';
	    		eval('nserror(nsmsgerror,[],10);');
		else

            		error(sprintf(...
			['Inductor %g-%g in series with another inductor or',...
			' another current source\n'],i1,i2));
		end
        else,
            is=1;
        end
		% Find magnetisation branch ib1 in parallel with the current source
		%  simulating saturation
		%  ------------------------------------------------------------------
		% ibmag>0 : mag. branch (ibmag) and current source in same direction
		% ibmag<0 : mag. branch (ibmag) and current source in reverse directions
		% ibmag = 0 :current source is not in parallel with mag. bramnch
		if ntr,
			ibr_mag=mat_tr(:,1)+mat_tr(:,2);  % mag. branch no. of all transfos
			n=find((ind_neu(ibr_mag,1)==neu1 & ind_neu(ibr_mag,2)==neu2) | ...
			        (ind_neu(ibr_mag,1)==neu2 & ind_neu(ibr_mag,2)==neu1));
			if isempty(n),
				ibmag=0;
			elseif ind_neu(ibr_mag(n),1)==neu1,
			% mag. branch (ibmag) and current source in same direction
				ibmag=ibr_mag(n);
			else
				ibmag=-ibr_mag(n);
			end
		else
			ibmag=0;
		end
		%
		%  Compute branch currents  [Ib]
		%  -----------------------------
		[Ib]=etaibra('I',is,icol,ibmag,ind_neusrc,ind_neusrcred,...
		                  Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc);
		%
		%  Compute voltages induced in transformers           
		% -----------------------------------------
		[Vtr]=etavtr(ibmag,rlc,mat_tr,Ib);
		%
		if debug_etat,
			fprintf('Source branch : %g\n',icol);
			for i=1:length(Ib), fprintf('Ib%g=%g\n',i,Ib(i)); end
			for itr=1:ntr,
				fprintf('Tr%g :',itr);
				for i=1:mat_tr(itr,2), fprintf('V%g=%g ',i,Vtr(itr,i)); end
				fprintf('\n');
			end
		end
		%
		% compute induced voltages in inductors of L_combi
		if nb_lcombi,
		[el,Vtr]=etaelc(0,i1,i2,'I',is,nores,L_combi,Il_relat,nb_ldep,...
			  ind_neu3,rlcm,rlc1, Ti3,nobr_z,nobr_zred,Tured,Z,Zb,...
			  mat_tr,Vtr,debug_etat);
		end
		% compute currents in capacitors of C_combi
		if nb_ccombi,
		ic_comb=etaicc(0,neu1,neu2,'I',is,nores,C_combi,Uc_relat,...
				nb_cdep,ind_neu,rlcm,rlc1,Ti2,nobr_z,Ib,mat_tr,debug_etat);
		end
    end
    %
	%
	% Off-diagonal elements of B; B(ivar,icol)
	% ----------------------------------------
	for ivar=1:nbvar1,
		ib=var_branche(ivar,1);
		j1=ind_neured(ib,1); j2=ind_neured(ib,2);
		R= rlc1(ib,1); L= rlc1(ib,2); C= rlc1(ib,3);
		%
		% Voltage Uj2_1 =  Uj2 - Uj1 between nodes j1 and j2
		%for a cuurent is entering node i2  and leaving node i1
		%  (current i1-->i2 in the source)
		Uj2_1=0;
		if nbres==1,Uj2_1=(Z(j1,i1) - Z(j1,i2) +Z(j2,i2) - Z(j2,i1))*is; end
		%
		if states(ivar,1)=='U',
		%-----------------------
			n=[];
			if length(C_combi)>0, n=find(C_combi==ib); end
			if length(n)>0,
				ic=ic_comb(n);
			else,
				if nbres==2 & is==0,
					ic=0;
				else
					if rlcm(ib,3)<=0 & L~=0 ...
				  	& ~any(L_combi(nb_lind+1:nb_lcombi)==ib),
						ic=0;
					else
						ic=Ib(ib);
					end
				end
			end
			% Invert sign of ic if source type 'I' in order to respect 
			% sign conventions for variables Uc Il and sources U I 
			if type=='I', ic=-ic; end
			%
			B(ivar,icol)=-ic/C;
		%
		elseif states(ivar,1)=='I',
		%-----------------------
			n=[]; 
			if length(L_combi)>0, n=find(L_combi==ib); end
			if length(n)>0,
				Ul=el(n);
			else
				if nbres==2 & is==0,   %voltage source feeding an open circuit 
					% Inductor voltage = 0V or 1V 
					if nores(j1)==nores(j2),
						Ul=0;
					else,
						if nores(i1)==nores(j1),
							Ul=1;
						else,
							Ul=-1;
						end
					end
				else,
					Ul=Uj2_1;
				end
			end
			% Invert sign of Ul if source type 'U' in order to respect 
			% sign conventions for variables Uc Il and sources U I  
			if type=='U', Ul=-Ul; end
			%
			B(ivar,icol)=-Ul/L;
		end
		%
		if debug_etat,
			fprintf(sprintf('B(%g,%g)=%g\n',ivar,icol,B(ivar,icol)));
			pause
		end
	end
	%
	% Compute column icol of matrix Dvn such that:
	% ============================================
	%
	%      [Vn]=[Cvn]*x + [Dvn]*u
	%
	%   where [Vn] = vector of node voltages of reduced resistive network
	%                where  - voltage sources have been short-circuited
	%                       - dependant L have been short-circuited
	%            x = vector of state variables
	%            u = input vector
	%
	if type=='I',
		is1=is; el1=-el; Vtr1=Vtr;
	else,
		is1=-is; el1=el; Vtr1=-Vtr;
	end
	[Vn]=  etavn(0,type,is1,i1,i2,Z,rlc1,source,liste_neu,ind_neu,...
		ind_neusrc,liste_neured,ind_neured,ind_neusrcred,noneured,...
		nbres,nores, el1,Tu,L_combi,nb_ldep,mat_tr,Vtr1);
	%
	if debug_etat,
		for i=1:nbneu,
			 fprintf('V%-4g %g\n',i,Vn(i));
		end
	end
	Dvn(1:nbneu-1,icol)=Vn(1:nbneu-1);
end
if nbvar1==0 & nbr_src~=0, B=zeros(0,nbr_src); end
