function [e,Vtr]=etaelc(ibs,neu1,neu2,type,is,nores,L_combi,Il_relat,...
					  nb_ldep,ind_neu,rlc,rlc1,...
                      Ti,nobr_z,nobr_zred,Tured,Z,Zb,mat_tr,Vtr,debug_etat)
%
%  ETAELC is a utility function used by ETATFUN to compute the voltages
%         induced across dependant inductors specified by (L_combi)
%         and transformers, when a current (is) is flowing in branch (ibs).
%
%
%  Input arguments
%  ===============
%
%    ibs     : Source branch number
%   neu1,neu2: indices of reduced nodes of branch ibs
%    type    : type of source branch source ('L', 'C','U', or 'I')
%     is     : current flowing out of the source
%    nores   : subnetwork numbers in which nodes neu1 neu2 are connected
%    L_combi : Vector of numbers of dependant and independant inductor
%               current state variables; The nb_ldep last values are numbers
%               of dependant currents
%                     ex:  L_combi =[ 8 11 13 3 5];  nb_ldep=2
%                                 2 dependant cuurents  : branches 3 and 5
%                                 depend on currents in branches 8 11 and 13
%   Il_relat :  Matrix (nb_ldep,nb_lcombi) of coefficients ki of relations
%              k1*I1+k2*I2, ...,+kn*Inb_lcombi=0 between
%              dependant and independant currents; 1 line/dependant current
%    nb_ldep :  Number of dependant inductor currents in vector L_combi
%    ind_neu :  Matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
%        rlc :  Matrix (nbr,6) or (nbr,7) 
%              specifying the network topology and R L C values;
%       rlc1 :  Matrix (nbr,3) of values of branch parameters
%        Ti  : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
% nobr_z     : vector of non-open branch numbers
% nobr_zred  : vector of non-open reduced branch numbers
% Tured      : Matrix (nbr,nbneured) of reduced voltage transformationn matrix
%                    relating all branch voltages [Ub] 
%                    and node voltages [Un],  such that:  [Ub]=[Tured].[Un]
%     mat_tr : Modified matrix (ntr,n) of transformers
%              and mutually coupled windings
%     Vtr    : Vector (ntr,n) of voltages across the n transformer windings 
% debug_etat : Flag used for debugging; normal value=0
%
%  Output arguments
%  ================
%      e     : Vector of induced voltages across all inductors of L_combi
%     Vtr    : Corrected transformer voltages,
%              including induced voltage in dependant magnetisation branch
%                      if Lmut,Rmut series (mutual)
%                         Lmag without parallel Rmag  (transfo)
%
%             
 
%                         G. Sybille April 1995
%===============================================================================
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[nbneu,n]=size(Ti); %number of non reduced nodes, excluding last node
[n,nbneured]=size(Tured); %number of reduced nodes, excluding last node
[nbr,n]=size(ind_neu);
[ntr,n]=size(mat_tr);
nb_lcombi=length(L_combi);
nb_lind=nb_lcombi-nb_ldep; % number of independant currents of de L_combi
%
if debug_etat, fprintf(['\nBranch %g type :',type,' is=%g\n'],ibs,is); end
%
% Compute branch voltages due to current is
% =========================================
Vbr=zeros(nbr,1);
if nores(neu1)==nores(neu2) & neu1~=neu2 & is~=0,
	In=zeros(nbneured,1);
	if neu1<=nbneured, In(neu1)=-is; end
	if neu2<=nbneured, In(neu2)=is; end
	Vbr(nobr_zred)=Tured(nobr_zred,:)*Z(1:nbneured,1:nbneured)*In;
elseif nores(neu1)~=nores(neu2),
	Vbr(ibs)=-1; % Voltage between 2 isolated networks
end
%
if type=='C',
	% Capacitor ibs =1V voltage source feeding current is 
	% ----------------------------------------------------
	if rlc(ibs,3)<=0 & rlc1(ibs,1)~=0, R=rlc1(ibs,1); else, R=0; end
	Vbr(ibs)=R*is -1;
elseif type=='L',
	% Inductor ibs = current source is 
	% ---------------------------------
	if rlc(ibs,3)<=0,
		R=rlc1(ibs,1); Vbr(ibs)=R*is;
	end
elseif type=='U',
	% 1V voltage source feeding current is into branch ibs
	% -----------------------------------------------------
	Vbr(ibs)=-1;
end
%
% Eliminate short-circuited branches (same node numbers) from the list
% of non open branches (nobr_z); The new list of branches (nobr_z1)
% is used to find the loop currents for each independant L
nobr_z1=nobr_z;
nobr_cc=find(ind_neu(:,1)==ind_neu(:,2));
for i=1:length(nobr_cc),
	nobr_z1=nobr_z1(find(nobr_z1~=nobr_cc(i)));
end
%
M=zeros(nb_lcombi);
U=zeros(nb_lcombi,1);
%
% Build matrix equation [M].[e]=[U]
%        where e = vector of induced voltages
% nb_lind independant loops, each associated to an independant branch
% nb_ldep dependant currents
%
% Sum of induced voltages e = sum of Vbranch of the loop
% ======================================================
for j1=1:nb_lind
	% Inject a 1A current into branch ibr 
	% and find the loop of branches (nobr) where this current is flowing
	% -------------------------------------------------------------------
	ibr=L_combi(j1);
	i1=ind_neu(ibr,1); i2=ind_neu(ibr,2);
	In=zeros(nbneu,1);
	if i1<=nbneu, In(i1)=-1; end
	if i2<=nbneu, In(i2)=1; end
	Ib=Ti(:,nobr_z1)\In;
	indice=find(abs(Ib)>1e-10)';
	nobr=nobr_z1(indice);
	 %
	 % eliminate voltage source branches
	 % except if source type==U  and nobr==ibs
	 % ----------------------------------------
	if type=='U',
		n=find(nobr<=nbr | nobr==ibs); % Keep branch ibs if type 'U'
	else,
		n=find(nobr<=nbr);
	end
	nobr=nobr(n); indice=indice(n);
	%
	nobr=nobr.*(sign(Ib(indice)))';
	nobr=[ibr,nobr]; 
	if debug_etat,
		str=sprintf('Inductor of branch %g --> loop:',ibr);
		for i=1:length(nobr), str=[str,num2str(nobr(i)),' ']; end
		fprintf([str,'\n']);
		nobr
		Vbranche=Vbr(abs(nobr),:)'
	end
	%
	% line j1 of [M][e]=sum of voltages induced in loop nobr 
	% ------------------------------------------------------
	for i=1:length(nobr),
		j2=find(L_combi==abs(nobr(i)));   
		if length(j2)>0, M(j1,j2)=sign(nobr(i)); end
	end  
	%
	% line j1 of [U]=sum of branch voltages Vbr=RIin loop nobr
	% --------------------------------------------------------
	% Do not consider voltage of the first inductive branch because RI=0
	% except if it is a source branch ibs, type R L C series
	ib1=2;
	if nobr(1)==ibs & (type=='L' | type=='C'),
		if rlc(ibs,3)<=0 , ib1=1;  end
	end
	%
	%Sum of voltage drops RI
	for i=ib1:length(nobr),
		U(j1)=U(j1)+Vbr(abs(nobr(i)))*sign(nobr(i)); %Sum of RI
	end
	%
	%Add voltages induced in transformer and mutual  secondary windings
    for i=1:length(nobr),
        for itr=1:ntr
		%ibmag_dep=Index no of dependantmag.  branch ( mutuals only). 
        ibmag_dep=find(L_combi==(mat_tr(itr,1)+mat_tr(itr,2)));
		%
		%  Modify voltages of secondary windings only
            for iwind=2:mat_tr(itr,2),
                if abs(nobr(i))==(mat_tr(itr,1)+iwind-1),
                    U(j1)=U(j1)+Vtr(itr,iwind)*sign(nobr(i)); % RI enroul. iwind
					%
					% If magnetisation branch is dependant (mutual)
                    % add voltage across Lmag = Ntour_sec/Ntour_prim in matrix M
                    if ~isempty(ibmag_dep),
                        % Add voltage induced in secondary windings
                        % = el(mag)* N2/N1 ou -N2/N1 
                        M(j1,ibmag_dep)=M(j1,ibmag_dep)+sign(nobr(i))*...
                                mat_tr(itr,3*(iwind+1))/mat_tr(itr,6);
                    end
                end
            end
        end
    end
end
%
% nb_ldep relations between dependant and independant currents 
% ============================================================
for i=1:nb_ldep,
	j1=nb_lind+i; 
	for j2=1:nb_lcombi,
		ib1=L_combi(j2);
		M(j1,j2)=Il_relat(i,j2)/rlc1(ib1,2);
	end
end
%
e=inv(M)*U;
%
if debug_etat,
	M
	U
	L_combi
	e
end
%
% Modify voltages of mutual and transformer windings
%---------------------------------------------------
% (Add voltage across dependant Lmag  (mutuals)
for itr=1:ntr
ibmag_dep=find(L_combi==(mat_tr(itr,1)+mat_tr(itr,2))); %index no. of mag branch
if debug_etat, fprintf('Tr%g=',itr); end
    if ~isempty(ibmag_dep),
        for iwind=1:mat_tr(itr,2),
            Vtr(itr,iwind)=Vtr(itr,iwind)-...
                           e(ibmag_dep)*mat_tr(itr,3*(iwind+1))/mat_tr(itr,6);
            if debug_etat, fprintf('V%g=%g ',iwind,Vtr(itr,iwind)); end
        end
    end
if debug_etat, fprintf('\n'); end
end
