function [A,B,C,D]=etafbak(A1,B1,C1,D1,a2,b2,c2,no_inp1,no_out1)
%
% ETAFBK  is a utility function used by ETATFUN to compute 
%         the state-space representation of a System 1 (a,b1,c1,d1) having
%		  outputs connected in feedback to its input through a second
%		  state-space system (a2,b2,c2,d2).
%
%         This function is typically used to compute the equivalent state-space
%         system  of a circuit with with closed switches.
%         System 1 is the linaer circuit with switches open.
%         System 2 is the transfer function of closed switches 
%                  modelled as R-L series elements.
%         Switche models are driven by voltages (NO_OUT1) and generate currents
%         injected at inputs NO_INP1 of System 1.
%
%
%                          +-------------+
%              u >---+--u1>| A1,B1,C1,D1 |---+-->y
%                    |     +-------------+   |
%  inputs NO_IN1 --> u2        system 1      | <----- outputs NO_OUT1
%                    |                       |
%                    |     +-------------+   |       
%                    +<----| A2,B2,C2,D2 |<- +     
%                          +-------------+
%                              system 2
%
%   Positive feedback is assumed ie u1=u+u2
%
%
% Input arguments
% ---------------
%  A1,B1,C1,D1 : System 1, n1 states, nu=nu1 inputs, ny=ny1 outputs
%  a2,b2,c2    : System 2, n2 states, nu2=inputs , ny2 outputs
%                Matrix d2 is assumed to be zero
%      NO_INP1 : Integer vector (ny2)  containing
%                indexes to inputs of system 1 connected to outputs of system 2
%      NO_OUT1 : Integer vector (nu2) containing
%                indexes to outputs of system 1 connected to inputs of system 2
%
%
% Output arguments
% ---------------
%  A ,B ,C ,D  : Resultant system having the same number of
%                inputs and outputs as system 1.

%   G. Sybille   96-04-18
%   Last revision  96-06-21
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[n1,nu]=size(B1);
%[ny,n1]=size(C1);
[ny,nu]=size(D1);
[n2,nu2]=size(b2);
%
%Adapt dimensions of system 2 to system 1 
% ---------------------------------------
A2=a2;
B2=zeros(n2,ny);
B2(:,no_out1)=b2;
C2=zeros(nu,n2);
C2(no_inp1,:)=c2;
%
% Compute A B C D of equivalent system
% ------------------------------------
A=zeros(n1+n2,n1+n2);
A(1:n1,1:n1)=A1;
A(1:n1,n1+1:n1+n2)=B1*C2;
A(n1+1:n1+n2,1:n1)=B2*C1;
A(n1+1:n1+n2,n1+1:n1+n2)=A2+B2*D1*C2;
%
B=zeros(n1+n2,nu);
B(1:n1,:)=B1;
B(n1+1:n1+n2,:)=B2*D1;
%
C=zeros(ny,n1+n2);
C(:,1:n1)=C1;
C(:,n1+1:n1+n2)=D1*C2;
%
D=D1;   
%
