function [H2]=etahlin(ninput,noutput,freq,lines)
%
% ETAHLIN is a utility function used by ETATFUN to compute the steady-state
%         values of states and outputs  of a circuit including 
%         distributed parameter transmission lines.
%
%                      +------+
%         U1 >---+--U->| H1(s)|---+->Y   H1 : Linear system including switches
%                |     +------+   |      H2 : Non linear system (dist. lines)
%                |   state-space  |
%                U2               |    
%                |     +------+   |      U=U1+U2 : inputs to linear system
%                +<----| H2(s)|<--+      Y       : outputs of linear system
%                      +------+
%                    line models
%
%   The electrical system is simulated by the closed loop system
%   shown above where:
%        H1 = transfer function of the linear system (A B C D)
%        H2 = transfer function of the non linear system
%   Two types of inputs are applied to the linear state-space system :
%        - LINEAR INPUTS     corresponding to linear voltage and
%                            current sources
%        - NON LINEAR INPUTS corresponding to current outputs from 
%                            distributed parameter lines
%   U  : U1+U2 Linear + non linear inputs
%   U1 : Complex column vector[NINPUT] of the inputs to the state-space system
%        where non linear inputs coming from line models  are set to zero
%   U2 : Complex column vector[NINPUT] of the inputs to the state-space system
%        where linear inputs are set to zero
%
%   ETASSDL computes the steady-sate values of outputs Y and states X
%   of the linear system from the steady state values of voltage and
%   current sources (U1).
%
% Input arguments
% ---------------
%   NINPUT: Number of inputs of state-space system=length(U1)
%  NOUTPUT: Number of outputs of state-space system=length(Y)
%    FREQ : Frequency of the applied sources U1
%   LINES : Matrix [nline,ncol] containing the transmission line parameters
%           Each row of the LINE matrix is used for a transmission line.
%           The number of columns of LINE depends on the number of phases of 
%           the transmission line.
%           For a NPHASE line, the first (4+3*NPHASE+NPHASE^2) columns are used.
%           For example, for a three-phase line, 22 columns are used.
%
%                 (1)   (2)   (3)   (4)  (5...                 ...) 
%              NPHASE  No_I  No_U  LONG  L/Xl   Zc  Rm   Speed Ti
%
%           NPHASE = Number of phases of the transmission line
%            No_I  = Input number in the SOURCE matrix corresponding to the
%                    first currrent source Is_1 of the line model.
%                    Each line model uses 2*NPHASE current sources specified
%                    in the SOURCE matrix as follows:
%                    Is_1 Is_2 ...Is_nphase for the sending end followed by
%                    Ir_1 Ir_2 ...Ir_nphase for the receiving end.
%            No_U  = Output number of the state-space corresponding to the
%                    first voltage output Vs_1 feeding the line model.
%                    Each line model uses 2*NPHASE voltage outputs 
%                    in the SOURCE matrix as follows:
%                    Vs_1 Vs_2 ...Vs_NPHASE for the sending end followed by
%                    Vr_1 Vr_2 ...Vr_NPHASE for the sending end followed by
%           LONG   = Length of the line (km) 
%             Zc   = Vector of the NPHASE modal caracteristic impedances (ohms)
%             Rm   = Vector of the NPHASE modal series linear resistances 
%                    (ohms/km)
%           speed  = Vector of the NPHASE modal propagation speeds (km/s)
%             Ti   = Transformation matrix from  mode to phase currents
%                    such that Iphase=Ti.Imode
%                    The  NPHASE*NPHASE matrix is given in vector format
%                    [col_1,col_2, ... col_NPHASE]
%
% Output arguments
% ---------------
%            H2    = Complex matrix (ninput,noutput) 
%                    Transfer function of the distributed parameter transmission
%                    lines.
%                                   U2=H2(jw)*Y
%                    where :  U2 = Column complex vector of cuurent outputs
%                                  of transmission line models
%                             Y  = Complex vector of voltage inputs of
%                                  transmission line models
%                                  (outputs of state-space system)

%   G. Sybille   96-03-29
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

w=2*pi*freq;
j=sqrt(-1);
[nline,ncol]=size(lines);
H2=zeros(ninput,noutput);
%
%
% Determination of transfer function of non linear system H2 (ninput,noutput)
% ===========================================================================
for iline=1:nline,
	nphase=lines(iline,1);
	noinp=lines(iline,2); % first source number
	noout=lines(iline,3); % first output number
	long=lines(iline,4);
	icol=5; Zmode=lines(iline,icol:icol+nphase-1); % caracteristic impedances
	icol=icol+nphase; Rmode=lines(iline,icol:icol+nphase-1);
	icol=icol+nphase; Smode=lines(iline,icol:icol+nphase-1); % prop. speed
	t0mode=long./Smode; % Mode delays
	icol=icol+nphase;
	Ti=reshape(lines(iline,icol:icol+nphase^2-1),nphase,nphase);
	%
	% Matrix Y (admittances connected at both ends of line)
	% ---------------------------------------------------------
	Zimp=zeros(nphase,nphase);
	for imode=1:nphase,
		Zimp(imode,imode)=Zmode(imode)+Rmode(imode)*long/4;
	end
	Y=Ti*inv(Zimp)*Ti';
	%
	n1=nphase+1; n2=2*nphase;
	Ysr=zeros(n2,n2);
	Ysr(1:nphase,1:nphase)=Y; Ysr(n1:n2,n1:n2)=Y;
	DD=zeros(n2,n2);
	DD(1:nphase,1:nphase)=-eye(nphase,nphase);
	DD(n1:n2,n1:n2)=eye(nphase,nphase);
	TTi=zeros(n2,n2);
	TTi(1:nphase,1:nphase)=Ti; TTi(n1:n2,n1:n2)=Ti;
	TTv=TTi;
	%
	% Matrix [Ym] such that Imode=Ym*Vmode
	% including resistive losses lumped at beginning/mid/end of line
	% --------------------------------------------------------------
	% Imode=[ImS1 S2 ... Sn ImR1 R2 ... Rn];
	% Vmode=[VmS1 S2 ... Sn VmR1 R2 ... Rn];
	%
	Ym=zeros(1,n2);
	for imode=1:nphase,
		gamal=j*w/Smode(imode)*long/2;
		A1=cosh(gamal);      B1=Zmode(imode)*sinh(gamal);
		C1=1/Zmode(imode)*sinh(gamal); D1=A1;
		ML=[A1 B1 ; C1 D1];    %ABCD matrix of losslessline of length long/2  
		MR1=[1 Rmode(imode)*long/4 ; 0 1]; % ABCD matrix of R/4 at both ends
		MR2=[1 Rmode(imode)*long/2 ; 0 1]; % ABCD matrix of R/2 at mid point    
		M=MR1*ML*MR2*ML*MR1;    % ABCD matrix of R/4|line/2|R/2|line/2|R/4
		Ym(imode,imode)=M(1,1)/M(1,2);
		Ym(imode+nphase,imode+nphase)=-M(1,1)/M(1,2);
		Ym(imode,imode+nphase)=-1/M(1,2);
		Ym(imode+nphase,imode)=+1/M(1,2);
	end
	%
	%
	% Transfer line impedance matrix Hline(2*nphasex2*nphase) 
	% H2= Phase currents/Phase voltages     
	% ---------------------------------------------------------
	H2(noinp:noinp+2*nphase-1,noout:noout+2*nphase-1)=Ysr + DD*TTi*Ym*TTi';
end
