function [Ib]=etaibra(type,is,ibs,ibr1,ind_neu,ind_neured,...
                       Ti2,Tured,Zb,Z,nobr_zred,nobr_zcc)
%
%   ETAIBRA is a utility function used by ETATFUN to compute branch currents
%           considering that some branches may be short-circuited
%
%
%  Input arguments
%  ===============
%
%    type    : type of source branch source ('L', 'C','U', or 'I')
%     is     : current flowing out of the source
%    ibs     : Source branch number (type L or C) or voltage source no (type U)
%    ibr1    : If type='U' : Branch no corresponding to athe voltage source
%              If type='I' : Transfo. magnetisation branch number
%                            (Branch in parallel with current source)
%                            ibr1>0: currents in same direction
%                            ibr1<0: currents in reverse directions            
%    ind_neu :  Matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
%  ind_neured:  Matrix(nbneured,2) of reduced branch numbers
%       Ti2  : Transformation matrix relating node and branch currents
%              including transformer contributions
%              and such that: [In]=[Ti2].[Ib]
%    Tured   : Transformation matrix relating reduced branch and node voltages
%              currents, such that:  [Vbr]=[Tured].[Vn]
%       Zb   : Vector of branch impedances
%       Z    : Impedance matrix (nbneu-1,nbneu-1) = inv(Yred)
%  nobr_zred : Vector of reduced and non-open branch numbers
%  nobr_zcc  : Vector of short circuited branches
%
%  Output argument
%  ===============
%    Ib      : Vector of branch currents
%

%                         G. Sybille April 1995
%                         Revised: Dec-4-95
%===============================================================================
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[nbr]=length(Zb); 
[nbneu,n]=size(Ti2); nbneu=nbneu+1;
[nbneured,n]=size(Z);
i1=ind_neured(ibs,1);
i2=ind_neured(ibs,2);
%
Ib=zeros(nbr,1);
if is==0, return; end
%
% Compute currents in non short-circuited branches  (nobr_zred)
% =============================================================
%
if i1==i2,
	Ib1=zeros(length(nobr_zred),1);
else,
	if length(nobr_zred)>0 ,
		In=zeros(nbneured-1,1);
		if i1<nbneured, In(i1)=-is; end
		if i2<nbneured, In(i2)=is; end
		Ib1=diag(ones(size(nobr_zred))./Zb(nobr_zred))*...
			Tured(nobr_zred,:)*Z(1:nbneured-1,1:nbneured-1)*In;
	end
end
%
% Add source branch current (L C or U) to non short-circuited branches
% --------------------------------------------------------------------
%MODIF
n=[]; if length(nobr_zcc)>0, n=find(nobr_zcc==ibs); end
%
% if (type=='L' | type=='C') & ~any(nobr_zcc==ibs)...
%   & ind_neu(ibs,1)~=ind_neu(ibs,2),
if (type=='L' | type=='C') & isempty(n) & ind_neu(ibs,1)~=ind_neu(ibs,2),
%MODIF
	Ib1=[is;Ib1];
	nobr_zred=[ibs,nobr_zred];
%
elseif type=='U'
	Ib1=[is;Ib1];
	nobr_zred=[ibr1,nobr_zred];
%
% ibr1= magnetisation branc in parallel with current source
% ---------------------------------------------------------
elseif type=='I' & ibr1~=0,
	% Modify contribution of magnetisation branch 
	% having a parallel current source (simulationg a saturable reactor)
	% ( include contribution of the current source)
	n=find(nobr_zred==abs(ibr1));
	Ib1(n)=Ib1(n)+sign(ibr1)*is;
end
%
if nbr>0, Ib(nobr_zred)=Ib1; end
%
%
% Compute currents in short-circuited branches (nobr_zcc)
% =======================================================
%
% Use relation [In]= Ti2(:,nobr_zred)*Ib1 +Ti2(:,nobr_zcc)*Ib_cc
%  where Ib1 = currents in non short-circuited branches calculated above
%         In = node currents
%      Ib_cc = currents in short-circuited branches to be determined
%
if length(nobr_zcc)>0,
	In=zeros(nbneu-1,1);
	if ind_neu(ibs,1)<nbneu, In(ind_neu(ibs,1))=-is; end
	if ind_neu(ibs,2)<nbneu, In(ind_neu(ibs,2))=is; end
	if length(nobr_zred)>0,
		if type=='I' & ibr1==0,
			In=In-Ti2(:,nobr_zred)*Ib1;
		else,
			In=-Ti2(:,nobr_zred)*Ib1;
		end
	end
	if ~all(In==0),
		Ib(nobr_zcc)=Ti2(:,nobr_zcc)\In;
	end
end
%
% Current in a capacitoir in parallel with a source inductance
n=[]; if length(nobr_zcc)>0, n=find(nobr_zcc==ibs); end
%if length(find(nobr_zcc==ibs))==1 & type=='L', Ib(ibs)=-is; end
if ~isempty(n) & type=='L', Ib(ibs)=-is; end
%
% Impose zero current in a branch short-circuited by a capacitor
% different from the one in the same parallel branch
if i1==i2 & length(find(nobr_zcc==ibs))==0,  Ib(ibs)=0; end
%
n=find(abs(Ib/is)<1e-10);
if ~isempty(n),
	Ib(n)=zeros(length(n),1);
end
%
