function [ic]=etaicc(ibs,neu1,neu2,type,is,nores,C_combi,Uc_relat,...
				       nb_cdep,ind_neu,rlc,rlc1,Ti,nobr_z,Ib,mat_tr,debug_etat)
%
%
%  ETAICC is a utility function used by ETATFUN to compute the currents
%         in dependant capacitors specified by (C_combi)
%         when a current (is) is flowing in branch (ibs).
%
%
%  Input arguments
%  ===============
%
%    ibs     : Source branch number
%   neu1,neu2: indices of reduced nodes of branch ibs
%    type    : type of source branch source ('L', 'C','U', or 'I')
%     is     : current flowing out of the source
%    nores   : subnetwork numbers in which nodes neu1 neu2 are connected
%    C_combi : Vector of numbers of dependant and independant capacitor 
%               voltage state variables; The nb_cdep last values are numbers
%               of dependant voltages
%               ex:  C_combi =[ 8 11 13 3 5];  nb_cdep=2
%                             2 dependant voltages : branches 3 and 5
%                             are dependant of voltages in branches 8 11 and 13
%  Uc_relat  : Matrix (nb_ldep,nb_lcombi) of coefficients ki of relations
%              k1*U1+k2*U2, ...,+kn*Unb_ccombi=0 between
%              dependant and independant voltages; 1 line/dependant voltage
%    nb_cdep : Number of dependant capacitor voltages in vector C_combi
%    ind_neu :  Matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
%        rlc :  Matrix (nbr,6) or (nbr,7) 
%              specifying the network topology and R L C values;
%       rlc1 :  Matrix (nbr,3) of values of branch parameters
%        Ti  : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
%     nobr_z : vector of non-open branch numbers
%         Ib : vector of branch currents
%     mat_tr : Modified matrix (ntr,n) of transformers 
% debug_etat : Flag used for debugging; normal value=0
%
%  Output argument
%  ===============
%      ic    : Vector of currents flowing in each capacitor of of C_combi
%             

%                         G. Sybille April 1995
%                                    Last revision: 96-06-21
%===============================================================================
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[nbneu,n]=size(Ti); %nbneu= number of nodes, excluding reference node
[nbr,n]=size(ind_neu);
[ntr,n]=size(mat_tr);
nb_ccombi=length(C_combi);
nb_cind=nb_ccombi-nb_cdep; %number of independant voltages of C_combi
C_ind=C_combi(1:nb_cind);
%
%
if debug_etat, fprintf(['\nBranch %g type :',type,' is=%g\n'],ibs,is); end
%
if is==0,
	ic=zeros(nb_ccombi,1);
	if debug_etat,
		ic
	end
	return
end
%
% Build vector n_select of nodes of branches specified in C_ind
% (numbers of nodes per subnetwork)-1 = (number of capacitors of the subnetwork)
%=========================================================================
%
% Determine subnetworks of independant C
% --------------------------------------
Y=zeros(nbneu+1,nbneu+1);
for i=1:nb_cind,
	ib=C_combi(i);
	i1=ind_neu(ib,1);i2=ind_neu(ib,2);
	Y([i1 i2],[i1 i2])= Y([i1 i2],[i1 i2])+[1 -1; -1 1];
end
%
node_cind=find(diag(Y~=0));
Y=Y(node_cind,node_cind);
[nosub,nbsub]=etaisole(Y);
%     
% Select (n-1) first nodes of each subnetwork
% -------------------------------------------
n_select=[];
for isub=1:nbsub
	n=find(nosub==isub);
	node_subnet=node_cind(n);
	% number of nodes to be used for the subtwork = ntotal-1 = number of Cind
	n=length(node_subnet)-1; 
	n_select=[n_select,node_subnet(1:n)'];
end
%
% Branch currents Ib1 due to current is in branch ibs
% ===================================================
Ib1=Ib;
%
%
% Build vector b_select of selected branches 
% for computation of currents injected into nodes  n_select of C_ind 
%    b_select= non open branche (nobr_z) not specified in C_combi
%    plus  C_dependant branches (open) in parallel with a resistance
% ===================================================================
%
b_select=[];
for i=1:length(nobr_z),
	n=find(C_combi==nobr_z(i));
	n1=find(C_combi(nb_cind+1:nb_ccombi)==nobr_z(i));
	rpara=0;
	if length(n1)>0, 
		if rlc1(nobr_z(i),1)~=0 , rpara=1; end
	end
	if length(n)==0 | rpara,
		b_select=[b_select,nobr_z(i)];
	end
end
%
% Include contribution of resistive branch in parallel with  C source
% --------------------------------------------------------------------
rpara=0;
if type=='C',
	if rlc1(ibs,1)~=0 & rlc(ibs,3)==1, rpara=1; end
end
if type=='C' | type=='U',
	if  rpara,
		% current in resistive branch in parallel with  C or U
		Ib1(ibs)=-1/rlc1(ibs,1);
		if isempty(find(b_select==ibs)), b_select=[b_select,ibs]; end
	else,
		% delete source branch U or C without R parallel
		% ----------------------------------------------
		b_select=b_select(find(b_select~=ibs));
	end
end
%
% TRANSFORMERS:
% - Determine if source branche =L leakage or Lmag in paral. with R (ibs_tr~=-1)
% - Add to b_select leakage inductive branches of transformers
%  (contribution of transfo windings to currents injected into nodes n_select)
% - Correct magnetisation branch (R-L parallel) current 
% -----------------------------------------------------------------------------
%
ibs_tr=-1;
for itr=1:ntr,
	%
	for iwind=1:mat_tr(itr,2),
		ibr=mat_tr(itr,1)+iwind-1;
		if ~any(b_select==ibr), b_select=[b_select,ibr];  end
		if ibr==ibs, ibs_tr=ibs;  end
	end
	%
	ibr=ibr+1;     % magnetisation branch
	% Add to magnetisation current flowing in Rmag the source current
	% du to inductive Lmag branch in  parallel with Rmag
	% for later calculation of node currents
	% of transformer secondaries (calculated by par [In]=[Ti2].[Ib] ) 
	if ibr==ibs ,
		if rlc1(ibs,1)~=0,
			ibs_tr=ibs;
			Ib1(ibs)=Ib1(ibs)+is;
		end
	end
end
%
if debug_etat,
	n_select
	nobr_z
	b_select
	Ib1
end
%
% Compute [I]= Sum of currents injected into nodes of independant C
% ==================================================================
if isempty(b_select),
	I=zeros(n_select)';
else,
	I=-Ti(n_select,b_select)*Ib1(b_select);
end
%
% Add source current injected into 2 appropriate nodes of n_select
% if source branch is not a capactive branch of C_combi
% and if source is not a transformer leakage inductance
%         (Contribution already condidered with branch of b_select)
% or if source branch is not a transformer magnetisation inductance
%         (Isource already include in Ib mag) 
% ---------------------------------------------------------------------
n=find(C_combi==ibs);
if ~(type=='C' & length(n)>0) & ibs~=ibs_tr, 
	i=find(n_select==neu1);
	if length(i)>0, I(i)=I(i)-is; end
	i=find(n_select==neu2);
	if length(i)>0, I(i)=I(i)+is; end
end
%
% Build [M] =current  matrix of C_combi
% =====================================
M=Ti(n_select,C_combi);
%
% Complete the nb_cdep lines of M 
% (relations between  dependant and independant voltages)
% -------------------------------------------------------
for i=1:nb_cdep,
	j1=nb_cind+i;
		for j2=1:nb_ccombi,
		ib1=C_combi(j2);
		M(j1,j2)=Uc_relat(i,j2)/rlc1(ib1,3);
	end
end
% Add nb_cdep zeros to  column vector I 
% -------------------------------------
I=[I;zeros(nb_cdep,1)];
%
% Solve matrix equation  [M].[ic]=[I]
%=====================================
ic=inv(M)*I;
%
if debug_etat,
	M
	I
	C_combi
	ic
end

