function [Y,L_combi,nb_ldep,Zb,nobr_z,Il_relat] = etailr(...
					  Y,Zb,rlc,rlc1,Ti,ind_neu,rmin,mat_tr,netsim_flag,fid)
%
%     ETAILR  is a utility function used by ETATFUN to determine the relations
%             between dependant and independant inductor currents.
%
%  Input arguments
%  ===============
%	
%         Y  : admittance matrix of resistive network (nbneu,nbneu)
%        Zb  : Vector ofbranch impedancesof resistive branches and branches
%              short-circuited (capacitors and voltage sources)
%        rlc : Matrix (nbr,6) or (nbr,7) 
%              specifying the network topology and R L C values;
%       rlc1 : Matrix (nbr,3) of values of branch parameters
%        Ti  : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
%    ind_neu : Matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
%      rmin : value of the minimum impedance used for a short-circuit
%     mat_tr: Matrix (ntr,n) of transformers and mutually coupled windings
%netsim_flag: Integer =0 if ETATFUN is called by ETAT or from the Matlab window.
%                      1 if ETATFUN is called by NETSIM
%       fid : File identifier of output file containing detailed messages 
%
%  Output arguments
%  ================
%        Y   : Modified admittance matrix
%              (dependant induvctive branches short-circuited)
%    L_combi : Vector of dependant and independant inductor
%              current state variables; The nb_ldep last values are numbers
%              of dependant currents.
%    nb_ldep : Number of dependant inductor currents in vector L_combi
%        Zb  : Vector of modified branch resistances 
%              (dependant branches short-circuited)                
%     nobr_z : Vector of corresponding non-open branch no
%   Il_relat : Matrix (nb_ldep,nb_lcombi) of coefficients ki in  relations
%              k1*I1+k2*I2, ...,+kn*Inb_lcombi=0 between
%              dependant and independant currents; 1 line/dependant current
%              as explained by the example below:
%                        
% Example:       L_combi =[ 8 11 13 3 5];  nb_ldep=2
% -------                 2 dependant currents   : branches 3 and 5
%                         depend on 3 branch currents : branches  8 11 and 13
%
%                If the 2 relations are : i3=  i8 -i13
%                                         i5= -i8 -i11
%                ETATILR returns:                    
%                           Il_relat =[-1   0   1   1   0  --> -i8 + i13 + i3=0
%                                       1   1   0   0   1] -->  i8 + i11 + i5=0
%                                       8  11  13   3   5
%
%                 
%
%

%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

Zb1=Zb;
[ntr,n]=size(mat_tr);
[nbr,n]=size(ind_neu);
L_combi=[]; L_dep=[]; nb_lcombi=0; nb_ldep=0;
Il_relat=[];
%
% Find isolated networks
% ---------------------
[nores,nbres]=etaisole(Y);
%
% Find 'isolating' and dependant inductive branches
% =================================================
Yelim=Y;
ib=0; nbrese=nbres; n=nbres;
while nbres~=1 & ib<nbr,
    ib=ib+1;
    if (rlc(ib,3)==0 & rlc(ib,5)~=0) | ...
       (rlc(ib,3)==1 & rlc(ib,4)==0 & rlc(ib,5)~=0 & rlc(ib,6)==0),
        % remplacer inductance de branche par 1 ohm et
        % verifier si nb de reseau isole diminue de 1
        Ymod=Y;
        i1=ind_neu(ib,1); i2=ind_neu(ib,2);
        Ymod(i1,i1)=Ymod(i1,i1)+1; Ymod(i2,i2)=Ymod(i2,i2)+1;
        Ymod(i1,i2)=Ymod(i1,i2)-1; Ymod(i2,i1)=Ymod(i1,i2);
        [nores,nbres1]=etaisole(Ymod);
        if nbres1==(nbres-1),
            L_combi=[L_combi,ib];
            nb_lcombi=nb_lcombi+1;
            % Find dependant inductive branches (to eliminate)
			% ------------------------------------------------
            R=rlc1(ib,1);
            if rlc(ib,3)==0 & R~=0, yy=1/R; else, yy=1/rmin; end;
            Yelim(i1,i1)=Yelim(i1,i1)+yy; Yelim(i2,i2)=Yelim(i2,i2)+yy;
            Yelim(i1,i2)=Yelim(i1,i2)-yy; Yelim(i2,i1)=Yelim(i1,i2);
            [nores,nbrese]=etaisole(Yelim);
            if nbrese==(n-1),
                n=nbrese;
                L_dep=[L_dep,ib];
                nb_ldep=nb_ldep+1;
				% Short-circuit branches to be eliminated
				if rlc(ib,3)<=0 & rlc(ib,4)~=0,
					Zb(ib)=rlc1(ib,1);
				else,
					Zb(ib)=rmin;
				end
            else,
                % Eliminate short-circuits for inductors to be kept
				% --------------------------------------------------
                Yelim(i1,i1)=Yelim(i1,i1)-yy;
                if abs(Yelim(i1,i1)*rmin)<1e-12, Yelim(i1,i1)=0; end
                Yelim(i2,i2)=Yelim(i2,i2)-yy;
                if abs(Yelim(i2,i2)*rmin)<1e-12, Yelim(i2,i2)=0; end
                Yelim(i1,i2)=Yelim(i1,i2)+yy;
                if abs(Yelim(i1,i2)*rmin)<1e-12, Yelim(i1,i2)=0; end
                Yelim(i2,i1)=Yelim(i1,i2);
            end
        end
    end
end
%
% Build nobr_z
% ------------
nobr_z=find(Zb~=0);
%
if ~nb_ldep & ntr==0, return; end
%
Y=Yelim;
%
% Sort L_dep (Place nb_ldep eliminated branches at the end)
% ---------------------------------------------------------
n=[];
for i=1:nb_lcombi,
	if any(L_dep==L_combi(i)),  n(i)=1; else; n(i)=0;  end;
end
[n,indice]=sort(n);
L_combi=L_combi(indice);
%
[nbneu,n]=size(Ti); %nbneu= number of nodes excluding reference node
nb_lind=nb_lcombi-nb_ldep; %number of independant currents in L_combi
%
% Relations between dependant and independant currents
% ====================================================
% Inject a curent courant in the nb_lind independant inductors 
% and find the loop branches (nobr) where this cuurent is flowing
% ---------------------------------------------------------------
%n=length(nobr_z)-ntr; %non-open branches, except transfos
n=length(nobr_z); % non-open branches
M=[];
%
for i=1:nb_lind
	ibr=L_combi(i);
	i1=ind_neu(ibr,1); i2=ind_neu(ibr,2);
	In=zeros(nbneu,1);
	if i1<=nbneu, In(i1)=-1; end
	if i2<=nbneu, In(i2)=1; end
	Ib=Ti(:,nobr_z(1:n))\In;
	nobr=nobr_z(1:n).*(sign(Ib))';
	nobr=nobr(find(abs(Ib)>1e-10 )');
	vec=zeros(1,nb_ldep);
	for j1=1:nb_ldep,
		ib1=find(L_combi(nb_lind+j1)==abs(nobr));
		if ~isempty(ib1),
			vec(j1)=sign(nobr(ib1));
		end
	end
	M=[M;vec];
end
%
% Update L_combi and Il_relat for dependant currents (mutuals)
% ------------------------------------------------------------------------
for itr=1:ntr,
    ib1=mat_tr(itr,1);
    nwind=+mat_tr(itr,2);
    ibmag=ib1+nwind;
    % Consider series magnetisation branches
    if rlc(ibmag,3)==0,
        L_dep=L_combi(nb_lind+1:nb_lcombi);
        M=[M,zeros(nb_lind,1)];
        nb_ldep=nb_ldep+1;
        %  Contribution  of winding currents to new column nb_ldep of M
        for k=1:nwind,
            ib=ib1+k-1;
            ndep=find(L_dep==ib);
            if ~isempty(ndep), %Winding current ib is already existing
                if ~isempty(M), M(:,nb_ldep)=M(:,nb_ldep)+M(:,ndep); end
            else
                L_ind=L_combi(1:nb_lind);
                nind=find(L_ind==ib);
                if isempty(nind), % create dependant winding current ib
                    L_combi=[L_ind,ib,L_combi(nb_lind+1:nb_lcombi)];
                    nb_lind=nb_lind+1;
                    nb_lcombi=nb_lcombi+1;
                    M(nb_lind,nb_ldep)=1;
                else % independant winding current already exists in L_combi
                    %M(nind,nb_ldep)=1;
                    M(nind,nb_ldep)=M(nind,nb_ldep)+1;
                end
            end
        end
        L_combi=[L_combi,ibmag];
        nb_lcombi=nb_lcombi+1;
        nobr_z=[nobr_z,ibmag];
        if  rlc(ibmag,4)~=0,
            Zb(ibmag)=rlc1(ibmag,1);
        else,
            Zb(ibmag)=rmin;
        end
    end
end
%
Il_relat=[-M',eye(nb_ldep,nb_ldep)];
%
% Print dependance relations
% ==========================
if nb_ldep>0,
	fprintf(fid,'\nThe following inductor currents are dependant: \n');
	for i=1:nb_ldep,
		n=L_combi(nb_lind+i);   
		str=sprintf('Il_b%g_n%g_%g = ',n,rlc(n,1),rlc(n,2));
		if length(find(Il_relat(i,:)~=0))==1,
			str=[str,'0'];
		else
			for n=1:nb_lcombi-nb_ldep,
				if sign(Il_relat(i,n))>0 & abs(Il_relat(i,n))==1,
					str1=' - Il_b';     
				elseif sign(Il_relat(i,n))>0 & abs(Il_relat(i,n))~=1,
					str1=sprintf(' - %g*Il_b',abs(Il_relat(i,n)));     
				elseif sign(Il_relat(i,n))<0 & abs(Il_relat(i,n))==1,
					str1=' + Il_b';     
				elseif sign(Il_relat(i,n))<0 & abs(Il_relat(i,n))~=1,
					str1=sprintf(' + %g*Il_b',abs(Il_relat(i,n)));     
				end
				if Il_relat(i,n)~=0, str=[str,str1,int2str(L_combi(n)),'_n',...
				sprintf('%g_%g',rlc(L_combi(n),1),rlc(L_combi(n),2)),' ']; end
			end 
		end
		fprintf(fid,[str,'\n']); 
	end
end
