function [rlc,liste_neu,ind_neu,ind_neusrc,ind_neusw,mat_tr,unit,rlccar]=...
		 etapar(rlc,switches,source,nb_sortie,neuref_auto,unit_auto,...
		 netsim_flag,fid)
%
%   ETAPAR is a utility function used by ETATFUN and ZCAL .
%          It performs preprocessing and validation of  matrices 
%          RLC, SWITCHES and SOURCES containing the input parameters to ETATFUN.
%
%  Input arguments
%  ===============
%       rlc : Matrix (nbr,6)  or (nbr,7)
%             specifying the network topology and R L C values;
%  switches : Switch matrix (nsw,7) 
%    source : Source matrix (nsrc,5) or (nsrc,6)
% nb_sortie : number of outputs
%neuref_auto: Reference node number used for transmission lines;
%             -1 Node is not specified (interactive mode)
%             (The user will be prompted  to specify a node number)
% unit_auto : units used to specify R L and C values in the rlc matrix
%             'OHM', 'OMU' 'P.U' or '%  '
%              unit(1)='%'indictes unit not specified (Interactive mode)
%             (The user will be prompted  to specify a unit) 
%netsim_flag: Integer =0 if ETATFUN is called by ETAT or from the Matlab window.
%                      1 if ETATFUN is called by NETSIM
%       fid : File identifier of output file containing detailed messages 
%
%
%  Output arguments
%  ===============
%       rlc : Modified matrix (nbr,8) or (nbr,10) 
%             specifying the network topology and R L C values;
%             If size(rlc) = nbr,6 column 7 is created to contain branch number
%             Column 8 is not used (previous format of mutuals)
%             For transmission lines, columns 9 and 10 contain capacitive branch
%             numbers
%             Matrix is sorted according to branch number (col. 7)              
%  liste_neu: vector (nbneu) of node numbers
%    ind_neu: matrix (nbr,2) of indices of branch node numbers in 'liste_neu'
%             ind_neu(ib,1) : beginning node of branch ib
%             ind_neu(ib,2) : end node of branch ib
% ind_neusrc: matrix (nsrc,2) of indices of source node numbers
%  ind_neusw: matrix (nsw,2) of indices of switch node numbers
%     mat_tr: matrix (ntr,n) of transformers and mutually coupled windings
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
%             ....
%      unit : Unit specified ; 3 car. string (OHM OMU or P.U)
%    rlccar : String matrix containing formated branch impedances
%             to be printed on ZCAL output graph

%   G. Sybille April 1995
%   Last revision : 97-02-17
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $
%===============================================================================
% Dimensions used in the function:
% --------------------------------
% number of noeuds                (nbneu)
% number of self impedances       (nbr_prop)
% number of mutual impedances     (nbr_mut)
% number of lines                 (nbr_ligne)
% number of transformers          (nxfo)      
% number of mutuals               (nmut)      
% number of switches              (nsw)      
% number of branches (total)      (nbr) = nbr_prop + nbr_mut
% number of sources               (nbr_src)      
%
[nbr,nbcol]=size(rlc);
if nbcol==10, rlc=rlc(:,1:8); nbcol=8; end
if nbcol==7, rlc=[rlc,zeros(nbr,1)]; nbcol=8; end
if nbcol~=0 & nbcol~=6 & nbcol~=8,
	str=sprintf('Incorrect dimensions for [rlc] :%gx%g;',nbr,nbcol);
	str=[str,sprintf(' Expected dimensions:  %gx6 ou %gx7',nbr,nbr)];
	error(str);
end
if nbcol==6, 
	% create column 7 and 8 
	% column 8 was previously used to specify mutuals;
	% this format is now obsolete
	rlc(1,8)=0;
end
%
nbr_mut=0;
nbr_ligne=0;
ntr=0;  mat_tr=[];
neu_ref=0;
nbneu=0; liste_neu=[]; 
if nbr, liste_neu(1)=rlc(1,1); nbneu=1; end
%
for ib=1:nbr,
	% Build vector liste_neu containing node numbers
	if ~any(liste_neu==rlc(ib,1)), nbneu=nbneu+1; liste_neu(nbneu)=rlc(ib,1); end
	if ~any(liste_neu==rlc(ib,2)), nbneu=nbneu+1; liste_neu(nbneu)=rlc(ib,2); end
	%
	if nbcol==6, rlc(ib,7)=ib; end
	if rlc(ib,8)>0, nbr_mut=nbr_mut+1; end
	if rlc(ib,3)<0, nbr_ligne=nbr_ligne+1; end
	if rlc(ib,3)<=0 & all(rlc(ib,4:6)==0),
	  error(sprintf(['Series branch %g located between nodes %g and %g ',...
					  'has a zero impedance'],ib,rlc(ib,1),rlc(ib,2)));
	end
	if rlc(ib,3)==1 & rlc(ib,5)==inf; rlc(ib,5)=0; end
	if rlc(ib,3)==0 & rlc(ib,6)==inf & unit=='OMU', rlc(ib,6)=0; end
end
nbr_prop=nbr-nbr_mut;
%
%
if nbcol==8,
	% Sort rlc in order to move couplings at the end; obsolete
	%---------------------------------------------------------
	% [y,i]=sort(rlc(:,8)); rlc=rlc(i,:);
	%rlc=classe(rlc,8,1,nbr); 
	% Sort rlc by increasing branch number (column 7 of rlc)
	%----------------------------------------------------------------------
	[y,i]=sort(rlc(1:nbr_prop,7)); rlc(1:nbr_prop,:)=rlc(i,:);
	%rlc=classe(rlc,7,1,nbr_prop);
end
%
%
% Build transformer and mutual matrix
% -----------------------------------
%     mat_tr: matrix (ntr,n) of transformers and mutually coupled windings
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
ib=0;
nxfo=0; nmut=0;
while ib<nbr,
    ib=ib+1;
    if rlc(ib,3)==2 | rlc(ib,3)==3,
        ntr=ntr+1; n=0;
        mat_tr(ntr,1:3)=[ib,0,0];
		ib1=ib;
		if rlc(ib1,3)==2,
			nxfo=nxfo+1;
		elseif rlc(ib1,3)==3,
			nmut=nmut+1;
		end
		%
        while rlc(ib,3)==rlc(ib1,3) & ib<nbr,
            if rlc(ib,3)==2 & rlc(ib,6)<=0,
                error(sprintf(['Transformer no. %g: nominal voltage',...
                ' of winding %g (branch %g) must be positive'],...
                nxfo,n,ib));
            end
            if rlc(ib,3)==3 & rlc(ib,6)~=0,
                error(sprintf(['Mutual no. %g: the content of',...
                ' column 6 of [rlc] matrix must be zero'...
				'for winding %g (branch %g) '], nmut,n,ib));
            end
            n=n+1; % number of windings
			%
			%Transfer 2nd terminal number, winding gresistance, voltage
			if rlc(ib,3)==2,
				mat_tr(ntr,3*n+1:3*n+3)=[rlc(ib,2),rlc(ib,4),rlc(ib,6)];
			elseif rlc(ib,3)==3,
				% voltage ratio =1 for mutuals
				mat_tr(ntr,3*n+1:3*n+3)=[rlc(ib,2),rlc(ib,4),1];
			end
			ib=ib+1;
        end
		%
		% Verification of mutuals and magnetisation branches
		% --------------------------------------------------
        if ~(rlc(ib,2)==rlc(ib-n,2) &...
			((rlc(ib1,3)==3 & rlc(ib,3)==0) | (rlc(ib1,3)==2 & rlc(ib,3)==1))),
				if rlc(ib1,3)==2,
					error([sprintf('Transfo  %g; %g windings;',nxfo,n),...
					 ' magnetisation branch not found']);
				elseif rlc(ib1,3)==3,
					error([sprintf('Mutual  %g; %g windings;',nmut,n),...
					 ' mutual branch not found']);
				end
        else,
            if rlc(ib,3)==1 & rlc(ib,4)<=0,
                error(sprintf(['Transformer no. %g: the parallel ',...
                'resistance of the magnetisation branch (branch %g) ',...
                'must be positive'],nxfo,ib)); end
            if rlc(ib,3)==0 & rlc(ib,5)==0,
                error(sprintf(['Mutual no. %g: the mutual inductance',...
                ' (branche %g) cannot be zero'],nmut,ib)); end
            mat_tr(ntr,2)=n; % nb of windings
            mat_tr(ntr,3)=rlc(ib,1);
        end
		%
		% Verification of winding resistances
		% -----------------------------------
		for i=2:n,
			if rlc(ib1,3)==2,
				if rlc(ib1+i-1,4)<=0,
					error(sprintf(['Transformer no. %g: ',...
					'the resistance of winding %g (branch %g)',...
					'must be positive'],nxfo,i,ib1+i-1));
				end
			elseif rlc(ib1,3)==3,
				if (rlc(ib1+i-1,4)-rlc(ib,4))<0,
					error(sprintf(['Mutual no. %g: the resistance',...
					' Rself-Rmut of winding %g (branches %g and %g)',...
					' must be positive or null'],nmut,i,ib1+i-1,ib));
				end
			end
		end
    end
end
%
nbr_add=0;
if nbr_ligne>0,
	if neuref_auto>=0,
		neu_ref=neuref_auto;
		fprintf(fid,'Reference node number for transmission lines : %g\n',...
		neu_ref); 
	else,
		neu_ref=input...
		('Reference node number (ground) for transmission lines : ');
		if ~any(liste_neu==neu_ref),
			fprintf(['Node %g does not exist; ',...
					 'it will be added to the list of nodes\n'],neu_ref);
		end
	end
	if ~any(liste_neu==neu_ref), nbneu=nbneu+1; liste_neu(nbneu)=neu_ref; end
	%
	% Sort liste_neu in order to have reference node in last position
	% ----------------------------------------------------------------
	n=find(liste_neu~=neu_ref);
	n1=find(liste_neu==neu_ref);
	liste_neu=[liste_neu(n),liste_neu(n1)];
	%
%-------------------------------------------------------------------------------
%For transmission lines, create a shunt branch connected to ground at each end 
% nbr_add = nb of additionnelles branches created for transmission lines
%--------------------------------------------------------------------------------
	% Add 2 columns (9 and 10) to [rlc] to keep shunt branch numbers
    rlc(nbr,10)=0;
    rlc_add=zeros(1,10); 
    nbr1=nbr;
    for ib=1:nbr1,
        if rlc(ib,3) <0, 
            for ineu=1:2,
                itrouve=0;
                neu1=rlc(ib,ineu);
                ibr=nbr1; % Find shunt branch in the list od added branches
                while ~itrouve & ibr < nbr,
                    ibr=ibr+1;
                    if ((rlc(ibr,1)== neu_ref & rlc(ibr,2)== neu1) |...
                    (rlc(ibr,2)== neu_ref & rlc(ibr,1)==neu1)) & rlc(ibr,3)==1,
                        % Parallel branch has been found
                        % Keep branch no in column 9 or 10
                        rlc(ib,8+ineu)=ibr;
                        itrouve=1;
                    end   
                end
                if ~itrouve,
                    % Add a new parallel branch
                    rlc_add(1,1)=neu1;
                    rlc_add(1,2)=neu_ref;
                    rlc_add(1,3)=1;
                    % rlc_add(1,4)=1e10;
                    rlc_add(1,7)=nbr_prop+1;
                    rlc=[rlc(1:nbr_prop,:); rlc_add; rlc(nbr_prop+1:nbr,:)];
                    nbr_prop=nbr_prop+1;
                    nbr=nbr+1;
                    rlc(ib,8+ineu)=nbr_prop;
                    nbr_add=nbr_add+1;
                end
            end  % fin de boucle ineu
        end
    end
end
%
%
if unit_auto(1)~='%',
	unit=unit_auto;
	fprintf(fid,['Unit specified : ',unit,'\n']);
else,
	fprintf('Allowed units are :OHM, PU or OMU (Ohms mH uF)\n');
	if ~exist('zcal_auto'), unit='   '; end
	while (~all(unit == 'OHM') &  ~all(unit == 'P.U') & ~all(unit == 'OMU')),
		unit=input('Specify units for R L C branches (OHM PU ou OMU):','s');
		dimu=size(unit);
		if dimu(2)==2 ,
			if all(unit=='PU'), unit='P.U'; else unit='   '; end
			dimu=size(unit);
		end
		if dimu(2)~=3, unit='   '; end
	end
end
unit1= ['(',unit,')'];
if nbr_ligne>0,
	rlccar1(1,:)='Node_1 Node_2 Type/L(km)'; % 28 car.
else
	rlccar1(1,:)='Node_1 Node_2 Type      '; % 28 car.
end
    rlccar2(1,:)='R(ohms)   ';
if all(unit == 'OHM'),
	rlccar3(1,:)='Xl(ohms)  ';
	rlccar4(1,:)='Xc(ohms)   Branch#';
elseif  all(unit =='P.U'),
	rlccar2(1,:)='R(pu)     ';
	rlccar3(1,:)='Xl(pu)    ';
	rlccar4(1,:)='Xc(pu)     Branch#';
else 
	rlccar3(1,:)='L(mH)     ';
	rlccar4(1,:)='C(uF)/U(V) Branch#';
end
str=[rlccar1(1,:),rlccar2(1,:),rlccar3(1,:),rlccar4(1,:),'\n'];
fprintf(fid,str);
for ib =1:nbr,
	if rlc(ib,3)==0,
		conex='S         ';
	elseif rlc(ib,3)==1,
		conex='P         ';
	elseif rlc(ib,3)<0,
		conex=sprintf('Li %6.1f ',-rlc(ib,3));
	elseif rlc(ib,3)==2,
		conex='Tr        ';
	elseif rlc(ib,3)==3,
		conex='Mut       ';
	end

	str=sprintf('%-6g %-6g ',rlc(ib,1),rlc(ib,2));
	str=[str,conex];
	rlccar1(ib+1,:)=str;
	str=sprintf('%-5.4g     ', rlc(ib,4));
	rlccar2(ib+1,:)=str(1:10);
	str=sprintf('%-5.4g     ',rlc(ib,5));
	rlccar3(ib+1,:)=str(1:10);
	str=sprintf('%-6.4g     %-8g',rlc(ib,6),rlc(ib,7));
	rlccar4(ib+1,:)=str(1:18);
	str=[rlccar1(ib+1,:),rlccar2(ib+1,:),rlccar3(ib+1,:),rlccar4(ib+1,:),'\n'];
	fprintf(fid,str);
end
rlccar=[rlccar1,rlccar2,rlccar3,rlccar4];
%
% Verify branch numbering
% -----------------------
for ib=1:nbr_prop,
	if ib ~= rlc(ib,7), 
		str=sprintf(...
	'Numbering of the %g branches is incorrect; Branch %g  not specified',...
		nbr_prop,ib); error(str);
	end
end
%
% Build matrix ind_neu(nbr,2) of index no of each branch
% ------------------------------------------------------
%  ind_neu(ib,1)= index du 1st node of branch ib
%  ind_neu(ib,2)= index du 2nd node of branch ib
ind_neu=[];
for ib=1:nbr,
	index=1;
	while rlc(ib,1)~=liste_neu(index), index=index+1; end
	ind_neu(ib,1)=index;
	index=1;
	while rlc(ib,2)~=liste_neu(index), index=index+1; end
	ind_neu(ib,2)=index;
end
%
%
fprintf(fid,'Number of nodes %g\n',nbneu);
fprintf(fid,'Number of branches %g\n',nbr_prop);
if nbr_ligne>0,
	fprintf(fid,'Number of lines %g\n',nbr_ligne);
	fprintf(fid,'Number of shunt branches added %g\n',nbr_add);
end
fprintf(fid,'Number of transformers %g\n',nxfo);
fprintf(fid,'Number of mutuals (inductive coupling) %g\n',nmut);
%
% Verification of SOURCE matrix
% ==============================
nbr_src=0; nbr_srcU=0; nbr_srcI=0;
ind_neusrc=[];
if exist('source') & ~isempty(source),
	[nbr_src,n]=size(source);
		if n~=3 & n~=5 & n~=6 & n~=7,
			str=sprintf(...
			'Dimensions of matrix [source] are incorrect :%gx%g;',...
			nbr_src,n);
			str=[str,sprintf(...
			' Expected dimensions: %gx3 or %gx5 or %gx6 or %gx7',...
			nbr_src,nbr_src)];
			error(str);
		end
%
	for ib=1:nbr_src,
		% Add source nodes to liste_neu
		i1=[]; if length(liste_neu)>0, i1=find(liste_neu==source(ib,1)); end
		if isempty(i1),
			liste_neu=[liste_neu,source(ib,1)];
			nbneu=nbneu+1;
			i1=nbneu;
		end
		ind_neusrc(ib,1)=i1;
		i2=find(liste_neu==source(ib,2));
		if isempty(i2),
			liste_neu=[liste_neu,source(ib,2)];
			nbneu=nbneu+1;
			i2=nbneu;
		end
		ind_neusrc(ib,2)=i2;
%
if source(ib,3)==0, nbr_srcU=nbr_srcU+1;
elseif source(ib,3)==1, nbr_srcI=nbr_srcI+1;
else,  
	str=sprintf...
	('Source type : %g beteen nodes %g and %g is not valid\n',...
	source(ib,3),source(ib,1),source(ib,2));
	error(str);
end
end
fprintf(fid,'Number of voltage sources %g\n',nbr_srcU);
fprintf(fid,'Number of current sources %g\n',nbr_srcI);
end
%
% Verification of SWITCH matrix
% =============================
ind_neusw=[];
if exist('switches')
	[nsw,n]=size(switches);
	if n~=7 & nsw~=0,
		str=sprintf('Dimensions of matrix [switches] are not correct :%gx%g;',...
		nsw,n);
		str=[str,sprintf(' Expected dimensions: %gx7',nsw)];
		error(str);
	end
	%
	for ib=1:nsw,
		% verifier node numbers
		i1=find(liste_neu==switches(ib,1));
		if ~isempty(i1),
			ind_neusw(ib,1)=i1;
		else
			error(sprintf(...
			'Switch connected betwen nodes %g et %g; Node %g is not valid\n',...
			switches(ib,1),switches(ib,2),switches(ib,1)));
		end   
		i2=find(liste_neu==switches(ib,2));
		if ~isempty(i2),
			ind_neusw(ib,2)=i2;
		else
			error(sprintf(...
			'Switch connected betwen nodes %g et %g; Node %g is not valid\n',...
			switches(ib,1),switches(ib,2),switches(ib,2)));
		end   
		%
		if switches(ib,3)~=0 & switches(ib,3)~=1,
			str=sprintf...
			('Status (%g) of switches %g is not valid 0 or 1 expected 1)\n',...
			switches(ib,3),ib);
			error(str);
		end
		%
		if switches(ib,5)==0,
			str=sprintf(...
			'A null inductance for switch %g is not allowed'\n',ib);
			error(str);
		end
		%
		flag=0;
		if switches(ib,6)<=nbr_src,
			if source(switches(ib,6),3)~=1, flag=1; end
		else,
			flag=1;
		end
		if flag,
			str=sprintf(...
			'Invalid current source number (col. 6) for switch %g\n',ib);
			error(str);
		end
		%
		if switches(ib,7)>nb_sortie,
			str=sprintf(...
			'Invalid output number (col. 7) for switch %g\n',ib);
			error(str);
		end
	end
	fprintf(fid,'Number of switches %g\n',nsw);
end
%
% Print SWITCH and SOURCE matrices	
% ================================
fprintf(fid,'\n');
if nbr_src, 
	fprintf(fid,'Source matrix:\n');
	[nbr_src,n]=size(source);
	fprintf(fid,'Node1 Node2 U/I  Mag.     Phase    Frequency Type\n'); 
	fprintf(fid,'           (0/1) (V/A)  (degrees)     (Hz)       \n'); 
	for i=1:nbr_src,
		if n==5,
			fprintf(fid,'%-5g %-5g %-4g %-8g %-8g\n',...
			source(i,1),source(i,2),source(i,3),source(i,4),source(i,5));
		elseif n==6 ,
			fprintf(fid,'%-5g %-5g %-4g %-8g %-8g %-8g\n',...
			source(i,1),source(i,2),source(i,3),source(i,4),source(i,5),...
			source(i,6));
		elseif n==7,
			switch source(i,7)
				case 1
					strdesc='% AC Voltage Source';
				case 2
					strdesc='% AC Current Source';
				case 10
					strdesc='% DC Voltage Source';
				case 11
					strdesc='% Controlled Voltage Source';
				case 12
					strdesc='% Controlled Current Source';
				case 8
					strdesc='% Breaker';
				case 9
					strdesc='% Arrester';
				case 14
					strdesc='% Saturable Transformer';
				case 21
					strdesc='% Diode';
				case 22
					strdesc='% Thyristor';
				case 23
					strdesc='% GTO';
				case 24
					strdesc='% MOSFET';
				case 25
					strdesc='% Ideal Switch';
				case 31
					strdesc='% Simplified Alternator';
				case 32
					strdesc='% Synchronous Machine';
				case 33
					strdesc='% Asynchronous Machine Stator';
				case 34
					strdesc='% Asynchronous Machine Rotor';
				case 35
					strdesc='% Permanent Magnet Motor';
				case 1001
					strdesc='% Distributed par. Line';
				otherwise
					strdesc='%';
			end
			fprintf(fid,'%-5g %-5g %-4g %-8g %-8g %-8g  %-2g %s\n',...
			source(i,1),source(i,2),source(i,3),source(i,4),source(i,5),...
			source(i,6),source(i,7),strdesc);
		end
	end
end
%
fprintf(fid,'\n');
if nsw, 
	fprintf(fid,'Switch matrix:\n');
	[nsw,n]=size(switches);
	fprintf(fid,'Node1 Node2 0/1  R.       L.       I#  U#\n'); 
	for i=1:nsw,    
		[nbr_src,n]=size(source);
		strdesc='';
		if n==7,
			switch source(switches(i,6),7)
				case 8
					strdesc='% Breaker';
				case 21
					strdesc='% Diode';
				case 22
					strdesc='% Thyristor';
				case 23
					strdesc='% GTO';
				case 24
					strdesc='% MOSFET';
				case 25
					strdesc='% Ideal Switch';
				otherwise
					strdesc='%';
			end
		end
		fprintf(fid,'%-5g %-5g %-4g %-8g %-8g %-3g %-3g %s\n',...
		switches(i,1),switches(i,2),switches(i,3),switches(i,4),...
		switches(i,5),switches(i,6),switches(i,7),strdesc);
	end
end
fprintf(fid,'\n');
