function [u,x,y,x0,x0sw,Asw,Bsw,Csw,Dsw,freq,H ]=...
		 etass(A,B,C,D,switches,source,rlsw,unit,freq_sys,line_dist)
%
% ETASS   is a utility function used by ETATFUN and NETSIM            
%         It computes the steady state output matrix y  and initial state
%         conditions x0 with multiple frequency voltage and current sources.
%         The steady-stae takes into consideration the status of switches
%         (open or closed) and their impedance (series R-L).
%
%         ETASS also returns the state matrices (Asw Bsw Csw Dsw) of the
%         electrical system including the closed switches. One extra state per
%         closed switch is added to the linear system with all switches open.
% 
%  Input arguments
%  ===============
%
%   A,B,C,D :  State-space matrices of the linear circuit with all switches open
%              A(nstates,nstates) B(nstates,ninput)
%              C(noutput,nstates) D(noutput,ninput)
%  SWITCHES : switch matrix (nsw,7) 
%    SOURCE : source matrix (nsrc,5) or (nsrc,6) or (nsrc,7)
%      RLSW : matrix (nsw,2) containing the switch resistance (ohms)
%             and inductance (Henries)
%      UNIT : units used to specify R L and C values in the rlc matrix 
%             unit= 'OHM' R L C values specied in ohms at fundamental frequeny
%                    specified par freq_sys.
%             unit= 'OMU' R L C values specied in ohms millihenries (mH) and 
%                    microfarads(uF)
%  FREQ_SYS : Fundamental frequency (Hz) considered for specification of
%             L and C reactances if unit = 'OHM';  default value = 60 Hz
%  LINE_DIST: matrix (nlines,n) containing the line parameters
%             (see ETATFUN function for description of line parameters)
%
%  Output arguments
%  ================
%     U,X,Y :  Matrices  U(ninput,nfreq) X(nstates,nfreq) Y(noutput,nfreq)
%              containing the steady-state complex values of inputs, states
%              and outputs . Each column correspond to a different source
%              frequency as specified by the next argument (freq).
%        X0 :  Column vector of initial values of state variables
%              considering the status (open or closed) of switches
%      X0SW :  Vector of initial values of switch currents
%    Asw,Bsw : State-space matrices of the circuit including the closed switches
%    Csw,Dsw   (one extra state per closed switch)
%      FREQ :  Column vector containing the (nfreq) source frequencies 
%              ordered by increasing frequencies.
%         H :  Complex matrix(noutput,ninput) containing the transfer impedance
%              matrix corresponding to the first non-zero frequency
%              of the FREQ vector.

%   G. Sybille January 1996
%              Last revision 97-02-17
%===============================================================================
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[nbvar1,nbvar1]=size(A);
[noutput,ninput]=size(D);
[nsw,n]=size(switches);
[nsrc,n]=size(source);
[nlines,n]=size(line_dist);
%
%
%  Feedback matrices As Bs Cs Ds  of the switch network
%  ----------------------------------------------------
nb_swf=0;
if nsw,
	no_swf=find(switches(:,3)==1); % closed switch number
	nb_swf=length(no_swf);       % number of closed switches
	if nb_swf,
		As=diag(-rlsw(no_swf,1)./rlsw(no_swf,2));
		Bs=diag(ones(nb_swf,1)./rlsw(no_swf,2));
		Cs=eye(nb_swf);       
		Ds=zeros(nb_swf);
		%
		% New matrices Asw Bsw Csw Dsw including closed switches
		% ------------------------------------------------------
		% previous call to control toolbox function replaced
		% by ETAFBAK
		%[Asw,Bsw,Csw,Dsw]=feedback(A,B,C,D,As,Bs,Cs,Ds,...
		%             switches(no_swf,6)',switches(no_swf,7)');
		%
		[Asw,Bsw,Csw,Dsw]=etafbak(A,B,C,D,As,Bs,Cs,...
		              switches(no_swf,6)',switches(no_swf,7)');
	else
		Asw=A; Bsw=B; Csw=C; Dsw=D;
	end
else
	Asw=A; Bsw=B; Csw=C; Dsw=D;
end
%
%
%  Compute steady state outputs with closed switches
%  =================================================
j=sqrt(-1);
[n,dim]=size(source); 
freq=[];
if dim==5,
	nfreq=1;
	freq=freq_sys;
elseif dim>=6,
	% Find source frequencies and form vector 'freq'
	freq=[-1];
	for isource=1:nsrc,
		if  ~any(freq==source(isource,6)),
			iswitch=[];
			if  dim<7
				% Do not consider current sources simulating switches
				if nsw>0 & source(isource,3)==1,
					iswitch=find(switches(:,6)==isource); 
				end
				% if source types are defined (col.7 of source), do not consider
				% current sources simulating non linear elements
				% Consider only voltage and current sources (type=1,2,10,11,12)
				% and current sources simulating machines (types 31 to 35)
			else,
				if  source(isource,7)~=1  & source(isource,7)~=2 & ...
					source(isource,7)~=10 &  source(isource,7)~=11 &...
					source(isource,7)~=12 & ...
					~(source(isource,7)>=31 & source(isource,7)<=35),
						iswitch=1;
				end
			end
			if isempty(iswitch), freq=[freq,source(isource,6)]; end
		end
	end
	freq=sort(freq);
	nfreq=length(freq);
	freq=freq(2:nfreq); % eliminate first value (-1)
	nfreq=nfreq-1;
end
%
u=[]; x=[]; y=[]; x_f=[];
x0=zeros(nbvar1,1); x0sw=zeros(nsw,1);
H=[];
if (dim==5 | dim>=6),
	for ifreq=1:nfreq,
		%Form input vector u_f of sources having the same frequency
        %----------------------------------------------------------
		u_f=source(:,4).*exp(j*source(:,5)*pi/180);
		if dim>=6,
			% Set to zero amplitudes of sources having 
	 		% a different frequency
			n=find(source(:,6)~=freq(ifreq));
			u_f(n)=zeros(length(n),1);
		end
		%
		if  freq(ifreq)==0 & rank(Asw)<(nbvar1+nb_swf),
			sI=j*2*pi*1e-5*eye(nbvar1+nb_swf); 
		else,
			sI=j*2*pi*freq(ifreq)*eye(nbvar1+nb_swf);
		end
		%
		%Compute transfer function matrix H_f(s)
		% including closed switches and distributed par. transmission lines
		%------------------------------------------------------------------ 
		if (nbvar1+nb_swf)>0,
			% System with states
			% ------------------
			H1=Csw*inv(sI-Asw)*Bsw+Dsw;
		else
			% Purely resistive system (no states)
			% -----------------------------------
			H1=Dsw;
		end
		%
		[noutput,ninput]=size(Dsw);
		if nlines,
			% H2 = transfer function of the lines
			%-------------------------------
			[H2]=etahlin(ninput,noutput,freq(ifreq),line_dist);
			% H_f = transfer function of the closed loop system @ freq(ifre)
			%--------------------------------------------
			I=eye(nsrc,nsrc);
			H_f=H1*inv(I-H2*H1);
			%
		else
			H2=zeros(ninput,noutput);
			H_f=H1;
		end
		%
		% Compute steady-state outputs and states
		%-----------------------------------------
		y_f=H_f*u_f;
		u2=H2*y_f; % H2 and u2 are null if there is no transmission line
		u1=u_f+u2;
		if (nbvar1+nb_swf)>0, x_f=inv(sI-Asw)*Bsw*u1; end

		% Update input vector u_f (currents in closed switches)
		if nb_swf, u_f(switches(no_swf,6))=x_f(nbvar1+1:nbvar1+nb_swf); end

		if freq(ifreq)>0,
			x0_f=imag(x_f);
		else
			x_f=real(x_f);
			x0_f=x_f;
		end
%
		%Initial conditions  with closed switches 
		%-----------------------------------------
		x0sw_f=zeros(nsw,1);
		if nb_swf, % nb_swf=number of closed switches
			x_f=x_f(1:nbvar1);
			x0sw_f(no_swf)=x0_f(nbvar1+1:nbvar1+nb_swf);
			x0_f=x0_f(1:nbvar1);
		end
		%
		% Update matrices u, x, y (1 column per frequency)
		% and column vector x0 of initial conditions
		% ------------------------------------------------
		u=[u,u_f];
		x=[x,x_f];
		y=[y,y_f];
		if ~isempty(x0_f) x0=x0+x0_f; end
		x0sw=x0sw+x0sw_f;
		%MODIF 97-05-15 G.S
		%if freq(ifreq)>0 & isempty(H), H=H_f; end
		H(:,:,ifreq)=H_f;
	end
end
