function [Y,C_combi,nb_cdep,Zb,nobr_z,Uc_relat]=etaucr(...
          Y,Y_cc,Zb,rlc,rlc1,Ti,nobr_cc,ind_neu,rmin,netsim_flag,fid);
%
%
%     ETAUCR  is a utility function used by ETATFUN to determine the relations
%             between dependant and independant capacitor voltages.
%
%  Input arguments
%  ===============
%	
%         Y  : admittance matrix of resistive network (nbneu,nbneu)
%       Y_cc : admittance matrix (nbneu,nbneu) of capcitive branches
%              where admittance of each C branch=1
%        Zb  : Vector of branch impedancesof resistive branches and branches
%              short-circuited (capacitors and voltage sources)
%        rlc : Matrix (nbr,6) or (nbr,7) 
%              specifying the network topology and R L C values;
%       rlc1 : Matrix (nbr,3) of values of branch parameters
%        Ti  : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
%     nobr_cc: Vector of purely capacitive branches
%    ind_neu : Matrix (2,nbr) of indices of branch node numbers in 'liste_neu'
%                    ind_neu(ib,1) : beginning node of branch ib
%                    ind_neu(ib,2) : end node of branch ib
%      rmin : value of the minimum impedance used for a short-circuit
%netsim_flag: Integer =0 if ETATFUN is called by ETAT or from the Matlab window.
%                      1 if ETATFUN is called by NETSIM
%       fid : File identifier of output file containing detailed messages 
%
%  Output arguments
%  ================
%        Y   : Modified admittance matrix
%              (dependant capacitive branches eliminated)
%    C_combi : Vector of dependant and independant capacitor voltage
%              state variables; The nb_cdep last values are numbers
%              of dependant voltages.
%    nb_cdep : Number of dependant capacitor voltages in vector C_combi
%        Zb  : Vector of modified branch resistances 
%              (dependant branches open)                
%     nobr_z : Vector of corresponding non-open branch no
%   Uc_relat : Matrix (nb_cdep,nb_ccombi) of coefficients ki in relations
%              k1*U1+k2*U2, ...,+kn*Unb_ccombi=0 between
%              dependant and independant voltages; 1 line/dependant voltage
%              as explained by the example below:
%
%                        
% example:       C_combi =[ 8 11 13 3 5];  nb_cdep=2
% -------                 2  dependant voltages  : branches 3 and 5
%                         depend on 3 voltages: branches 8 11 and 13
%                If the two relations are : U3=  U8 -U13
%                                           U5= -U8 -U11
%                ETATILR returns:                    
%                           Uc_relat =[-1   0   1   1   0  --> -U8 + U13 + U3=0
%                                       1   1   0   0   1] -->  U8 + U11 + U5=0
%                                       8  11  13   3   5
%

%            G. Sybille avril 1995
%               Revised 96-06-21
%==============================================================================
%
%                 
%
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

nb_cdep=0; 
C_combi=[]; C_dep=[];
Uc_relat=[];
nbneu=max(size(Y));
[nores,nbres]=etaisole(Y_cc);
yy=1/rmin;
Ti_cc=Ti(:,nobr_cc);
%
% Find dependant capacitive branches
% ----------------------------------
for i=length(nobr_cc):-1:1,
	ib=nobr_cc(i);
	% eliminate the branch and verify if the resulting number of isolated 
	% networks decreases by 1
	Ymod=Y;
	i1=ind_neu(ib,1); i2=ind_neu(ib,2);
	Y_cc(i1,i1)=Y_cc(i1,i1)-1; Y_cc(i2,i2)=Y_cc(i2,i2)-1;
	Y_cc(i1,i2)=Y_cc(i1,i2)+1; Y_cc(i2,i1)=Y_cc(i1,i2);
	[nores,nbres1]=etaisole(Y_cc);
	if nbres1==nbres,
		C_dep=[C_dep,ib];
		nb_cdep=nb_cdep+1;
		% modify Zb Ti_cc and nobr_cc
		% ----------------------------
		if rlc(ib,3)==1 & rlc1(ib,1)~=0,
			Zb(ib)=rlc1(ib,1);
		else,
			Zb(ib)=0; % open branch
		end
		n=find(nobr_cc==ib);
		Ti_cc=Ti_cc(:,[1:n-1,n+1:max(size(nobr_cc))]);
		nobr_cc=nobr_cc([1:n-1,n+1:max(size(nobr_cc))]);
		% modify Y (delete dependant C branch)
		%-------------------------------------
		Y(i1,i1)=Y(i1,i1)-yy;
		if abs(Y(i1,i1)*rmin)<1e-12, Y(i1,i1)=0; end
		Y(i2,i2)=Y(i2,i2)-yy;
		if abs(Y(i2,i2)*rmin)<1e-12, Y(i2,i2)=0; end
		Y(i1,i2)=Y(i1,i2)+yy;
		if abs(Y(i1,i2)*rmin)<1e-12, Y(i1,i2)=0; end
		Y(i2,i1)=Y(i1,i2);
	else,
	% reestablish initial Y_cc 
	% ------------------------
	Y_cc(i1,i1)=Y_cc(i1,i1)+1; Y_cc(i2,i2)=Y_cc(i2,i2)+1;
	Y_cc(i1,i2)=Y_cc(i1,i2)-1; Y_cc(i2,i1)=Y_cc(i1,i2);
	end
end
%
% Build nobr_z  
% --------------
nobr_z=find(Zb~=0);
%
if ~nb_cdep, return; end
%
% Find relations between dependant and independant Uc
% ---------------------------------------------------
% Inject a current in the nb_cdep dependant capacitors
% and find the loop of branches (nobr) where this current is flowing
M=[]; nb_cind=0;
for i=1:nb_cdep
    ib=C_dep(i);
    i1=ind_neu(ib,1); i2=ind_neu(ib,2);
    In=zeros(nbneu-1,1);
    if i1<nbneu, In(i1)=-1; end
    if i2<nbneu, In(i2)=1; end
    Ib=Ti_cc\In;
    nobr=nobr_cc.*(sign(Ib))';
    nobr=nobr(find(abs(Ib)>1e-10 )');
	for ibr=1:max(size(nobr)),
		n=[];
		if ~isempty(C_combi), n=find(C_combi==abs(nobr(ibr))); end
		if ~isempty(n),
			M(i,n)=sign(nobr(ibr));
		else,
			C_combi=[C_combi,abs(nobr(ibr))];
			nb_cind=nb_cind+1;
			M(i,nb_cind)=sign(nobr(ibr));
		end
	end
end
Uc_relat=[M,eye(nb_cdep,nb_cdep)];
%
C_combi=[C_combi,C_dep];
%
% Print dependance relations
% --------------------------
if nb_cdep>0,
	fprintf(fid,'\nThe following capacitor voltages are dependant: \n');
	for i=1:nb_cdep,
		n=C_combi(nb_cind+i);
		str=sprintf('Uc_b%g_n%g_%g = ',n,rlc(n,1),rlc(n,2));
		for n=1:nb_cind,        
			if sign(Uc_relat(i,n))>0,
				str1=' - Uc_b';
			elseif sign(Uc_relat(i,n))<0,
				str1=' + Uc_b';
			end
			if Uc_relat(i,n)~=0, str=[str,str1,int2str(C_combi(n)),'_n',...
			sprintf('%g_%g',rlc(C_combi(n),1),rlc(C_combi(n),2)),' ']; end
		end
		fprintf(fid,[str,'\n']);
	end
end

