 function [V]=  etavn(ibs,type,is,i1,i2,Z,rlc1,source,liste_neu,...
                 ind_neu,ind_neusrc,liste_neured,ind_neured,ind_neusrcred,...
				 noneured,nbres,nores,el,Tu,L_combi,nb_ldep,mat_tr,Vtr)
%
%  ETAVN is a utility function used by ETATFUN to compute the node voltages [V]
%        due to the current source no (ibs) type L C U or I and magnitude (is).
%
%  The node voltages are calculated with the following matrix eaquation
%
%                       [V]= [Cvn].x + [Dvn].[u]
%
%                 ou :  V  = node voltages
%                       x  = state vector   
%                       u  = input vector (U I) 
%                       Cvn= Matrix (nbneu-1,nbvar)
%                       Dvn= Matrix (nbneu-1,nbr_src)
%
%
%
%  Input arguments
%  ===============
%
%        ibs : Source branch number
%       type : Type of source branch source ('L', 'C','U', or 'I')
%         is : current flowing out of the source
%     i1  i2 : Indices of input and output nodes of branch ibs
%          Z : Impedance matrix (nbneu-1,nbneu-1) = inv(Yred)
%       rlc1 :  Matrix (nbr,3) of values of branch parameters
%               in Ohms,Henries,Farads
%     source : Source matrix (nsrc,5) or (nsrc,6)
%  liste_neu : Vector of non reduced node numbers 
%    ind_neu : Matrix (nbr,2) of indices of branch node numbers in (liste_neu)
%                    ind_neu(ib,1) : input node of branch ib
%                    ind_neu(ib,2) : output node of branch ib
% ind_neusrc : Matrix (nbsrc,2) of indices of source node numbers in (liste_neu)
%liste_neured: Vector of reduced node numbers 
%  ind_neured: Matrix(nbr,2) of indices of reduced branch node numbers
%              in '(liste_neured)
%ind_neusrcred:Matrix (nbsrc,2) of indices of reduced source node numbers
%              in '(liste_neured)
%   noneured : Vector of node numbers of the reduced network
%      nbres : Number of isolated sub-networks
%      nores : Vector of sub-network number for each node in (noneured)
%         el : Vector of voltages induced in inductors of L_combi
%         Tu : Transformation matrix relating reduced branch and node voltages
%              currents, such that:  [Vbr]=[Tu].[Vn]
%    L_combi : Vector of dependant and independant inductor
%              current state variables; The nb_ldep last values are 
%              dependant currents.
%    nb_ldep : Number of dependant inductor currents in vector L_combi
%     mat_tr : Modified matrix (ntr,n) of transformers
%              and mutually coupled windings
%        Vtr : Vector (ntr,n) of voltages across the n transformer windings 
%
%
%  Output argument
%  ===============
%          V : Vector of node voltages
%

%                         G. Sybille April 1995
%===============================================================================
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[nbr,n]=size(ind_neu);
nbneu=length(liste_neu);
nbneured=length(liste_neured);
nb_lcombi=length(L_combi);
nb_lind=nb_lcombi-nb_ldep;
%
V=zeros(nbneu,1);
if nbres==1 & nbneured>1 & i1~=i2,
	In=zeros(nbneured-1,1);
	if i1<nbneured,  In(i1)=-is; end 
	if i2<nbneured,  In(i2)=is; end 
	Vn=Z(1:nbneured-1,1:nbneured-1)*In;
	V(noneured(1:nbneured-1),1)=Vn;
%
%
elseif nbres==1 & i1==i2, % Current source feeding a short-circuit
                          %	All, voltages are set to zero
elseif nbres==2 & nores(i1)~=nores(i2),
	ineu1=find(nores==1);
	ineu2=find(nores==2);
	if nores(i1)==1,
		if nores(nbneured)==2,
			V(noneured(ineu1))=ones(length(ineu1),1); 
		else,
			V(noneured(ineu2))=-ones(length(ineu2),1); 
		end
	else,
		if nores(nbneured)==2,
			V(noneured(ineu1))=-ones(length(ineu1),1);
		else,
			V(noneured(ineu2))=ones(length(ineu2),1);
		end
	end
%
%
else,
	fprintf ('%g isolated networks \n',nbres);
	for ires=1:nbres,
		fprintf('Sub-network %g:\n',ires);
		noeuds=liste_neured(find(nores==ires))
	end
	error([]);
end
%
% Update short-circuited node volatges
% ------------------------------------
% Process branches having 2 short-circuited nodes
%
if nbr>0,
	nobr=find(ind_neured(:,1)==ind_neured(:,2));
else
	nobr=[];
end
%
for i=1:length(nobr),
	ib=nobr(i);
	noneu=liste_neured(ind_neured(ib,1));
	ineu=find(liste_neu==noneu);
	V(ind_neu(ib,1),1)=V(ineu);
	V(ind_neu(ib,2),1)=V(ineu);
end
%
% Process sources having 2 short-circuited nodes
%
if ~isempty(ind_neusrcred),
	nobr=find(ind_neusrcred(:,1)==ind_neusrcred(:,2));
else
	nobr=[];
end
for i=1:length(nobr),
	ib=nobr(i);
	noneu=liste_neured(ind_neusrcred(ib,1));
	ineu=find(liste_neu==noneu);
	V(ind_neusrc(ib,1),1)=V(ineu);
	V(ind_neusrc(ib,2),1)=V(ineu);
end
%
% Refer all voltages to the last node of liste_neu
% ================================================
V=V-V(nbneu);
% Modify node voltages in order to consider induced voltages
% in dependant and independant inductive branches
% ===========================================================
%
if nb_lcombi,
	%Branch voltages with dependant short-circuited 
	Vbr=V(ind_neu(:,1))-V(ind_neu(:,2)); % branches of matrix rlc
	if ~isempty(ind_neusrc),
		n=find(source(:,3)==0); % branch numbers of voltage sources;
		Vbr=[Vbr;V(ind_neusrc(n,1))-V(ind_neusrc(n,2))]; % branches  mat. source
	end
	%
	for i=1:nb_lcombi,
		ib=L_combi(i);
		if i<=nb_lind,
			% Correct voltage across independant branches
			% -------------------------------------------
			if ib==ibs, % Branche source L ou C
				if type=='L',
					% branche source L: e= RI + Ul induite (el)
					Vbr(ib)=rlc1(ib,1)*is+el(i); 
				else,  % type=C
					% branche source C: e= RI + Ul induite (el) + UC (1V)
					Vbr(ib)=rlc1(ib,1)*is+el(i)+1.0; 
				end
			else,
				% branche ouverte : tension el
				Vbr(ib)=el(i);
			end
		%
		else
		% Correct voltage across dependant branches 
		% ------------------------------------------
			if type=='C' & ib==ibs, %branche RLC serie
				Vbr(ib)=rlc1(ib,1)*is+el(i)+1.0; 
			else
				Vbr(ib)=Vbr(ib)+el(i);  % RI+ tension induite (el)
			end
		end
	end
	%
	% Complete branch voltages with voltages induced across
	% secondary windings of transformers
	% ------------------------------------------------------
	[ntr,n]=size(mat_tr);
	for itr=1:ntr,
		for iwind=2:mat_tr(itr,2),
			Vbr=[Vbr;Vtr(itr,iwind)];
		end
	end
	%
	% Corrected node voltages
	% -----------------------
	V(1:nbneu-1)=Tu\Vbr;
end
