function [Vtr]=etavtr(ibmag,rlc,mat_tr,Ib)
%
%  ETAVTR is a utility function used by ETATFUN to compute the voltages 
%         across windings of tranformers used in transformer models
%         (including leakage and magnetisation branches).
%
%  Input arguments
%  ===============
%     ibmag : No of magnetisation branch 
%             A negative branch no indicates that the current in the source  
%             and in the magnetisation branch (ibmag) are in reverse directions
%       rlc : Matrix (nbr,6) or (nbr,7) 
%             specifying the network topology and R L C values;
%     mat_tr: matrix (ntr,n) of transformers and mutually coupled windings
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
%             ....
%        Ib : Vector of branch currents
%
%  Output arguments
%  ================
%       Vtr : Vector of transformer winding voltages
%             Vtr(ntr,1) Primary voltage (across magnetising branch)
%             Vtr(ntr,2) Voltage across winding #2 (including R2*I2)
%             Vtr(ntr,3) Voltage across winding #3 (including R3*I3)     
%              ...
%

%                         G. Sybille April 1995
%===============================================================================
%
%
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

[ntr,n]=size(mat_tr);
Vtr=[];
%
for itr=1:ntr
	nb_wind=mat_tr(itr,2); % number of windings
	ib1=mat_tr(itr,1);  % branch no of primary winding
	ib3=ib1+nb_wind; %branch no of magnetising branch
	R1=mat_tr(itr,5); % primary resistance
	Rm=rlc(ib3,4);
	% ILmag= current in magnetising inductance 
	% or in parallel current source simulating saturable inductor
	if  abs(ibmag)==ib3, ILmag=sign(ibmag); else, ILmag=0; end
	%
	%
	% Compute primary current in ideal transformer
	I1=0;
	for i=2:nb_wind, % (i=secondary winding #)
	    ib2=ib1+i-1; % (ib2=branch # of secondary winding)
		N=mat_tr(itr,3*(i+1))/mat_tr(itr,6);
		I1=I1+N*Ib(ib2) ;
	end
	%
	V1=Rm*(Ib(ib1)-ILmag+I1) ; % Primary voltage across mag. branch
	Vtr(itr,1)=V1;
	%
	% Compute secondary voltages
	for i=2:nb_wind, 
	    ib2=ib1+i-1; 
		k=3*(i+1);  
		R2=mat_tr(itr,k-1);
		N=mat_tr(itr,k)/mat_tr(itr,6);
		V2=V1*N + R2*Ib(ib2);
		Vtr(itr,i)=V2;
	end
end
