function [Yred,ind_neured,ind_neusrcred,liste_neured,nbneured,...
		 nobr_zred,nobr_zel,noneured,Tired,Tured]=...
		 eta_yred(ibs,type,Y,nobr_cc,ind_neu,nobr_srcc,ind_neusrc,...
		 rlc,source,liste_neu,nobr_z,Ti)
%
%   ETAYRED  is a utility function used by ETATFUN to reduce the Y admittance
%            matrix of a resistive network with short-circuited branches.
%
%  Input arguments
%  ===============
%        ibs : Source branch number
%       type : Type of source branch source ('L', 'C','U', or 'I')
%          Y : Original admittance matrix (nbmeu,nbneu) to be reduced
%    nobr_cc : Vector of rlc branches to be short-circuited
%    ind_neu : Matrix (nbr,2) of indices of branch node numbers in Y
%                    ind_neu(ib,1) : input node of branch ib
%                    ind_neu(ib,2) : output node of branch ib
%  nobr_srcc : Vector of voltage source branches to be short-circuited
% ind_neusrc : Matrix (nbsrc,2) of indices of source node numbers in Y
%        rlc : Matrix (nbr,6) or (nbr,7) 
%              specifying the network topology and R L C values;
%     source : Source matrix (nbsrc,5) or (nbsrc,6)
%  liste_neu : Vector of non reduced node numbers 
%     nobr_z : Vector of branch # of Ti 
%         Ti : Transformation matrix relating node and branch currents
%              and such that: [In]=[Ti].[Ib]
%
%
%  Output arguments
%  ================
%       Yred : Reduced admittance matrix(nbneured,nbneured)
%  ind_neured: Matrix(nbr,2) of indices of reduced branch nodes in Yred
%              in '(liste_neured)
%ind_neusrcred:Matrix (nbsrc,2) of indices of reduced source nodes in Yred
%liste_neured: Vector of reduced node numbers
%   nbneured : Number of reduced nodes
%  nobr_zred : Vector of reduced branches  ; not short-circuited
%   nobr_zel : Vector of eliminated branches; short-circuited
%   noneured : Vector (nbneured) of reduced node numbers
%     Tured  : Transformation matrix (nbrtot,nbneured-1)
%              relating reduced branch and node voltages
%              (nbrtot = total number of branches)
%                              [Ub]=[Tured].[Unred]
%     Tired  : Transformation matrix (nbneured-1,nbrtot) 
%              relating reduced node and branch currents  
%                   
%
% Example:    Network with 6 branches and 6 nodes; 
% ========    Branches 2 and 4 (purely capacitive) to be  short-circuited
%             nobr_cc=[2 4]
%             Y=matrix 6x6; nodes ; liste_neu= [10 20 60 30 50 40]
%         
%             Branche  Noeuds     ind_neu
%             -------  -------     ------
%                 1    10  20      [1  2
%                 2    20  60       2  3
%                 3    30  50       4  5
%                 4    20  30       2  4
%                 5    30  40       4  6   2nd branch to short-circuit
%                 6    20  50       2  5]  1st branch to short-circuit
%
% iteration  short-circuited nodes      nodes of Yred       node indices     
% ---------- ---------------------     --------------    ------------------
%      0            -                 10 20 60 30 50 40  [1 2 3 4 5 6] original
%      1         20 et 50             10 20 60 30 40     [1 2 3 4 2 5]
%      2         30 et 40             10 20 60 30        [1 2 3 4 2 4]
%                                                      
%             
%  After reduction:
%             Branch   Nodes     ind_neured
%             -------  -------     ------
%                 1    10  20      [1  2
%                 2    20  60       2  3
%                 3    30  40       4  2
%                 4    20  30       2  4
%                 5    30  30*      4  4    branch  short-circuited
%                 6    20  20*      2  2]   branch short-circuited
%
%

%   G. Sybille April 1995 ; Revised Sept. 20 1996
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

Yred=Y;
nbneu=max(size(Y));
[nbr,n]=size(ind_neu);
[nbsrc,n]=size(ind_neusrc);
ind_neured=ind_neu;
ind_neusrcred=ind_neusrc;
liste_neured=liste_neu;
noneured=1:nbneu;
nobr_zred=nobr_z;
nobr_zel=[];
%
Tired=Ti;
% Rebuild last line of  Ti (corresponding to reference node)

if nbneu>2,
	Tired=[Tired;-sum(Tired)];
else,
	Tired=[Tired;-Tired];
end
%
% For a source branch 'C' (capacitor) or 'U' (voltage source)
% Eliminate source branch  ibs from the list of branches to short-circuit
% and from nobr_zred (list of reduced branches)
%
nobr_cc1=nobr_cc;
nobr_srcc1=nobr_srcc;
 if type=='C' ,
		 if length(nobr_cc1)>0, nobr_cc1=nobr_cc1(find(nobr_cc1~=ibs)); end
		 % Keep branch ibs in nobr_zred if it is a branch with paral. resistance
		 if ~(rlc(ibs,3)==1 & rlc(ibs,4)~=0),
			 nobr_zred=nobr_zred(find(nobr_zred~=ibs));
		 end
 elseif type=='U',
		 if length(nobr_srcc1)>0,
			nobr_srcc1=nobr_srcc1(find(nobr_srcc1~=ibs)); end
		 nobr1=nbr;
		 for i=1:ibs,
			 if source(i,3)==0, nobr1=nobr1+1; end
		 end
		 nobr_zred=nobr_zred(find(nobr_zred~=nobr1));
 end
%
% Reduce Y by considering purely capacitive baranches and  voltage sources
% -------------------------------------------------------------------------
%       k=1 : capacitive branches 
%       k=2 : voltage sources 
%
for k=1:2
	if k==1,
		nbrcc=length(nobr_cc1);
	else,
		nbrcc=length(nobr_srcc1);
	end
	%
	for ib=1:nbrcc
		% Find node numbers of branch to eliminate
		% ----------------------------------------
		if k==1,
			nobr=nobr_cc1(ib);
			nobr1=nobr;
			i1=min(ind_neured(nobr,1),ind_neured(nobr,2));
			i2=max(ind_neured(nobr,1),ind_neured(nobr,2));
		else,
			nobr=nobr_srcc1(ib);
			i1=min(ind_neusrcred(nobr,1),ind_neusrcred(nobr,2));
			i2=max(ind_neusrcred(nobr,1),ind_neusrcred(nobr,2));
			nobr1=nbr;
			for i=1:nobr,
				if source(i,3)==0, nobr1=nobr1+1; end
			end
		end
		%
		if i2>i1,
			% Eliminate node i2= node with largest index number
			% -------------------------------------------------
			n=find([1:nbneu]~=i2);
			liste_neured=liste_neured(n);
			noneured=noneured(n);
			Yred(i1,:)=Yred(i1,:)+Yred(i2,:); % ajouter rangee i2 a rangee i1
			Yred(:,i1)=Yred(:,i1)+Yred(:,i2); % ajouter colonne i2 a colonne i1
			Tired(i1,:)=Tired(i1,:)+Tired(i2,:);
			%
			% Update ind_neured
			%------------------
			if ~isempty(ind_neured)
				n=find(ind_neured(:,1)==i2);
				if n, ind_neured(n,1)=i1*ones(size(n)); end
				n=find(ind_neured(:,2)==i2);
				if n, ind_neured(n,2)=i1*ones(size(n)); end
				ind_neured=ind_neured-(ind_neured>i2);
			end
			%
			%Update ind_neusrcred
			%--------------------
			if ~isempty(ind_neusrcred), n=find(ind_neusrcred(:,1)==i2);
				else, n=0;end
			if n, ind_neusrcred(n,1)=i1*ones(size(n)); end
			if ~isempty(ind_neusrcred), n=find(ind_neusrcred(:,2)==i2);
				else, n=0;end
			if n, ind_neusrcred(n,2)=i1*ones(size(n)); end
			ind_neusrcred=ind_neusrcred-(ind_neusrcred>i2);
			%
			% Eliminer ligne i2 et et colonne i2 de Yred
			n_select=1:nbneu;
			n_select(i2)=0;
			n_select=find(n_select>0);
			Yred=Yred(n_select,n_select);
			% Eliminer ligne i2 de Tired
			Tired=Tired(n_select,:);
			nbneu=nbneu-1;
			%
			%eliminate short-circuited branch or voltage source from in nobr_z
			% ----------------------------------------------------------------
			nobr_zred=nobr_zred(find(nobr_zred~=nobr1));
			nobr_zel=[nobr_zel,nobr1];
		%
		end
	end
end
nbneured=length(liste_neured);
%
% eliminate from nobr_z  branches in parallel with short-circuited branches
% -------------------------------------------------------------------------
for i=1:length(nobr_z),
	ib=nobr_z(i);
	if ib<=nbr & length(nobr_zred)>0,
		if ind_neured(ib,1)==ind_neured(ib,2),
			nobr_zred=nobr_zred(find(nobr_zred~=ib));
		end
	end
end
%
% eliminate last line of Tired
Tired=Tired(1:nbneured-1,:);
% former Tured
Tured=Tired';
