function [Y0,Y,Ti2]=etayttr(Y0,Y,Ti,mat_tr,liste_neu,ind_neu)
%
%    ETAYTTR is a utility function used by ETATFUN to update
%            Contribution of transformers and mutuals to admittance
%            matrix Y and current transformation matrix Ti.
%
%  Input arguments
%  ===============
%        Y0 : Admittance matrix of resistive branches
%         Y : Admittance matrix of non open branches
%        Ti : Transformation matrix relating node and branch currents
%             including transformer contributions
%             and such that: [In]=[Ti].[Ib]
%     mat_tr: Matrix (ntr,n) of transformers and mutually coupled windings
%             mat_tr(itr,1)= branch number of 1st winding
%             mat_tr(itr,2)= number of windings
%             mat_tr(itr,3)= intermediate node number
%             mat_tr(itr,4)= Node number of 2nd terminal of 1st winding 
%             mat_tr(itr,5)= Resistance                  of 1st winding
%             mat_tr(itr,6)= Nominal volatage            of 1st winding
%             mat_tr(itr,7:9)   = Node #, Res. Voltage of 2nd winding
%             mat_tr(itr,10:12) = Node #, Res. Voltage of 2nd winding
%  liste_neu : Vector (nbneu) of non reduced node numbers 
%    ind_neu : Matrix (nbr,2) of indices of branch node numbers in (liste_neu)
%                    ind_neu(ib,1) : input node of branch ib
%                    ind_neu(ib,2) : output node of branch ib
%
%  Output arguments
%  ================
%        Y0  : Modified admittance matrix incuding contribution of transfos
%        Y   : Modified admittance matrix used to detect isolated networks    
%              (with admittances y=1 for primary and secondary sides
%              between intermediate node and 2nd terminal of winding)
%        Ti2 : Modified current transformation matrix
%

%                         G. Sybille April 1995
%===============================================================================
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

Ti2=Ti;
[ntr,n]=size(mat_tr);
nbneu=length(liste_neu);
%
for itr=1:ntr
	ib1=mat_tr(itr,1); % branch no mof 1st winding
	n=mat_tr(itr,2);   % number of windings
	ind_neutr=[];
	neu_tr=mat_tr(itr,4);
 	ind_neutr(1:2)=[ind_neu(ib1,2),find(liste_neu==neu_tr)];
	Ytr=zeros(2*n);
	Titr=zeros(2*n,n+1);
	%
	for i=3:2:2*n-1,
		k=4+3*(i-1)/2;
		neu_tr=mat_tr(itr,k);
	 	ind_neutr(i:i+1)=[ind_neu(ib1+(i-1)/2,2),find(liste_neu==neu_tr)];
		NT =mat_tr(itr,k+2)/mat_tr(itr,6); % transformation ratio: N2/N1= V2/V1
		RN=mat_tr(itr,k+1);  % secondary resistance
		Ytr(1,1)=Ytr(1,1)+NT^2/RN; Ytr(1,2)=Ytr(1,2)-NT^2/RN;
		Ytr(1,i)=Ytr(1,i)-NT/RN; Ytr(1,i+1)=Ytr(1,i+1)+NT/RN;
		Ytr(i,1)=Ytr(i,1)-NT/RN; Ytr(i,2)=Ytr(i,2)+NT/RN;
		Ytr(i,i)=Ytr(i,i)+1/RN; Ytr(i,i+1)=Ytr(i,i+1)-1/RN;
		%
		k1=(i-1)/2+1;
		Titr(1,k1)=Titr(1,k1)-NT;
		Titr(i,1)=Titr(i,1)-1/NT;
		Titr(i,n+1)=Titr(i,n+1)+1/NT;
		for ii=2:n,
			if ii~=k1,
				NT1=mat_tr(itr,3*(ii+1))/mat_tr(itr,k+2); % transformation ratio
				Titr(i,ii)=Titr(i,ii)-NT1;
			end
		end
	end
	Ytr(2:2:2*n,:)=-Ytr(1:2:2*n-1,:);
	Titr(2:2:2*n,:)=-Titr(1:2:2*n-1,:);
%
	% Windings connected together :
	% Reduce ind_neutr and Ytr if ind_neutr has identical nodes
	%----------------------------------------------------------
	nneu=length(ind_neutr);
	i=1;
	while i<=nneu,
		n=find(ind_neutr==ind_neutr(i));
		ineu=1:nneu;
		if length(n)>1,
			i1=n(1);
			for k=2:length(n),
				i2=n(k); % row and column numbers to be grouped
				Ytr(i1,:)=Ytr(i1,:)+Ytr(i2,:); %add line i2 to i1
				Ytr(:,i1)=Ytr(:,i1)+Ytr(:,i2); %add column i2 to i1  
				Titr(i1,:)=Titr(i1,:)+Titr(i2,:); %add line i2 to i1
				ineu=ineu(find(ineu~=i2)); % nodes to be kept
			end
			ind_neutr=ind_neutr(ineu);
			Ytr=Ytr(ineu,ineu);
			Titr=Titr(ineu,:);
		end
		nneu=length(ind_neutr);
		i=i+1;
	end
	%
	Y0(ind_neutr,ind_neutr)=Y0(ind_neutr,ind_neutr)+Ytr;
	n=find(ind_neutr~=nbneu); % exclude last node 
  	Ti2(ind_neutr(n),ib1:ib1+mat_tr(itr,2))=...
  	Ti2(ind_neutr(n),ib1:ib1+mat_tr(itr,2))+Titr(n,:);
	%
    % In Y add branches with admittace y=1 for primary and secondaries
	% ----------------------------------------------------------------
    % connexion entre noeud intermediare (magnetisation) 
	% et 2eme borne du 1er enroulement .
	i1=find(liste_neu==mat_tr(itr,3)); % intermediate node for mag. branch
	i2=find(liste_neu==mat_tr(itr,4)); % 2nd terminal of 1st winding
	Y(i1,i1)=Y(i1,i1)+1; Y(i2,i2)=Y(i2,i2)+1;
	Y(i1,i2)=Y(i1,i2)-1; Y(i2,i1)=Y(i2,i1)-1;
	%
    % connexions between auxiliary node and 2nd terminal of each secondary wind.
	for i=2:mat_tr(itr,2);
		i1=find(liste_neu==(mat_tr(itr,3)+0.1*i)); % aux. node 2nd winding
		i2=find(liste_neu==mat_tr(itr,3*i+1));     % 2nd terminal of 2nd winding
		Y(i1,i1)=Y(i1,i1)+1; Y(i2,i2)=Y(i2,i2)+1;
		Y(i1,i2)=Y(i1,i2)-1; Y(i2,i1)=Y(i2,i1)-1;
	end
end
