function []=nsdplini(line_dist,freq,y,ninput,block_name,nb_models,models)
%
% NSDPLINI is a utility function used by NETSIM  to initialize  
%          the four delay blocks of distributed parameter line models
%          in order to start simulation in steady state.
%
% Input arguments
% ---------------
%LINE_DIST: matrix [nline,ncol] containing the transmission line parameters
%           Each row of the LINE matrix is used for a transmission line.
%           The number of columns of LINE depends on the number of phases of 
%           the transmission line.
%           For a NPHASE line, the first (4+3*NPHASE+NPHASE^2) columns are used.
%           For example, for a three-phase line, 22 columns are used.
%
%                 (1)   (2)   (3)   (4)  (5...                 ...) 
%              NPHASE  No_I  No_U  LONG  L/Xl   Zc  Rm   Speed Ti
%
%           NPHASE = Number of phases of the transmission line
%            No_I  = Input number in the SOURCE matrix corresponding to the
%                    first currrent source Is_1 of the line model.
%                    Each line model uses 2*NPHASE current sources specified
%                    in the SOURCE matrix as follows:
%                    Is_1 Is_2 ...Is_nphase for the sending end followed by
%                    Ir_1 Ir_2 ...Ir_nphase for the receiving end.
%            No_U  = Output number of the state-space corresponding to the
%                    first voltage output Vs_1 feeding the line model.
%                    Each line model uses 2*NPHASE voltage outputs 
%                    in the SOURCE matrix as follows:
%                    Vs_1 Vs_2 ...Vs_NPHASE for the sending end followed by
%                    Vr_1 Vr_2 ...Vr_NPHASE for the sending end followed by
%           LONG   = Length of the line (km) 
%             Zc   = Vector of the NPHASE modal caracteristic impedances (ohms)
%             Rm   = Vector of the NPHASE modal series linear resistances 
%                    (ohms/km)
%           speed  = Vector of the NPHASE modal propagation speeds (km/s)
%             Ti   = Transformation matrix from  mode to phase currents
%                    such that Iphase=Ti.Imode
%                    The  NPHASE*NPHASE matrix is given in vector format
%                    [col_1,col_2, ... col_NPHASE]
%    FREQ : vector of source frequencies
%     Y   : complex output matrix (noutput,nfreq);
%   NINPUT: number of inputs to state space system
% BLOCK_NAME : String = Name of subsystem (voltage or current sensor)
%           containing the SIMULINK model
%

%       G. Sybille   96-04-10
%         Revised:   Nov-02-96
%   modif PB 07 aout 97 pour le concept de library pour modellib
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.4 $

w=2*pi*freq;
j=sqrt(-1);
[nline,ncol]=size(line_dist);
[noutput,nfreq]=size(y);
%
%
% Computation of sinusoidal voltage and current sources 
% for initialization of time delay histories
% ======================================================
% first 3 components = sending end
% last 3 components = receiving end
% The first 2*nphase lines are used  in each column ; 
%
t0max=0;
for iline=1:nline,
    nphase=line_dist(iline,1);
    long=line_dist(iline,4);
    Smode=line_dist(iline,5+2*nphase:4+3*nphase);
    t0=long/min(Smode);
    t0max=max(t0max,t0);
end
t0max=t0max*1.1;
tvec=(0:t0max/5:t0max); % time histories for delay initialization
n_time=length(tvec);
%
%
for iline=1:nline

	ptr_models=sum(nb_models(1:9));
	mskentries=get_param(deblank(models(ptr_models+iline,:)),'Mask Entries');

	nphase=line_dist(iline,1);
	noinp=line_dist(iline,2); % first source number
	noout=line_dist(iline,3); % first output number
	long=line_dist(iline,4);
	icol=5; Zmode=line_dist(iline,icol:icol+nphase-1); % Z characteristic 
	icol=icol+nphase; Rmode=line_dist(iline,icol:icol+nphase-1);
	icol=icol+nphase; Smode=line_dist(iline,icol:icol+nphase-1); % prop. speed
	t0mode=long./Smode; % Mode delays
	icol=icol+nphase;
	Ti=reshape(line_dist(iline,icol:icol+nphase^2-1),nphase,nphase);
	n1=nphase+1; n2=2*nphase;
	TTi=zeros(n2,n2);
	TTi(1:nphase,1:nphase)=Ti; TTi(n1:n2,n1:n2)=Ti;
	Vlinet=zeros(2*nphase,n_time);
	Ilinet=zeros(2*nphase,n_time);
	for ifreq=1:nfreq,
		w=2*pi*freq(ifreq);
		%
		%
		% Vline and Iline are complex vectors (2*nphase,1) containing the
		% time voltage and current histories (modal components)
		% Vlinet and Ilinet are real matrices (2*nphase,n_time) comtaining the
		% corresponding voltage and current waveforms used to initialize delays
		%
		% Vlinet= voltage waveform initializing delays 1 & 2
		% -----------------------------------------------------
		% Vmode delayed = Ti'*Vphase*expt(-jwt0)
		Vmode=TTi'*y(noout:noout+2*nphase-1,ifreq);
		Vline=Vmode.*exp(-j*w*[t0mode,t0mode]');
		Vlinet=Vlinet+imag(Vline*exp(j*w*tvec));
		%
		% Ilinet= current waveform initializing delays 3 & 4
		% Imode*h delayed = Ti*Iphase*h*exp(-j*w*t0)
		% -----------------------------------------------------
		% H2= Transfer function matrix of lines = curent outputs/voltage inputs
		[H2]=etahlin(ninput,noutput,freq(ifreq),line_dist);
		u2=H2*y(:,ifreq);  % Line current outputs
		h=(Zmode-Rmode*long/4)./(Zmode+Rmode*long/4);
		Imode=inv(TTi)*u2(noinp:noinp+2*nphase-1);
		Iline=Imode.*[h,h]'.*exp(-j*w*[t0mode,t0mode]');
		Ilinet=Ilinet+imag(Iline*exp(j*w*tvec));
	end


 	% Initialisation of sinusoidal and voltage current sources
    	strini=mat2str([tvec',Vlinet(1:nphase,:)'],4);
    	mskentries=[mskentries,'|',strini];
    	strini=mat2str([tvec',Vlinet(nphase+1:2*nphase,:)'],4);
    	mskentries=[mskentries,'|',strini];
    	strini=mat2str([tvec',Ilinet(1:nphase,:)'],4);
    	mskentries=[mskentries,'|',strini];
	strini=mat2str([tvec',Ilinet(nphase+1:2*nphase,:)'],4);
    	mskentries=[mskentries,'|',strini];

   	% Initialisation of time delays.
    	tini=num2str(t0max,4); 
    	mskentries=[mskentries,'|',tini];

	% set mask enetries of the model.
	model=[block_name,'/source/dist_line/d',num2str(iline)];
	set_param(model,'MaskValueString',mskentries);
end


