function [zz,xx,Ybus1,node_list,node12,unit,rlccar]=...
	zcalfun(rlc,switches,source,unit,node12,valx,power2sys_flag,...
	z_option,li_option,freq_sys,scsource,noderef,ibrv,iparv,fparv);
% ZCALFUN computes the impedance between two nodes of an electrical circuit.
%         Two options are available:
%      1) Computation of the impedance as function of frequency.
%      2) Sensitivity analysis of the impedance at a particular frequency 
%         when some parameters of a particular branch  are varied.
%     
%  Input arguments
%  ===============
%       The number of input arguments may be 6 or 15.
%       Only the first 6 arguments must be specified for option 1.
%       Arguments 7 to 15 allow use of option 2 and some other options
%       explained below.
%
%       RLC : Branch matrix (nbr,6) or (nbr,7) specifying the network topology
%             and R L C values; see format below
%  SWITCHES : switch matrix (nsw,7) ; see format  below
%    SOURCE : source matrix (nsrc,5) or (nsrc,6) or nsrc(7) ; see format  below
%      UNIT : units used to specify R L and C values in the rlc matrix 'rlc
%             unit= 'OHM' R L C values specied in ohms at fundamental frequeny
%                    specified by FREQ_SYS 
%                    Default system frequency = 60 Hz.
%             unit= 'OMU' R L C values specied in ohms millihenries (mH) and 
%                    microfarads(uF)
%             unit= 'PU'  R L C values specied in pu 
%                    This option is not availble when transformers are used
%    NODE12 : Vector (1,2) containing  the two node numbers between
%             which impedance is calculated
%      VALX : Vector (nval) which can contain one of the following depending on
%             the option used (Z_OPTION=1 or 2):   
%             - option 1 Z(F): VALX contains the frequencies F to bu used
%             - option 2 Z(K): VALX contains the multiplication factors K
%                              to be applied to the nominal values of R L C 
%                              of the specified branch to be varied
%                              See arguments IBRV IPARV FPARV below.
%
%       The following argument is optionnal
%NETSIM_FLAG: Integer controlling printing of outputs while ETATFUN is executing
%             Integer =0  Outputs will be printed
%                     =1  (default value) Version# is printed
%                         Outputs will not be printed
%                     =2  Silent mode                        
%
%       The following 8 arguments are optional:
%
%  Z_OPTION : Integer =  Option for impedance calculation
%             Z_OPTION=1 (Default value when 6 inpout argu,ments are specified)
%                        Impedance Z(F) is computed as function of frequencies
%                        specified in vector VALX
%             Z_OPTION=2 Impedance Z(K) is computed at a particular frequency
%                        FPARV as function of a range of multiplicative 
%                        parameter K specified in vector VALX.
%                        The varaition is applied to some parametres (max. 4)
%                        of a particular branch defined by arguments 
%                        IBRV, IPARV and FPARV.
% LI_OPTION : integer= type of model to be used for transmission lines
%             LI_OPTION=1 : One pi section per line
%             LI_OPTION=2 : Distributed R L C parameters
%  FREQ_SYS : Fundamental frequency (Hz) considered for specification of
%             L and C reactances if unit = 'OHM';  default value = 60 Hz
%  SCSOURCE : Integer specifying if voltage sources should be open or    
%             short-circuited for impedance calculation
%             SCSOURCE=0 (Default value) All voltage sources open
%             SCSOURCE=1                 All voltage sources short-circuited
%   NODEREF : Integer; Reference node number specifying ground connection of 
%             transmission lines (Default value = 0)
%             The next 3 parameters are used with Z_OPTION=2
%      IBRV : Integer specifying the variable branch number of the RLC matrix
%     IPARV : Integer vector (4)  specifying which parameters should be
%             varied in branch IBRV
%
%             The four components of the IPARV vector should be 0 or 1. 
%             A '1' in the Ith component is used to specify variation of the
%             Ith prameter of the branch; '0' means no variation.
%               IPARV(1)=1 : variation of the resistive part
%               IPARV(2)=1 : variation of the inductive part
%               IPARV(3)=1 : variation of the capacitive part
%               IPARV(4)=1 : for lines only :variation of the line length.
%     FPARV : Real = frequency (Hz) at which the impedance will be computed.
%             
%
%  Output arguments
%  ================
%      ZZ   : Conplex vector of impedance values
%      XX   : Real vector containing either:  
%              - frequencies used to compute ZZ (Z_OPTION=1)
%              - Values of mutplication factors applied to the variable
%                branch parameters (Z_OPTION=2)
%     YBUS1 : Complex bus admittance matrix; sparse matrix (nbnode,nbnode)
%             computed for the first frequency contained in XX.
%             As the YBUS1 matrix contains as many rows and columns as nodes
%             in the circuit, it is singular (rank=nbnode-1).
%             The impedance matrix Z (nbnode-1,nbnode-1), can be obtained
%             by inverting  Ybus1(1:nbnode-1,1:nbnode-1);
%  NODE_LIST: Integer vector (NBNODE) containing the list of nodes referring
%             to YBUS1 matrix. The last node is the reference node
%
%             The next 3 arguments returned by ZCALFUN are used when ZCALFUN 
%             is called from the script ZCAL.
%             ZCAL prompts the user to specify UNIT and  NODE12.
%             RLCCAR is a character matrix containing the RLC branch matrix 
%             formatted to be displayed on the Z graph produced by ZCAL.  
%                   
%
% Format of the RLC input matrix
% ==============================
%         Two formats are allowed:
%         6 columns :  No branch numbering; 
%                      Branch numbers correspond to the RLC line numbers(1-nbr)
%         7 columns :  Branch numbering is imposed by the user
%
%         Each line of the RLC matrix must be specified according to the
%         following format:
%
%                 (1)   (2)  (3)  (4)  (5)  (6)   [7]
%               node1 node2  TYPE  R    Xl  Xc   [Nobr] for a RLC or line branch
%          or   node1 node2  TYPE  R    L   C    [Nobr]
%               node1 node2  TYPE  R    Xl  U    [Nobr] for a transformer branch
%          or   node1 node2  TYPE  R    L   U    [Nobr]
%
%               node1 : Node number 1 of the branch
%               node2 : Node number 2 of the branch
%                       Note:
%                       Node numbers must be positive or zero.
%                       Decimal node numbers are allowed.                       
%                TYPE : Code indicating the type of connexion of R L C elements
%                       (0-3) or the transmission line length (negative value).
%                       0 : series R L C
%                       1 : parallel R L C
%                       2 : transformer winding
%                       3 : coupled (mutual) winding
%                           For a mutual or a tranformer having N windings,
%                           N+1 consecutive lines must be specified:
%                          -N lines TYPE=2 or TYPE=3; (one line per winding)
%                           Each line specifies R/L/U or R/Xl/Xc
%                           where : R/L R/Xl = winding resistance and leakage
%                                reactance or winding resist. and self reactance
%                                   U = Nominal voltage of transformer winding 
%                                       (specify 0 for TYPE 3)
%                          - one line TYPE=1 for the magnetizing branch of a
%                            transformer (parallel R/L or R/XLl)
%                            or oneline TYPE=0 for a mutual impedance
%                            (series R/L or R/XL)
% 
%                       Negative value:  transmission line length (km)
%                       ---------------
%                        For a transmission line, the R,L,C or R/Xl/Xc values
%                        must be specified in (ohm/km) or (ohm,mH,uF /km)
%
%                R    : Branch resistance (ohms or pu)
%                Xl   : Branch inductive reactance  (ohms or pu at FREQ_SYS)
%                       or transformer winding leakage reactance (ohms)
%                 L   : Branch inductance (mH)
%                Xc   : Branch capacitive reactance (ohms or pu at FREQ_SYS)
%                       (The negative sign of Xc is optional)
%                 C   : Capacitance (uF)
%                 U   : Nominal voltage of transformer winding
%                       Same units Volts or kV must be used for each winding.
%                       For a mutual (TYPE=3), this value must be set to zero.
%
%                       Zero values
%                       ----------- 
%                       For R, L or Xl, C or Xc, in a series or parallel branch
%                       indicates that the corresponding element does not exist.
%
%                       Restrictions for transformers and mutuals
%                       -----------------------------------------
%                     - Nul values are not allowed for winding resistances
%                       of transformers and mutuals. Specify a very low value
%                      (e.g. 1e-6 pu based on rated voltage and power)
%                       to simulate a quasi ideal transformer.
%                       The leakage reactances can be set to zero.
%                     - The resistive part of the magnetizing branch of a   
%                       transformer must have a finite value. Specify a very
%                       high value (low losses, e.g 1e4 pu or 0.01% based on 
%                       rating) to simulate a quasi ideal transformer.
%                       The inductive part of the magnetizing branch can be 
%                       set to infinite (No reactive losses; specify Xm=0)
%                     - A nul value is not allowed for the mutual inductance 
%                       of coupled windings. The resistive part of 
%                       series mutual branch can set to zero. 
%
%    Format of the SOURCE input matrix
%    =================================
%         Three formats are allowed:
%         5 columns :  All sources are generating the same frequency specified
%                      by FREQ_SYS
%         6 columns : The frequency of each source is specified in col. 6
%         7 columns : This column is used to specify the type
%                     of non linear element modelled by the current source
%
%         Each line of the SOURCE matrix must be specified according to the
%         following format:
%
%                 (1)   (2)  (3)    (4)   (5)    [6]    [7]
%               node1 node2  TYPE  AMP   PHASE  [FREQ]  [MODEL]
%
%         node1,node2 :Node numbers corresponding to the source terminals
%                      Polarity conventions:
%                        Voltage source : Node 1 is the positive terminal.
%                        Current source : Current is flowing out of node 1. 
%                TYPE : Code indicating type of source
%                       0= voltage source
%                       1= current source
%
%                       The  next 4 parameters are used by  ETATFUN/NETSIM
%                 AMP : amplitude of the AC or DC voltage or current (V,A or pu)
%               PHASE : phase of the AC voltage or current (degres)
%                FREQ : Frequency (Hz) of the generated voltage or current.
%                       For a DC voltage or current source specify
%                       PHASE=0 and FREQ=0. AMP can be set to a negative value. 
%                       Generated signal:
%                                          AMP*sin(2*pi*FREQ*t+PHASE) for AC
%                                          AMP for DC
%               MODEL : integer specifying the type of non linear element
%                       modelled by the current source (saturable inductance,
%                       thyristor, switch ...) ;  Used by NETSIM only.
%
% 
%    Format of the SWITCHES input matrix
%    ===================================
%    Switches are non linear elements simulating mechanical or electronic
%    devices such as circuit breakers diodes or thyristors. As other non linear
%    elements, they are simulated by current sources driven by the voltage
%    appearing across their terminals. Therefore, they cannot have a perfectly
%    zero impedance. They are simulated as ideal switches in series with a
%    series R-L circuit.
%    The switch parameters must be specified  in a line of the SWITCHES matrix 
%    in 7 different columns according to the following format:
%
%                 (1)   (2)  (3)    (4)   (5)     (6)     (7)
%              node1  node2 STATUS   R    L/Xl    No_I    No_U
%
%        node1, node2 : Node numbers cooresponding to the switch terminals
%              STATUS : Code indicating the initial status of the switch at t=0
%                       0= open
%                       1= closed
%                  R  : Resistance of the switch when closed (ohms or pu)
%                L/Xl : Inductance of the switch when closed (mH)
%                       or inductive reactance (ohms or pu at FREQ_SYS)
%                       Note: For these last two fields, the same units as
%                             specified for the RLC matrix (UNIT) must be used.
%                             Nul inductance values arec not allowed. However,
%                             the resistance value can be set to zero.
%
%                       The  next 2 parameters are used by  ETATFUN/NETSIM
%                       They specify the current input number
%                       and the voltage output number to be used for 
%                       interconnecting the switch model to state-space block
%
%                no_I : Current input number coming from the output of
%                       the switch model.
%                no_U : Voltage output number driving the input of the 
%                       switch model.
%
%  Output arguments
%  ================
%                   
%   See also  ETATFUN

%   G. Sybille April 1995
%              Last Revision : June  25-96
%===============================================================================
%   Copyright (c) 1997 by TEQSIM Inc. and The MathWorks, Inc.
%   under sublicense from Hydro-Quebec.
%   $Revision: 1.2 $

nargs = nargin;
if nargs==6,
	z_option=1;
	li_option=-1;
	freq_sys=60;
	scsource=1;
	noderef=0;
	ibrv=0;
	iparv=[];
	fparv=0;
	power2sys_flag=1;
elseif  nargs==7,
	z_option=1;
	li_option=-1;
	freq_sys=60;
	scsource=1;
	noderef=0;
	ibrv=0;
	iparv=[];
	fparv=0;
elseif nargs==15,
else,
    error( sprintf...
	('Number of input arguments (%g) must be 6, 7 or 15',nargs));
end
%
%
if power2sys_flag<=1,
	% Indicate version #
	% ------------------
	version='4.5';
	fprintf('\nZCAL Version %s\n',version);
end
%
% Validation et pretraitement des matrices d'entree 'rlc' et 'source'
% ------------------------------------------------------------------
[rlc,node_list,ind_neu,ind_neusrc,ind_neusw, mat_tr,unit,rlccar]=etapar(...
    rlc,switches,source,999,noderef,unit,power2sys_flag,0);
%
%
%
rmin=1e-8; %Resistance simulant court-circuit pour sources de tension
j=sqrt(-1);
%
nbneu=length(node_list);
[nbr,n]=size(rlc);
nbr_mut=length(find(rlc(:,8)>0));
nbr_prop=nbr-nbr_mut;
nbr_ligne=length(find(rlc(:,3)<0));
[nbr_src,n]=size(source);
if nbr_src>0, nbr_srcU=length(find(source(:,3)==0)); else, nbr_srcU=0; end
[ntr,n]=size(mat_tr);
[nsw,n]=size(switches);
%
% Options automatiques; 
% --------------------
li_option_auto=li_option;
neu1_auto=node12(1);
neu2_auto=node12(2);
%
% Options pour modeles de ligne
% -----------------------------
if nbr_ligne>0,
	fprintf(['\n2 options sont disponibles pour les modeles de lignes:\n',...
			 '    1: chaque troncon de ligne represente par un pi\n'...
			 '    2: parametres distribues\n']);
	if li_option_auto>0,
		li_option=li_option_auto;
		fprintf('option choisie: %g\n',li_option);
	else,
		li_option=0;
	end
	while li_option~=1 & li_option~=2,
		li_option=input('option desiree (1 ou 2) :');
	end
	if li_option==2, li_option=0; end
end
%
if nbr_srcU>0,
	if power2sys_flag==0, 
		fprintf(['%g sources de tensions ont ete specifiees ',...
		'dans la matrice ''source''\n'],nbr_srcU);
	end
	if scsource==0,
		nbr_srcU=0;
		if power2sys_flag==0, 
		fprintf('Les sources de tension seront ouvertes\n'); end
	elseif scsource==1,
		if power2sys_flag==0, 
		fprintf('Les sources de tension seront court-circuitees\n'); end
	else,
		str=input(...
	'Desirez vous remplacer ces sources par des court-circuits ? (o/n) :',...
		's');
		if str(1)~='o', nbr_srcU=0; end
	end
end
%
Zbr=sparse(zeros(nbr_prop)); 
%
% Former matrices d'incidence des tensions et courants Ti et Tu telles que :
% ------------------------------------------------------------------------
%                     [Vbr]=[Tu].[Vneu]
%                     [Ineu]=[Ti].[Ibr]
%   ou    Tu = matrice d'incidence (nbr_prop lignes; nbneu colonnes)
%         Ti = matrice d'incidence (nbneu lignes; nbr_prop colonnes)
%        Vbr = matrice colonne des tensions de branche
%        Vneu= matrice colonne des tensions de noeuds
%        Ineu= matrice colonne des tensions de noeuds
%        Ibr = matrice colonne des tensions de branche
%
Tu=sparse(zeros(nbr_prop,nbneu));
for ib =1:nbr_prop,
	if ind_neu(ib,1)~=ind_neu(ib,2),
		Tu(ib,ind_neu(ib,1))=1;
		Tu(ib,ind_neu(ib,2))=-1;
	end
end
Ti=Tu';
%
%
% Former matrice d'impedance de branches zrlc et convertir Ohm mH uF en ohms
% ---------------------------------------------------------------------------
for ib=1:nbr,
	if all(unit=='OMU'),
		zrlc(ib,1)=rlc(ib,4);
		zrlc(ib,2)=rlc(ib,5)*0.002*pi*freq_sys;
		if rlc(ib,6)~=0.0,
			zrlc(ib,3)=1/(rlc(ib,6)*2e-6*pi*freq_sys);
		else
			zrlc(ib,3)=0;
		end
	else
		for i=1:3,
		zrlc(ib,i)=rlc(ib,i+3);
		end
	end
	if rlc(ib,3)<0,
		zrlc(ib,4)=-rlc(ib,3); % longueur de ligne
	else
		zrlc(ib,4)=0;
	end
end
%
if neu1_auto>=0 & neu2_auto>=0,
	neu1=neu1_auto; neu2=neu2_auto;
	if power2sys_flag==0, fprintf('Noeud 1=%g  Noeud 2=%g\n',neu1,neu2); end
else
	fprintf...
	('\nSpecifier les 2 noeuds entre lesquels l impedance doit etre evaluee\n');
	neu1=-9999; neu2=-9999;
end
%
while ~any(node_list==neu1),
	if neu1~=-9999 fprintf('Noeud no %g invalide\n',neu1); end
	neu1=input (' No du noeud 1:');
end
ineu1=1;
while node_list(ineu1)~=neu1,
	ineu1=ineu1+1;
end
%
while ~any(node_list==neu2),
	if neu2~=-9999 fprintf('Noeud no %g invalide\n',neu2); end
	neu2=input (' No du noeud 2:');
end
ineu2=1;
while node_list(ineu2)~=neu2,
	ineu2=ineu2+1;
end
%
node12(1)=neu1;
node12(2)=neu2;
%
if z_option==2,
	znom=zeros(1,4);
	for i=1:4,
		if iparv(i)==1, znom(i)=zrlc(ibrv,i); end
	end
	f=fparv;
end
%
for ii=1:length(valx),
	val=valx(ii);
	xx(ii)=val;
%
%   Construire la matrice Zbr telle que: [Vbr]=[Zbr].[Ibr]
%   ------------------------------------------------------
	for ib=1:nbr,
		%Traiter impedances de branches serie ou parallele, couplees ou non
		if rlc(ib,3)>=0,
			if z_option ==1,
				f=val;
			else
				if ib==ibrv,
					for ivar=1:3,
						if znom(ivar)~=0
							zrlc(ib,ivar)=znom(ivar)*val;
							if all(unit=='OMU') & ivar==3,
								zrlc(ib,3)=znom(3)/val;
							end
						end
					end
				end
			end
	%
			r=zrlc(ib,1);
			xl=zrlc(ib,2)*f/freq_sys;
			xc=-abs(zrlc(ib,3))*freq_sys/f;
	%
			if rlc(ib,3)==0,
	%       Branche serie
	%       -------------
				zbr=r+j*(xl+xc);
	%
			elseif rlc(ib,3)==1,
	%       Branche parallele
	%       -----------------
				zbr=0;
				yy=0;
				if r~=0,
					yy=yy+1/r;
				end
				if xl~=0,
					yy=yy+1/(0 + j*xl);
				end
				if  xc~=0,
					yy=yy+1/(0 + j*xc);
				end
				if abs(yy)~=0, zbr=1/yy; end
	%
			elseif rlc(ib,3)==2 | rlc(ib,3)==3,
	%       Resistance et Inductance de fuite transformateur ou mutuelle
	%       ------------------------------------------------------------
				zbr=r+j*xl;
			end
	%
			if  rlc(ib,8)>0,
				%Impedance mutuelle
				%-------------------
				ib1=rlc(ib,7);
				ib2=rlc(ib,8);
				Zbr(ib1,ib2)=zbr; 
				Zbr(ib2,ib1)=zbr; 
			else
				%Impedance propre
				%----------------
				Zbr(ib,ib)=zbr; 
			end
		end %fin if
	end %fin for
%
	% Taiter les lignes
	% -----------------
	if nbr_ligne>0,
		for ib=1:nbr,
			if rlc(ib,3)<0,
				if z_option ==1,
					f=val;
				else
					if ib==ibrv,
						for ivar=1:4,
							if znom(ivar)~=0
								zrlc(ib,ivar)=znom(ivar)*val;
								if all(unit=='OMU') & ivar==3,
									zrlc(ib,3)=znom(3)/val;
								end
							end    
						end
					end
				end
				r=zrlc(ib,1);
				% ligne representee par un pi equivalent
				% ------------------------------------
				if li_option, % ligne representee par un pi equivalent
					xl=zrlc(ib,2);       % reactance ind. lineique a  freq_sys
					xc=-abs(zrlc(ib,3)); % reactance cap. lineique a freq_sys
					[zbr,y_sh] =  etazline(zrlc(ib,4),r,xl,-1/xc);
					% calculer impedance serie et admittance du pi a freq. f
					zbr=real(zbr) +j*imag(zbr)*f/freq_sys;
					y_sh=real(y_sh) +j*imag(y_sh)*f/freq_sys;
				else, 
				% ligne a parametres distribues
				% -----------------------------
					xl=zrlc(ib,2)*f/freq_sys;
					xc=-abs(zrlc(ib,3))*freq_sys/f;
					[zbr,y_sh] =  etazline(zrlc(ib,4),r,xl,-1/xc);
				end
				%
				Zbr(ib,ib)=zbr; 
				% Ajouter y_sh a l'admitttance de chacune des branches shunt
				% des 2 extremites de ligne
				for icol=9:10,
					ib1=rlc(ib,icol);
					if abs(Zbr(ib1,ib1))~=0, yy=1/Zbr(ib1,ib1); else yy=0; end
					Zbr(ib1,ib1)=1.0/(yy+ y_sh);
				end
			end
		end
	end
	% Taiter les transformateurs
	% --------------------------
	for itr=1:ntr,
		ib=mat_tr(itr,1);    % no de branche 1er enroulement
		nwind=mat_tr(itr,2); % nombre d'enroulements        
		Zmag=Zbr(ib+nwind,ib+nwind);
		for iwind1=1:nwind
			ib1=ib+iwind1-1;
			% rap1 = Ntours enroulement iwind1 / Ntours enroulement #1
			rap1=mat_tr(itr,3*(iwind1+1))/mat_tr(itr,6);
			for iwind2=1:nwind
				ib2=ib+iwind2-1;
				% rap2 = Ntours enroulement iwind2 / Ntours enroulement #1
				rap2=mat_tr(itr,3*(iwind2+1))/mat_tr(itr,6);
				if iwind1==iwind2,
					if rlc(ib1,3)==2,
						% corriger impedance propre pour transformateur
						Zbr(ib1,ib1)=Zbr(ib1,ib1)+Zmag*rap1^2;
					end
				else,
					% impedance mutuelle
					Zbr(ib1,ib2)=Zmag*rap1*rap2;
				end
			end
		end
	end
%
%   Construire la matrice Ybus telle que: [Ineu]=[Ybus].[Vneu]
%   ----------------------------------------------------------
	Ybus=Ti*inv(Zbr)*Tu;
%
%	 Modifier Ybus pour tenir compte des sources de tension
%	 =======================================================
%	Remplacer les admittances entre noeuds i et j par court-circuit (y=1/rmin)
	if nbr_srcU>0,
		yy=1/rmin;
		for ib=1:nbr_src,
			if source(ib,3)==0, % source de tension
				i1=ind_neusrc(ib,1);
				i2=ind_neusrc(ib,2);
				Ybus(i1,i1)=Ybus(i1,i1)+yy;
				Ybus(i2,i2)=Ybus(i2,i2)+yy;
				Ybus(i1,i2)=Ybus(i1,i2)-yy;
			Ybus(i2,i1)=Ybus(i1,i2);  
			end
		end
	end
	%
	%
	% Modifier Ybus pour tenir compte des interrupteurs fermes
	% ========================================================
	for ib=1:nsw,
		if switches(ib,3)==1, % switch fermee
			r=switches(ib,4);
			if all(unit=='OMU'),
				xl=switches(ib,5)*0.002*pi*freq_sys;
			else
				xl=switches(ib,5);
			end
			yy=1/(r+j*xl);
			i1=ind_neusw(ib,1);
			i2=ind_neusw(ib,2);
			Ybus(i1,i1)=Ybus(i1,i1)+yy;
			Ybus(i2,i2)=Ybus(i2,i2)+yy;
			Ybus(i1,i2)=Ybus(i1,i2)-yy;
			Ybus(i2,i1)=Ybus(i1,i2);  
		end
	end
	%
	% Verifier reseaux isoles
	% =======================
	if ii==1,
		[nores,nbres]=etaisole(Ybus);
	%
		if nbres>1,
			fprintf('%g reseaux isoles ont ete detectes, soit:\n',nbres);
			for ires=1:nbres,
				fprintf('reseau %g:\n',ires);
				noeuds=node_list(find(nores==ires))
			end
			error(sprintf('Les %g reseaux ci-dessus sont isoles',nbres));
		end
		Ybus1=Ybus;
	end
	%
	% Construire  matrice Y1 non singuliere 
	% -------------------------------------
	% (derniere ligne et colonne de Ybus sont eliminees)
	dim1=nbneu-1;
	Y1=Ybus(1:dim1,1:dim1);
%
	Z=inv(Y1);
	if ineu1<=dim1 & ineu2<=dim1,
		zz(ii)=Z(ineu1,ineu1)+Z(ineu2,ineu2)-2*Z(ineu1,ineu2);
	else
		if ineu1<=dim1,
			zz(ii)=Z(ineu1,ineu1);
		elseif ineu2<=dim1,
			zz(ii)=Z(ineu2,ineu2);
		end
	end
	if power2sys_flag==0,
		str=sprintf('freq=%5gHz',f);
		if z_option==2,
			str=[str,sprintf(' k=%-5g ',val)];
		end
		str=[str,sprintf(' Z%g-%g = ',neu1,neu2)];
		str=[str,sprintf('%g %gi\n',real(zz(ii)),imag(zz(ii)) )];
		fprintf(str);
	end
end
